#!/usr/bin/perl
# -*- Mode: perl -*-
#======================================================================
# FILE: Parser.pm
# CREATOR: eric 02 September 1999
#
# DESCRIPTION:
#   
#
#  $Id: Parser.pm,v 1.4 2000/05/24 04:41:32 eric Exp $
#  $Locker:  $
#
# (C) COPYRIGHT 2000, Eric Busboom, http://www.softwarestudio.org
#
# This package is free software and is provided "as is" without express
# or implied warranty.  It may be used, redistributed and/or modified
# under the same terms as perl itself. ( Either the Artistic License or the
# GPL. ) 
#
# The Original Code is Parser.pm. The Initial Developer of the Original
# Code is Eric Busboom
#
#======================================================================

package Net::ICal::Parser;
use ICAL:Component;

sub parse_string
{

  $string_r = shift; 

  
  
}



1;

