# This file was automatically generated by SWIG
package Net__ICal;
require Exporter;
require DynaLoader;
@ISA = qw(Exporter DynaLoader);
package Net::ICal;
bootstrap Net__ICal;
var_Net__ICal_init();
@EXPORT = qw( );
1;
