/*
 * FILE : netical_wrap.c
 * 
 * This file was automatically generated by :
 * Simplified Wrapper and Interface Generator (SWIG)
 * Version 1.1 (Patch 5)
 * 
 * Portions Copyright (c) 1995-1998
 * The University of Utah and The Regents of the University of California.
 * Permission is granted to distribute this file in any manner provided
 * this notice remains intact.
 * 
 * Do not make changes to this file--changes will be lost!
 *
 */


#define SWIGCODE
/* Implementation : PERL 5 */

#define SWIGPERL
#define SWIGPERL5
#ifdef __cplusplus
#include <math.h>
#include <stdlib.h>
extern "C" {
#endif
#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"
#undef free
#undef malloc
#include <string.h>
#ifdef __cplusplus
}
#endif
/* Definitions for compiling Perl extensions on a variety of machines */

#if defined(WIN32) || defined(__WIN32__)
#   if defined(_MSC_VER)
#	define SWIGEXPORT(a,b) __declspec(dllexport) a b
#   else
#	if defined(__BORLANDC__)
#	    define SWIGEXPORT(a,b) a _export b
#	else
#	    define SWIGEXPORT(a,b) a b
#	endif
#   endif
#else
#   define SWIGEXPORT(a,b) a b
#endif

#ifdef PERL_OBJECT
#define MAGIC_PPERL  CPerl *pPerl = (CPerl *) this;
#define MAGIC_CAST   (int (CPerl::*)(SV *, MAGIC *))
#define SWIGCLASS_STATIC 
#else
#define MAGIC_PPERL
#define MAGIC_CAST
#define SWIGCLASS_STATIC static
#endif


/*****************************************************************************
 * $Header: /home/beazley/SWIG/SWIG1.1b3/swig_lib/perl5/RCS/perl5ptr.swg,v 1.1 1996/12/26 22:17:29 beazley Exp beazley $
 *
 * perl5ptr.swg
 *
 * This file contains supporting code for the SWIG run-time type checking
 * mechanism.  The following functions are available :
 *
 * SWIG_RegisterMapping(char *origtype, char *newtype, void *(*cast)(void *));
 *
 *      Registers a new type-mapping with the type-checker.  origtype is the
 *      original datatype and newtype is an equivalent type.  cast is optional
 *      pointer to a function to cast pointer values between types (this
 *      is only used to cast pointers from derived classes to base classes in C++)
 *      
 * SWIG_MakePtr(char *buffer, void *ptr, char *typestring);
 *     
 *      Makes a pointer string from a pointer and typestring.  The result is returned
 *      in buffer.
 *
 * char * SWIG_GetPtr(SV *obj, void **ptr, char *type)
 *
 *      Gets a pointer value from a Perl5 scalar value.  If there is a 
 *      type-mismatch, returns a character string to the received type.  
 *      On success, returns NULL.
 *
 *
 * You can remap these functions by making a file called "swigptr.swg" in
 * your the same directory as the interface file you are wrapping.
 *
 * These functions are normally declared static, but this file can be
 * can be used in a multi-module environment by redefining the symbol
 * SWIGSTATIC.
 *
 * $Log: perl5ptr.swg,v $
 * Revision 1.1  1996/12/26 22:17:29  beazley
 * Initial revision
 *
 *****************************************************************************/

#include <stdlib.h>

#ifdef SWIG_GLOBAL
#ifdef __cplusplus
#define SWIGSTATIC extern "C"
#else
#define SWIGSTATIC
#endif
#endif

#ifndef SWIGSTATIC
#define SWIGSTATIC static
#endif

/* These are internal variables.   Should be static */

typedef struct SwigPtrType {
  char               *name;
  int                 len;
  void               *(*cast)(void *);
  struct SwigPtrType *next;
} SwigPtrType;

/* Pointer cache structure */

typedef struct {
  int                 stat;               /* Status (valid) bit             */
  SwigPtrType        *tp;                 /* Pointer to type structure      */
  char                name[256];          /* Given datatype name            */
  char                mapped[256];        /* Equivalent name                */
} SwigCacheType;

static int SwigPtrMax  = 64;           /* Max entries that can be currently held */
static int SwigPtrN    = 0;            /* Current number of entries              */
static int SwigPtrSort = 0;            /* Status flag indicating sort            */
static SwigPtrType *SwigPtrTable = 0;  /* Table containing pointer equivalences  */
static int SwigStart[256];             /* Table containing starting positions    */

/* Cached values */

#define SWIG_CACHESIZE  8
#define SWIG_CACHEMASK  0x7
static SwigCacheType SwigCache[SWIG_CACHESIZE];  
static int SwigCacheIndex = 0;
static int SwigLastCache = 0;

/* Sort comparison function */
static int swigsort(const void *data1, const void *data2) {
	SwigPtrType *d1 = (SwigPtrType *) data1;
	SwigPtrType *d2 = (SwigPtrType *) data2;
	return strcmp(d1->name,d2->name);
}

/* Binary Search function */
static int swigcmp(const void *key, const void *data) {
  char *k = (char *) key;
  SwigPtrType *d = (SwigPtrType *) data;
  return strncmp(k,d->name,d->len);
}

/* Register a new datatype with the type-checker */

#ifndef PERL_OBJECT
SWIGSTATIC 
void SWIG_RegisterMapping(char *origtype, char *newtype, void *(*cast)(void *)) {
#else
SWIGSTATIC
#define SWIG_RegisterMapping(a,b,c) _SWIG_RegisterMapping(pPerl, a,b,c)
void _SWIG_RegisterMapping(CPerl *pPerl, char *origtype, char *newtype, void *(*cast)(void *)) {
#endif

  int i;
  SwigPtrType *t = 0, *t1;

  if (!SwigPtrTable) {     
    SwigPtrTable = (SwigPtrType *) malloc(SwigPtrMax*sizeof(SwigPtrType));
    SwigPtrN = 0;
  }
  if (SwigPtrN >= SwigPtrMax) {
    SwigPtrMax = 2*SwigPtrMax;
    SwigPtrTable = (SwigPtrType *) realloc(SwigPtrTable,SwigPtrMax*sizeof(SwigPtrType));
  }
  for (i = 0; i < SwigPtrN; i++)
    if (strcmp(SwigPtrTable[i].name,origtype) == 0) {
      t = &SwigPtrTable[i];
      break;
    }
  if (!t) {
    t = &SwigPtrTable[SwigPtrN];
    t->name = origtype;
    t->len = strlen(t->name);
    t->cast = 0;
    t->next = 0;
    SwigPtrN++;
  }
  while (t->next) {
    if (strcmp(t->name,newtype) == 0) {
      if (cast) t->cast = cast;
      return;
    }
    t = t->next;
  }
  t1 = (SwigPtrType *) malloc(sizeof(SwigPtrType));
  t1->name = newtype;
  t1->len = strlen(t1->name);
  t1->cast = cast;
  t1->next = 0;
  t->next = t1;
  SwigPtrSort = 0;
}

/* Make a pointer value string */

SWIGSTATIC 
void SWIG_MakePtr(char *_c, const void *_ptr, char *type) {
  static char _hex[16] =
  {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
   'a', 'b', 'c', 'd', 'e', 'f'};
  unsigned long _p, _s;
  char _result[20], *_r;    /* Note : a 64-bit hex number = 16 digits */
  _r = _result;
  _p = (unsigned long) _ptr;
  if (_p > 0) {
    while (_p > 0) {
      _s = _p & 0xf;
      *(_r++) = _hex[_s];
      _p = _p >> 4;
    }
    *_r = '_';
    while (_r >= _result)
      *(_c++) = *(_r--);
  } else {
    strcpy (_c, "NULL");
  }
  if (_ptr)
    strcpy (_c, type);
}

/* Define for backwards compatibility */

#define _swig_make_hex   SWIG_MakePtr 

/* Function for getting a pointer value */

#ifndef PERL_OBJECT
SWIGSTATIC 
char *SWIG_GetPtr(SV *sv, void **ptr, char *_t)
#else
SWIGSTATIC
#define SWIG_GetPtr(a,b,c) _SWIG_GetPtr(pPerl,a,b,c)
char *_SWIG_GetPtr(CPerl *pPerl, SV *sv, void **ptr, char *_t)
#endif
{
  char temp_type[256];
  char *name,*_c;
  int  len,i,start,end;
  IV   tmp;
  SwigPtrType *sp,*tp;
  SwigCacheType *cache;

  /* If magical, apply more magic */

  if (SvGMAGICAL(sv))
    mg_get(sv);

  /* Check to see if this is an object */
  if (sv_isobject(sv)) {
    SV *tsv = (SV*) SvRV(sv);
    if ((SvTYPE(tsv) == SVt_PVHV)) {
      MAGIC *mg;
      if (SvMAGICAL(tsv)) {
	mg = mg_find(tsv,'P');
	if (mg) {
	  SV *rsv = mg->mg_obj;
	  if (sv_isobject(rsv)) {
	    tmp = SvIV((SV*)SvRV(rsv));
	  }
	}
      } else {
	return "Not a valid pointer value";
      }
    } else {
      tmp = SvIV((SV*)SvRV(sv));
    }
    if (!_t) {
      *(ptr) = (void *) tmp;
      return (char *) 0;
    }
  } else if (sv == &sv_undef) {            /* Check for undef */
    *(ptr) = (void *) 0;
    return (char *) 0;
  } else if (SvTYPE(sv) == SVt_RV) {       /* Check for NULL pointer */
    *(ptr) = (void *) 0;
    if (!SvROK(sv)) 
      return (char *) 0;
    else
      return "Not a valid pointer value";
  } else {                                 /* Don't know what it is */
      *(ptr) = (void *) 0;
      return "Not a valid pointer value";
  }
  if (_t) {
    /* Now see if the types match */      

    if (!sv_isa(sv,_t)) {
      _c = HvNAME(SvSTASH(SvRV(sv)));
      if (!SwigPtrSort) {
	qsort((void *) SwigPtrTable, SwigPtrN, sizeof(SwigPtrType), swigsort);  
	for (i = 0; i < 256; i++) {
	  SwigStart[i] = SwigPtrN;
	}
	for (i = SwigPtrN-1; i >= 0; i--) {
	  SwigStart[SwigPtrTable[i].name[0]] = i;
	}
	for (i = 255; i >= 1; i--) {
	  if (SwigStart[i-1] > SwigStart[i])
	    SwigStart[i-1] = SwigStart[i];
	}
	SwigPtrSort = 1;
	for (i = 0; i < SWIG_CACHESIZE; i++)  
	  SwigCache[i].stat = 0;
      }
      /* First check cache for matches.  Uses last cache value as starting point */
      cache = &SwigCache[SwigLastCache];
      for (i = 0; i < SWIG_CACHESIZE; i++) {
	if (cache->stat) {
	  if (strcmp(_t,cache->name) == 0) {
	    if (strcmp(_c,cache->mapped) == 0) {
	      cache->stat++;
	      *ptr = (void *) tmp;
	      if (cache->tp->cast) *ptr = (*(cache->tp->cast))(*ptr);
	      return (char *) 0;
	    }
	  }
	}
	SwigLastCache = (SwigLastCache+1) & SWIG_CACHEMASK;
	if (!SwigLastCache) cache = SwigCache;
	else cache++;
      }

      start = SwigStart[_t[0]];
      end = SwigStart[_t[0]+1];
      sp = &SwigPtrTable[start];
      while (start < end) {
	if (swigcmp(_t,sp) == 0) break;
	sp++;
	start++;
      }
      if (start >= end) sp = 0;
      if (sp) {
	while (swigcmp(_t,sp) == 0) {
	  name = sp->name;
	  len = sp->len;
	  tp = sp->next;
	  while(tp) {
	    if (tp->len >= 255) {
	      return _c;
	    }
	    strcpy(temp_type,tp->name);
	    strncat(temp_type,_t+len,255-tp->len);
	    if (sv_isa(sv,temp_type)) {
	      /* Get pointer value */
	      *ptr = (void *) tmp;
	      if (tp->cast) *ptr = (*(tp->cast))(*ptr);

	      strcpy(SwigCache[SwigCacheIndex].mapped,_c);
	      strcpy(SwigCache[SwigCacheIndex].name,_t);
	      SwigCache[SwigCacheIndex].stat = 1;
	      SwigCache[SwigCacheIndex].tp = tp;
	      SwigCacheIndex = SwigCacheIndex & SWIG_CACHEMASK;
	      return (char *) 0;
	    }
	    tp = tp->next;
	  } 
	  /* Hmmm. Didn't find it this time */
 	  sp++;
	}
      }
      /* Didn't find any sort of match for this data.  
	 Get the pointer value and return the received type */
      *ptr = (void *) tmp;
      return _c;
    } else {
      /* Found a match on the first try.  Return pointer value */
      *ptr = (void *) tmp;
      return (char *) 0;
    }
  } 
  *ptr = (void *) tmp;
  return (char *) 0;
}

/* Compatibility mode */

#define _swig_get_hex  SWIG_GetPtr
/* Magic variable code */
#ifndef PERL_OBJECT
#define swig_create_magic(s,a,b,c) _swig_create_magic(s,a,b,c)
static void _swig_create_magic(SV *sv, char *name, int (*set)(SV *, MAGIC *), int (*get)(SV *,MAGIC *)) {
#else
#define swig_create_magic(s,a,b,c) _swig_create_magic(pPerl,s,a,b,c)
static void _swig_create_magic(CPerl *pPerl, SV *sv, char *name, int (CPerl::*set)(SV *, MAGIC *), int (CPerl::*get)(SV *, MAGIC *)) {
#endif
  MAGIC *mg;
  sv_magic(sv,sv,'U',name,strlen(name));
  mg = mg_find(sv,'U');
  mg->mg_virtual = (MGVTBL *) malloc(sizeof(MGVTBL));
  mg->mg_virtual->svt_get = get;
  mg->mg_virtual->svt_set = set;
  mg->mg_virtual->svt_len = 0;
  mg->mg_virtual->svt_clear = 0;
  mg->mg_virtual->svt_free = 0;
}

#define SWIG_init    boot_Net__ICal

#define SWIG_name   "Net::ICal::boot_Net__ICal"
#define SWIG_varinit "Net::ICal::var_Net__ICal_init();"
#ifdef __cplusplus
extern "C"
#endif
#ifndef PERL_OBJECT
SWIGEXPORT(void,boot_Net__ICal)(CV* cv);
#else
SWIGEXPORT(void,boot_Net__ICal)(CPerl *, CV *cv);
#endif

#include "ical.h"

#include <sys/types.h> /* for size_t */
#include <time.h>

#ifndef PERL_OBJECT
#define swig_setiv(a,b) _swig_setiv(a,b)
static void _swig_setiv(char *name, long value) { 
#else
#define swig_setiv(a,b) _swig_setiv(pPerl,a,b)
static void _swig_setiv(CPerl *pPerl, char *name, long value) { 
#endif
     SV *sv; 
     sv = perl_get_sv(name,TRUE | 0x2);
     sv_setiv(sv, (IV) value);
     SvREADONLY_on(sv);
}

#ifndef PERL_OBJECT
#define swig_setpv(a,b) _swig_setpv(a,b)
static void _swig_setpv(char *name, char *value) { 
#else
#define swig_setpv(a,b) _swig_setpv(pPerl,a,b)
static void _swig_setpv(CPerl *pPerl, char *name, char *value) { 
#endif
     SV *sv; 
     sv = perl_get_sv(name,TRUE | 0x2);
     sv_setpv(sv, value);
     SvREADONLY_on(sv);
}

#ifdef PERL_OBJECT
#define MAGIC_CLASS _wrap_Net__ICal_var::
class _wrap_Net__ICal_var : public CPerl {
public:
#else
#define MAGIC_CLASS
#endif
SWIGCLASS_STATIC int swig_magic_readonly(SV *sv, MAGIC *mg) {
    MAGIC_PPERL
    sv = sv; mg = mg;
    croak("Value is read-only.");
    return 0;
}


#ifdef PERL_OBJECT
};
#endif

XS(_wrap_icaltime_null_time) {

    struct icaltimetype * _result;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 0) || (items > 0)) 
        croak("Usage: icaltime_null_time();");
    _result = (struct icaltimetype *) malloc(sizeof(struct icaltimetype ));
    *(_result) = icaltime_null_time();
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icaltime_is_null_time) {

    int  _result;
    struct icaltimetype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaltime_is_null_time(t);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icaltime_is_null_time. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (int )icaltime_is_null_time(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icaltime_normalize) {

    struct icaltimetype * _result;
    struct icaltimetype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaltime_normalize(t);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icaltime_normalize. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (struct icaltimetype *) malloc(sizeof(struct icaltimetype ));
    *(_result) = icaltime_normalize(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icaltime_day_of_year) {

    short  _result;
    struct icaltimetype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaltime_day_of_year(t);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icaltime_day_of_year. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (short )icaltime_day_of_year(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icaltime_from_day_of_year) {

    struct icaltimetype * _result;
    short  _arg0;
    short  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icaltime_from_day_of_year(doy,year);");
    _arg0 = (short )SvIV(ST(0));
    _arg1 = (short )SvIV(ST(1));
    _result = (struct icaltimetype *) malloc(sizeof(struct icaltimetype ));
    *(_result) = icaltime_from_day_of_year(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icaltime_day_of_week) {

    short  _result;
    struct icaltimetype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaltime_day_of_week(t);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icaltime_day_of_week. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (short )icaltime_day_of_week(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icaltime_start_doy_of_week) {

    short  _result;
    struct icaltimetype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaltime_start_doy_of_week(t);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icaltime_start_doy_of_week. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (short )icaltime_start_doy_of_week(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icaltime_from_timet) {

    struct icaltimetype * _result;
    time_t * _arg0;
    int  _arg1;
    int  _arg2;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 3) || (items > 3)) 
        croak("Usage: icaltime_from_timet(v,is_date,is_utc);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"time_tPtr")) {
        croak("Type error in argument 1 of icaltime_from_timet. Expected time_tPtr.");
        XSRETURN(1);
    }
    _arg1 = (int )SvIV(ST(1));
    _arg2 = (int )SvIV(ST(2));
    _result = (struct icaltimetype *) malloc(sizeof(struct icaltimetype ));
    *(_result) = icaltime_from_timet(*_arg0,_arg1,_arg2);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icaltime_from_string) {

    struct icaltimetype * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaltime_from_string(str);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (struct icaltimetype *) malloc(sizeof(struct icaltimetype ));
    *(_result) = icaltime_from_string(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icaltime_as_timet) {

    time_t * _result;
    struct icaltimetype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaltime_as_timet(struct icaltimetype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icaltime_as_timet. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (time_t *) malloc(sizeof(time_t ));
    *(_result) = icaltime_as_timet(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"time_tPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icaltime_as_ctime) {

    char * _result;
    struct icaltimetype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaltime_as_ctime(struct icaltimetype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icaltime_as_ctime. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (char *)icaltime_as_ctime(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icaltime_week_number) {

    short  _result;
    short  _arg0;
    short  _arg1;
    short  _arg2;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 3) || (items > 3)) 
        croak("Usage: icaltime_week_number(day_of_month,month,year);");
    _arg0 = (short )SvIV(ST(0));
    _arg1 = (short )SvIV(ST(1));
    _arg2 = (short )SvIV(ST(2));
    _result = (short )icaltime_week_number(_arg0,_arg1,_arg2);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icaltime_from_week_number) {

    struct icaltimetype * _result;
    short  _arg0;
    short  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icaltime_from_week_number(week_number,year);");
    _arg0 = (short )SvIV(ST(0));
    _arg1 = (short )SvIV(ST(1));
    _result = (struct icaltimetype *) malloc(sizeof(struct icaltimetype ));
    *(_result) = icaltime_from_week_number(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icaltime_compare) {

    int  _result;
    struct icaltimetype * _arg0;
    struct icaltimetype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icaltime_compare(a,b);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icaltime_compare. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icaltimetypePtr")) {
        croak("Type error in argument 2 of icaltime_compare. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (int )icaltime_compare(*_arg0,*_arg1);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icaltime_days_in_month) {

    short  _result;
    short  _arg0;
    short  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icaltime_days_in_month(month,year);");
    _arg0 = (short )SvIV(ST(0));
    _arg1 = (short )SvIV(ST(1));
    _result = (short )icaltime_days_in_month(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icaltime_utc_offset) {

    time_t * _result;
    struct icaltimetype * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icaltime_utc_offset(tt,tzid);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icaltime_utc_offset. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    _result = (time_t *) malloc(sizeof(time_t ));
    *(_result) = icaltime_utc_offset(*_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"time_tPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icaltime_local_utc_offset) {

    time_t * _result;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 0) || (items > 0)) 
        croak("Usage: icaltime_local_utc_offset();");
    _result = (time_t *) malloc(sizeof(time_t ));
    *(_result) = icaltime_local_utc_offset();
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"time_tPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icaltime_as_utc) {

    struct icaltimetype * _result;
    struct icaltimetype * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icaltime_as_utc(tt,tzid);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icaltime_as_utc. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    _result = (struct icaltimetype *) malloc(sizeof(struct icaltimetype ));
    *(_result) = icaltime_as_utc(*_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icaltime_as_zone) {

    struct icaltimetype * _result;
    struct icaltimetype * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icaltime_as_zone(tt,tzid);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icaltime_as_zone. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    _result = (struct icaltimetype *) malloc(sizeof(struct icaltimetype ));
    *(_result) = icaltime_as_zone(*_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icaldurationtype_from_timet) {

    struct icaldurationtype * _result;
    time_t * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaldurationtype_from_timet(t);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"time_tPtr")) {
        croak("Type error in argument 1 of icaldurationtype_from_timet. Expected time_tPtr.");
        XSRETURN(1);
    }
    _result = (struct icaldurationtype *) malloc(sizeof(struct icaldurationtype ));
    *(_result) = icaldurationtype_from_timet(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaldurationtypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icaldurationtype_from_string) {

    struct icaldurationtype * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaldurationtype_from_string(char *);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (struct icaldurationtype *) malloc(sizeof(struct icaldurationtype ));
    *(_result) = icaldurationtype_from_string(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaldurationtypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icaldurationtype_as_timet) {

    time_t * _result;
    struct icaldurationtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaldurationtype_as_timet(duration);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaldurationtypePtr")) {
        croak("Type error in argument 1 of icaldurationtype_as_timet. Expected struct icaldurationtypePtr.");
        XSRETURN(1);
    }
    _result = (time_t *) malloc(sizeof(time_t ));
    *(_result) = icaldurationtype_as_timet(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"time_tPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalperiodtype_duration) {

    time_t * _result;
    struct icalperiodtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalperiodtype_duration(period);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalperiodtypePtr")) {
        croak("Type error in argument 1 of icalperiodtype_duration. Expected struct icalperiodtypePtr.");
        XSRETURN(1);
    }
    _result = (time_t *) malloc(sizeof(time_t ));
    *(_result) = icalperiodtype_duration(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"time_tPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalperiodtype_end) {

    time_t * _result;
    struct icalperiodtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalperiodtype_end(period);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalperiodtypePtr")) {
        croak("Type error in argument 1 of icalperiodtype_end. Expected struct icalperiodtypePtr.");
        XSRETURN(1);
    }
    _result = (time_t *) malloc(sizeof(time_t ));
    *(_result) = icalperiodtype_end(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"time_tPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icaltime_add) {

    struct icaltimetype * _result;
    struct icaltimetype * _arg0;
    struct icaldurationtype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icaltime_add(t,d);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icaltime_add. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icaldurationtypePtr")) {
        croak("Type error in argument 2 of icaltime_add. Expected struct icaldurationtypePtr.");
        XSRETURN(1);
    }
    _result = (struct icaltimetype *) malloc(sizeof(struct icaltimetype ));
    *(_result) = icaltime_add(*_arg0,*_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icaltime_subtract) {

    struct icaldurationtype * _result;
    struct icaltimetype * _arg0;
    struct icaltimetype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icaltime_subtract(t1,t2);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icaltime_subtract. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icaltimetypePtr")) {
        croak("Type error in argument 2 of icaltime_subtract. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (struct icaldurationtype *) malloc(sizeof(struct icaldurationtype ));
    *(_result) = icaltime_subtract(*_arg0,*_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaldurationtypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalenum_reqstat_desc) {

    char * _result;
    icalrequeststatus  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalenum_reqstat_desc(stat);");
    _arg0 = (icalrequeststatus )SvIV(ST(0));
    _result = (char *)icalenum_reqstat_desc(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalenum_reqstat_major) {

    short  _result;
    icalrequeststatus  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalenum_reqstat_major(stat);");
    _arg0 = (icalrequeststatus )SvIV(ST(0));
    _result = (short )icalenum_reqstat_major(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalenum_reqstat_minor) {

    short  _result;
    icalrequeststatus  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalenum_reqstat_minor(stat);");
    _arg0 = (icalrequeststatus )SvIV(ST(0));
    _result = (short )icalenum_reqstat_minor(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalenum_num_to_reqstat) {

    icalrequeststatus  _result;
    short  _arg0;
    short  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalenum_num_to_reqstat(major,minor);");
    _arg0 = (short )SvIV(ST(0));
    _arg1 = (short )SvIV(ST(1));
    _result = (icalrequeststatus )icalenum_num_to_reqstat(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalenum_property_kind_to_string) {

    char * _result;
    icalproperty_kind  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalenum_property_kind_to_string(kind);");
    _arg0 = (icalproperty_kind )SvIV(ST(0));
    _result = (char *)icalenum_property_kind_to_string(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalenum_string_to_property_kind) {

    icalproperty_kind  _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalenum_string_to_property_kind(string);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty_kind )icalenum_string_to_property_kind(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalenum_value_kind_to_string) {

    char * _result;
    icalvalue_kind  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalenum_value_kind_to_string(kind);");
    _arg0 = (icalvalue_kind )SvIV(ST(0));
    _result = (char *)icalenum_value_kind_to_string(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalenum_value_kind_by_prop) {

    icalvalue_kind  _result;
    icalproperty_kind  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalenum_value_kind_by_prop(kind);");
    _arg0 = (icalproperty_kind )SvIV(ST(0));
    _result = (icalvalue_kind )icalenum_value_kind_by_prop(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalenum_parameter_kind_to_string) {

    char * _result;
    icalparameter_kind  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalenum_parameter_kind_to_string(kind);");
    _arg0 = (icalparameter_kind )SvIV(ST(0));
    _result = (char *)icalenum_parameter_kind_to_string(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalenum_string_to_parameter_kind) {

    icalparameter_kind  _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalenum_string_to_parameter_kind(string);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalparameter_kind )icalenum_string_to_parameter_kind(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalenum_component_kind_to_string) {

    char * _result;
    icalcomponent_kind  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalenum_component_kind_to_string(kind);");
    _arg0 = (icalcomponent_kind )SvIV(ST(0));
    _result = (char *)icalenum_component_kind_to_string(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalenum_string_to_component_kind) {

    icalcomponent_kind  _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalenum_string_to_component_kind(string);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalcomponent_kind )icalenum_string_to_component_kind(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalenum_property_kind_to_value_kind) {

    icalvalue_kind  _result;
    icalproperty_kind  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalenum_property_kind_to_value_kind(kind);");
    _arg0 = (icalproperty_kind )SvIV(ST(0));
    _result = (icalvalue_kind )icalenum_property_kind_to_value_kind(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalenum_method_to_string) {

    char * _result;
    icalproperty_method  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalenum_method_to_string(icalproperty_method );");
    _arg0 = (icalproperty_method )SvIV(ST(0));
    _result = (char *)icalenum_method_to_string(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalenum_string_to_method) {

    icalproperty_method  _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalenum_string_to_method(string);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty_method )icalenum_string_to_method(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalenum_status_to_string) {

    char * _result;
    icalproperty_status  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalenum_status_to_string(icalproperty_status );");
    _arg0 = (icalproperty_status )SvIV(ST(0));
    _result = (char *)icalenum_status_to_string(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalenum_string_to_status) {

    icalproperty_status  _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalenum_string_to_status(string);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty_status )icalenum_string_to_status(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalattachtype_new) {

    struct icalattachtype * _result;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 0) || (items > 0)) 
        croak("Usage: icalattachtype_new();");
    _result = (struct icalattachtype *)icalattachtype_new();
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icalattachtypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalattachtype_add_reference) {

    struct icalattachtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalattachtype_add_reference(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalattachtypePtr")) {
        croak("Type error in argument 1 of icalattachtype_add_reference. Expected struct icalattachtypePtr.");
        XSRETURN(1);
    }
    icalattachtype_add_reference(_arg0);
    XSRETURN(argvi);
}

XS(_wrap_icalattachtype_free) {

    struct icalattachtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalattachtype_free(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalattachtypePtr")) {
        croak("Type error in argument 1 of icalattachtype_free. Expected struct icalattachtypePtr.");
        XSRETURN(1);
    }
    icalattachtype_free(_arg0);
    XSRETURN(argvi);
}

XS(_wrap_icalattachtype_set_url) {

    struct icalattachtype * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalattachtype_set_url(v,url);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalattachtypePtr")) {
        croak("Type error in argument 1 of icalattachtype_set_url. Expected struct icalattachtypePtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalattachtype_set_url(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalattachtype_get_url) {

    char * _result;
    struct icalattachtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalattachtype_get_url(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalattachtypePtr")) {
        croak("Type error in argument 1 of icalattachtype_get_url. Expected struct icalattachtypePtr.");
        XSRETURN(1);
    }
    _result = (char *)icalattachtype_get_url(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalattachtype_set_base64) {

    struct icalattachtype * _arg0;
    char * _arg1;
    int  _arg2;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 3) || (items > 3)) 
        croak("Usage: icalattachtype_set_base64(v,base64,owns);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalattachtypePtr")) {
        croak("Type error in argument 1 of icalattachtype_set_base64. Expected struct icalattachtypePtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    _arg2 = (int )SvIV(ST(2));
    icalattachtype_set_base64(_arg0,_arg1,_arg2);
    XSRETURN(argvi);
}

XS(_wrap_icalattachtype_get_base64) {

    char * _result;
    struct icalattachtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalattachtype_get_base64(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalattachtypePtr")) {
        croak("Type error in argument 1 of icalattachtype_get_base64. Expected struct icalattachtypePtr.");
        XSRETURN(1);
    }
    _result = (char *)icalattachtype_get_base64(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalattachtype_set_binary) {

    struct icalattachtype * _arg0;
    char * _arg1;
    int  _arg2;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 3) || (items > 3)) 
        croak("Usage: icalattachtype_set_binary(v,binary,owns);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalattachtypePtr")) {
        croak("Type error in argument 1 of icalattachtype_set_binary. Expected struct icalattachtypePtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    _arg2 = (int )SvIV(ST(2));
    icalattachtype_set_binary(_arg0,_arg1,_arg2);
    XSRETURN(argvi);
}

XS(_wrap_icalattachtype_get_binary) {

    void * _result;
    struct icalattachtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalattachtype_get_binary(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalattachtypePtr")) {
        croak("Type error in argument 1 of icalattachtype_get_binary. Expected struct icalattachtypePtr.");
        XSRETURN(1);
    }
    _result = (void *)icalattachtype_get_binary(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"voidPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalreqstattype_from_string) {

    struct icalreqstattype * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalreqstattype_from_string(str);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (struct icalreqstattype *) malloc(sizeof(struct icalreqstattype ));
    *(_result) = icalreqstattype_from_string(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icalreqstattypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalreqstattype_as_string) {

    char * _result;
    struct icalreqstattype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalreqstattype_as_string(struct icalreqstattype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalreqstattypePtr")) {
        croak("Type error in argument 1 of icalreqstattype_as_string. Expected struct icalreqstattypePtr.");
        XSRETURN(1);
    }
    _result = (char *)icalreqstattype_as_string(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_new) {

    icalvalue * _result;
    icalvalue_kind  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_new(kind);");
    _arg0 = (icalvalue_kind )SvIV(ST(0));
    _result = (icalvalue *)icalvalue_new(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalvaluePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_new_clone) {

    icalvalue * _result;
    icalvalue * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_new_clone(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_new_clone. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _result = (icalvalue *)icalvalue_new_clone(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalvaluePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_new_from_string) {

    icalvalue * _result;
    icalvalue_kind  _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalvalue_new_from_string(kind,str);");
    _arg0 = (icalvalue_kind )SvIV(ST(0));
    _arg1 = (char *) SvPV(ST(1),na);
    _result = (icalvalue *)icalvalue_new_from_string(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalvaluePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_free) {

    icalvalue * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_free(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_free. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    icalvalue_free(_arg0);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_is_valid) {

    int  _result;
    icalvalue * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_is_valid(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_is_valid. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _result = (int )icalvalue_is_valid(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_as_ical_string) {

    char * _result;
    icalvalue * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_as_ical_string(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_as_ical_string. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _result = (char *)icalvalue_as_ical_string(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_isa) {

    icalvalue_kind  _result;
    icalvalue * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_isa(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_isa. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _result = (icalvalue_kind )icalvalue_isa(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_isa_value) {

    int  _result;
    void * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_isa_value(void *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_isa_value. Expected voidPtr.");
        XSRETURN(1);
    }
    _result = (int )icalvalue_isa_value(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_compare) {

    icalparameter_xliccomparetype  _result;
    icalvalue * _arg0;
    icalvalue * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalvalue_compare(a,b);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_compare. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,(char *) 0 )) {
        croak("Type error in argument 2 of icalvalue_compare. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _result = (icalparameter_xliccomparetype )icalvalue_compare(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_new_attach) {

    icalvalue * _result;
    struct icalattachtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_new_attach(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalattachtypePtr")) {
        croak("Type error in argument 1 of icalvalue_new_attach. Expected struct icalattachtypePtr.");
        XSRETURN(1);
    }
    _result = (icalvalue *)icalvalue_new_attach(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalvaluePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_get_attach) {

    struct icalattachtype * _result;
    icalvalue * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_get_attach(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_get_attach. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _result = (struct icalattachtype *) malloc(sizeof(struct icalattachtype ));
    *(_result) = icalvalue_get_attach(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icalattachtypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_set_attach) {

    icalvalue * _arg0;
    struct icalattachtype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalvalue_set_attach(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_set_attach. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icalattachtypePtr")) {
        croak("Type error in argument 2 of icalvalue_set_attach. Expected struct icalattachtypePtr.");
        XSRETURN(1);
    }
    icalvalue_set_attach(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_new_binary) {

    icalvalue * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_new_binary(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalvalue *)icalvalue_new_binary(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalvaluePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_get_binary) {

    char * _result;
    icalvalue * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_get_binary(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_get_binary. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _result = (char *)icalvalue_get_binary(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_set_binary) {

    icalvalue * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalvalue_set_binary(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_set_binary. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalvalue_set_binary(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_new_boolean) {

    icalvalue * _result;
    int  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_new_boolean(v);");
    _arg0 = (int )SvIV(ST(0));
    _result = (icalvalue *)icalvalue_new_boolean(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalvaluePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_get_boolean) {

    int  _result;
    icalvalue * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_get_boolean(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_get_boolean. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _result = (int )icalvalue_get_boolean(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_set_boolean) {

    icalvalue * _arg0;
    int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalvalue_set_boolean(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_set_boolean. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _arg1 = (int )SvIV(ST(1));
    icalvalue_set_boolean(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_new_caladdress) {

    icalvalue * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_new_caladdress(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalvalue *)icalvalue_new_caladdress(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalvaluePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_get_caladdress) {

    char * _result;
    icalvalue * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_get_caladdress(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_get_caladdress. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _result = (char *)icalvalue_get_caladdress(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_set_caladdress) {

    icalvalue * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalvalue_set_caladdress(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_set_caladdress. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalvalue_set_caladdress(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_new_date) {

    icalvalue * _result;
    struct icaltimetype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_new_date(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icalvalue_new_date. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (icalvalue *)icalvalue_new_date(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalvaluePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_get_date) {

    struct icaltimetype * _result;
    icalvalue * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_get_date(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_get_date. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _result = (struct icaltimetype *) malloc(sizeof(struct icaltimetype ));
    *(_result) = icalvalue_get_date(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_set_date) {

    icalvalue * _arg0;
    struct icaltimetype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalvalue_set_date(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_set_date. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icaltimetypePtr")) {
        croak("Type error in argument 2 of icalvalue_set_date. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    icalvalue_set_date(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_new_datetime) {

    icalvalue * _result;
    struct icaltimetype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_new_datetime(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icalvalue_new_datetime. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (icalvalue *)icalvalue_new_datetime(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalvaluePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_get_datetime) {

    struct icaltimetype * _result;
    icalvalue * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_get_datetime(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_get_datetime. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _result = (struct icaltimetype *) malloc(sizeof(struct icaltimetype ));
    *(_result) = icalvalue_get_datetime(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_set_datetime) {

    icalvalue * _arg0;
    struct icaltimetype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalvalue_set_datetime(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_set_datetime. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icaltimetypePtr")) {
        croak("Type error in argument 2 of icalvalue_set_datetime. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    icalvalue_set_datetime(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_new_datetimedate) {

    icalvalue * _result;
    struct icaltimetype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_new_datetimedate(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icalvalue_new_datetimedate. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (icalvalue *)icalvalue_new_datetimedate(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalvaluePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_get_datetimedate) {

    struct icaltimetype * _result;
    icalvalue * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_get_datetimedate(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_get_datetimedate. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _result = (struct icaltimetype *) malloc(sizeof(struct icaltimetype ));
    *(_result) = icalvalue_get_datetimedate(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_set_datetimedate) {

    icalvalue * _arg0;
    struct icaltimetype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalvalue_set_datetimedate(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_set_datetimedate. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icaltimetypePtr")) {
        croak("Type error in argument 2 of icalvalue_set_datetimedate. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    icalvalue_set_datetimedate(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_new_datetimeperiod) {

    icalvalue * _result;
    struct icalperiodtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_new_datetimeperiod(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalperiodtypePtr")) {
        croak("Type error in argument 1 of icalvalue_new_datetimeperiod. Expected struct icalperiodtypePtr.");
        XSRETURN(1);
    }
    _result = (icalvalue *)icalvalue_new_datetimeperiod(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalvaluePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_get_datetimeperiod) {

    struct icalperiodtype * _result;
    icalvalue * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_get_datetimeperiod(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_get_datetimeperiod. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _result = (struct icalperiodtype *) malloc(sizeof(struct icalperiodtype ));
    *(_result) = icalvalue_get_datetimeperiod(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icalperiodtypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_set_datetimeperiod) {

    icalvalue * _arg0;
    struct icalperiodtype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalvalue_set_datetimeperiod(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_set_datetimeperiod. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icalperiodtypePtr")) {
        croak("Type error in argument 2 of icalvalue_set_datetimeperiod. Expected struct icalperiodtypePtr.");
        XSRETURN(1);
    }
    icalvalue_set_datetimeperiod(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_new_duration) {

    icalvalue * _result;
    struct icaldurationtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_new_duration(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaldurationtypePtr")) {
        croak("Type error in argument 1 of icalvalue_new_duration. Expected struct icaldurationtypePtr.");
        XSRETURN(1);
    }
    _result = (icalvalue *)icalvalue_new_duration(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalvaluePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_get_duration) {

    struct icaldurationtype * _result;
    icalvalue * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_get_duration(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_get_duration. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _result = (struct icaldurationtype *) malloc(sizeof(struct icaldurationtype ));
    *(_result) = icalvalue_get_duration(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaldurationtypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_set_duration) {

    icalvalue * _arg0;
    struct icaldurationtype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalvalue_set_duration(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_set_duration. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icaldurationtypePtr")) {
        croak("Type error in argument 2 of icalvalue_set_duration. Expected struct icaldurationtypePtr.");
        XSRETURN(1);
    }
    icalvalue_set_duration(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_new_float) {

    icalvalue * _result;
    float  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_new_float(v);");
    _arg0 = (float ) SvNV(ST(0));
    _result = (icalvalue *)icalvalue_new_float(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalvaluePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_get_float) {

    float  _result;
    icalvalue * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_get_float(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_get_float. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _result = (float )icalvalue_get_float(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setnv(ST(argvi++), (double) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_set_float) {

    icalvalue * _arg0;
    float  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalvalue_set_float(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_set_float. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _arg1 = (float ) SvNV(ST(1));
    icalvalue_set_float(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_new_geo) {

    icalvalue * _result;
    struct icalgeotype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_new_geo(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalgeotypePtr")) {
        croak("Type error in argument 1 of icalvalue_new_geo. Expected struct icalgeotypePtr.");
        XSRETURN(1);
    }
    _result = (icalvalue *)icalvalue_new_geo(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalvaluePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_get_geo) {

    struct icalgeotype * _result;
    icalvalue * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_get_geo(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_get_geo. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _result = (struct icalgeotype *) malloc(sizeof(struct icalgeotype ));
    *(_result) = icalvalue_get_geo(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icalgeotypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_set_geo) {

    icalvalue * _arg0;
    struct icalgeotype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalvalue_set_geo(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_set_geo. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icalgeotypePtr")) {
        croak("Type error in argument 2 of icalvalue_set_geo. Expected struct icalgeotypePtr.");
        XSRETURN(1);
    }
    icalvalue_set_geo(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_new_integer) {

    icalvalue * _result;
    int  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_new_integer(v);");
    _arg0 = (int )SvIV(ST(0));
    _result = (icalvalue *)icalvalue_new_integer(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalvaluePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_get_integer) {

    int  _result;
    icalvalue * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_get_integer(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_get_integer. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _result = (int )icalvalue_get_integer(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_set_integer) {

    icalvalue * _arg0;
    int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalvalue_set_integer(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_set_integer. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _arg1 = (int )SvIV(ST(1));
    icalvalue_set_integer(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_new_method) {

    icalvalue * _result;
    icalproperty_method  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_new_method(v);");
    _arg0 = (icalproperty_method )SvIV(ST(0));
    _result = (icalvalue *)icalvalue_new_method(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalvaluePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_get_method) {

    icalproperty_method  _result;
    icalvalue * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_get_method(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_get_method. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _result = (icalproperty_method )icalvalue_get_method(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_set_method) {

    icalvalue * _arg0;
    icalproperty_method  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalvalue_set_method(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_set_method. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _arg1 = (icalproperty_method )SvIV(ST(1));
    icalvalue_set_method(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_new_period) {

    icalvalue * _result;
    struct icalperiodtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_new_period(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalperiodtypePtr")) {
        croak("Type error in argument 1 of icalvalue_new_period. Expected struct icalperiodtypePtr.");
        XSRETURN(1);
    }
    _result = (icalvalue *)icalvalue_new_period(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalvaluePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_get_period) {

    struct icalperiodtype * _result;
    icalvalue * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_get_period(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_get_period. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _result = (struct icalperiodtype *) malloc(sizeof(struct icalperiodtype ));
    *(_result) = icalvalue_get_period(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icalperiodtypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_set_period) {

    icalvalue * _arg0;
    struct icalperiodtype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalvalue_set_period(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_set_period. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icalperiodtypePtr")) {
        croak("Type error in argument 2 of icalvalue_set_period. Expected struct icalperiodtypePtr.");
        XSRETURN(1);
    }
    icalvalue_set_period(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_new_string) {

    icalvalue * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_new_string(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalvalue *)icalvalue_new_string(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalvaluePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_get_string) {

    char * _result;
    icalvalue * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_get_string(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_get_string. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _result = (char *)icalvalue_get_string(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_set_string) {

    icalvalue * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalvalue_set_string(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_set_string. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalvalue_set_string(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_new_text) {

    icalvalue * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_new_text(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalvalue *)icalvalue_new_text(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalvaluePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_get_text) {

    char * _result;
    icalvalue * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_get_text(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_get_text. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _result = (char *)icalvalue_get_text(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_set_text) {

    icalvalue * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalvalue_set_text(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_set_text. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalvalue_set_text(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_new_time) {

    icalvalue * _result;
    struct icaltimetype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_new_time(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icalvalue_new_time. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (icalvalue *)icalvalue_new_time(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalvaluePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_get_time) {

    struct icaltimetype * _result;
    icalvalue * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_get_time(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_get_time. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _result = (struct icaltimetype *) malloc(sizeof(struct icaltimetype ));
    *(_result) = icalvalue_get_time(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_set_time) {

    icalvalue * _arg0;
    struct icaltimetype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalvalue_set_time(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_set_time. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icaltimetypePtr")) {
        croak("Type error in argument 2 of icalvalue_set_time. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    icalvalue_set_time(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_new_trigger) {

    icalvalue * _result;
    union icaltriggertype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_new_trigger(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"union icaltriggertypePtr")) {
        croak("Type error in argument 1 of icalvalue_new_trigger. Expected union icaltriggertypePtr.");
        XSRETURN(1);
    }
    _result = (icalvalue *)icalvalue_new_trigger(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalvaluePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_get_trigger) {

    union icaltriggertype * _result;
    icalvalue * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_get_trigger(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_get_trigger. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _result = (union icaltriggertype *) malloc(sizeof(union icaltriggertype ));
    *(_result) = icalvalue_get_trigger(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"union icaltriggertypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_set_trigger) {

    icalvalue * _arg0;
    union icaltriggertype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalvalue_set_trigger(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_set_trigger. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"union icaltriggertypePtr")) {
        croak("Type error in argument 2 of icalvalue_set_trigger. Expected union icaltriggertypePtr.");
        XSRETURN(1);
    }
    icalvalue_set_trigger(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_new_uri) {

    icalvalue * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_new_uri(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalvalue *)icalvalue_new_uri(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalvaluePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_get_uri) {

    char * _result;
    icalvalue * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_get_uri(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_get_uri. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _result = (char *)icalvalue_get_uri(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_set_uri) {

    icalvalue * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalvalue_set_uri(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_set_uri. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalvalue_set_uri(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_new_utcoffset) {

    icalvalue * _result;
    int  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_new_utcoffset(v);");
    _arg0 = (int )SvIV(ST(0));
    _result = (icalvalue *)icalvalue_new_utcoffset(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalvaluePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_get_utcoffset) {

    int  _result;
    icalvalue * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_get_utcoffset(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_get_utcoffset. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _result = (int )icalvalue_get_utcoffset(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_set_utcoffset) {

    icalvalue * _arg0;
    int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalvalue_set_utcoffset(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_set_utcoffset. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _arg1 = (int )SvIV(ST(1));
    icalvalue_set_utcoffset(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_new_query) {

    icalvalue * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_new_query(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalvalue *)icalvalue_new_query(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalvaluePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_get_query) {

    char * _result;
    icalvalue * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_get_query(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_get_query. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _result = (char *)icalvalue_get_query(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_set_query) {

    icalvalue * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalvalue_set_query(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_set_query. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalvalue_set_query(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_new_status) {

    icalvalue * _result;
    icalproperty_status  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_new_status(v);");
    _arg0 = (icalproperty_status )SvIV(ST(0));
    _result = (icalvalue *)icalvalue_new_status(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalvaluePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_get_status) {

    icalproperty_status  _result;
    icalvalue * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalvalue_get_status(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_get_status. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _result = (icalproperty_status )icalvalue_get_status(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalvalue_set_status) {

    icalvalue * _arg0;
    icalproperty_status  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalvalue_set_status(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalvalue_set_status. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    _arg1 = (icalproperty_status )SvIV(ST(1));
    icalvalue_set_status(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_new) {

    icalparameter * _result;
    icalparameter_kind  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_new(kind);");
    _arg0 = (icalparameter_kind )SvIV(ST(0));
    _result = (icalparameter *)icalparameter_new(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalparameterPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_new_clone) {

    icalparameter * _result;
    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_new_clone(p);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_new_clone. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _result = (icalparameter *)icalparameter_new_clone(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalparameterPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_new_from_string) {

    icalparameter * _result;
    icalparameter_kind  _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalparameter_new_from_string(kind,value);");
    _arg0 = (icalparameter_kind )SvIV(ST(0));
    _arg1 = (char *) SvPV(ST(1),na);
    _result = (icalparameter *)icalparameter_new_from_string(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalparameterPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_free) {

    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_free(parameter);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_free. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    icalparameter_free(_arg0);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_as_ical_string) {

    char * _result;
    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_as_ical_string(parameter);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_as_ical_string. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalparameter_as_ical_string(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_is_valid) {

    int  _result;
    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_is_valid(parameter);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_is_valid. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _result = (int )icalparameter_is_valid(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_isa) {

    icalparameter_kind  _result;
    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_isa(parameter);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_isa. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _result = (icalparameter_kind )icalparameter_isa(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_isa_parameter) {

    int  _result;
    void * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_isa_parameter(param);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_isa_parameter. Expected voidPtr.");
        XSRETURN(1);
    }
    _result = (int )icalparameter_isa_parameter(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_set_xname) {

    icalparameter * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalparameter_set_xname(param,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_set_xname. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalparameter_set_xname(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_get_xname) {

    char * _result;
    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_get_xname(param);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_get_xname. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalparameter_get_xname(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_set_xvalue) {

    icalparameter * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalparameter_set_xvalue(param,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_set_xvalue. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalparameter_set_xvalue(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_get_xvalue) {

    char * _result;
    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_get_xvalue(param);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_get_xvalue. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalparameter_get_xvalue(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_new_altrep) {

    icalparameter * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_new_altrep(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalparameter *)icalparameter_new_altrep(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalparameterPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_get_altrep) {

    char * _result;
    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_get_altrep(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_get_altrep. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalparameter_get_altrep(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_set_altrep) {

    icalparameter * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalparameter_set_altrep(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_set_altrep. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalparameter_set_altrep(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_new_cn) {

    icalparameter * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_new_cn(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalparameter *)icalparameter_new_cn(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalparameterPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_get_cn) {

    char * _result;
    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_get_cn(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_get_cn. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalparameter_get_cn(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_set_cn) {

    icalparameter * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalparameter_set_cn(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_set_cn. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalparameter_set_cn(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_new_cutype) {

    icalparameter * _result;
    icalparameter_cutype  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_new_cutype(v);");
    _arg0 = (icalparameter_cutype )SvIV(ST(0));
    _result = (icalparameter *)icalparameter_new_cutype(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalparameterPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_get_cutype) {

    icalparameter_cutype  _result;
    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_get_cutype(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_get_cutype. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _result = (icalparameter_cutype )icalparameter_get_cutype(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_set_cutype) {

    icalparameter * _arg0;
    icalparameter_cutype  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalparameter_set_cutype(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_set_cutype. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _arg1 = (icalparameter_cutype )SvIV(ST(1));
    icalparameter_set_cutype(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_new_delegatedfrom) {

    icalparameter * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_new_delegatedfrom(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalparameter *)icalparameter_new_delegatedfrom(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalparameterPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_get_delegatedfrom) {

    char * _result;
    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_get_delegatedfrom(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_get_delegatedfrom. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalparameter_get_delegatedfrom(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_set_delegatedfrom) {

    icalparameter * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalparameter_set_delegatedfrom(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_set_delegatedfrom. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalparameter_set_delegatedfrom(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_new_delegatedto) {

    icalparameter * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_new_delegatedto(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalparameter *)icalparameter_new_delegatedto(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalparameterPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_get_delegatedto) {

    char * _result;
    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_get_delegatedto(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_get_delegatedto. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalparameter_get_delegatedto(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_set_delegatedto) {

    icalparameter * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalparameter_set_delegatedto(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_set_delegatedto. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalparameter_set_delegatedto(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_new_dir) {

    icalparameter * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_new_dir(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalparameter *)icalparameter_new_dir(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalparameterPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_get_dir) {

    char * _result;
    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_get_dir(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_get_dir. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalparameter_get_dir(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_set_dir) {

    icalparameter * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalparameter_set_dir(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_set_dir. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalparameter_set_dir(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_new_encoding) {

    icalparameter * _result;
    icalparameter_encoding  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_new_encoding(v);");
    _arg0 = (icalparameter_encoding )SvIV(ST(0));
    _result = (icalparameter *)icalparameter_new_encoding(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalparameterPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_get_encoding) {

    icalparameter_encoding  _result;
    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_get_encoding(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_get_encoding. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _result = (icalparameter_encoding )icalparameter_get_encoding(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_set_encoding) {

    icalparameter * _arg0;
    icalparameter_encoding  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalparameter_set_encoding(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_set_encoding. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _arg1 = (icalparameter_encoding )SvIV(ST(1));
    icalparameter_set_encoding(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_new_fbtype) {

    icalparameter * _result;
    icalparameter_fbtype  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_new_fbtype(v);");
    _arg0 = (icalparameter_fbtype )SvIV(ST(0));
    _result = (icalparameter *)icalparameter_new_fbtype(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalparameterPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_get_fbtype) {

    icalparameter_fbtype  _result;
    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_get_fbtype(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_get_fbtype. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _result = (icalparameter_fbtype )icalparameter_get_fbtype(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_set_fbtype) {

    icalparameter * _arg0;
    icalparameter_fbtype  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalparameter_set_fbtype(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_set_fbtype. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _arg1 = (icalparameter_fbtype )SvIV(ST(1));
    icalparameter_set_fbtype(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_new_fmttype) {

    icalparameter * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_new_fmttype(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalparameter *)icalparameter_new_fmttype(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalparameterPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_get_fmttype) {

    char * _result;
    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_get_fmttype(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_get_fmttype. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalparameter_get_fmttype(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_set_fmttype) {

    icalparameter * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalparameter_set_fmttype(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_set_fmttype. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalparameter_set_fmttype(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_new_language) {

    icalparameter * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_new_language(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalparameter *)icalparameter_new_language(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalparameterPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_get_language) {

    char * _result;
    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_get_language(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_get_language. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalparameter_get_language(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_set_language) {

    icalparameter * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalparameter_set_language(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_set_language. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalparameter_set_language(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_new_member) {

    icalparameter * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_new_member(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalparameter *)icalparameter_new_member(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalparameterPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_get_member) {

    char * _result;
    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_get_member(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_get_member. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalparameter_get_member(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_set_member) {

    icalparameter * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalparameter_set_member(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_set_member. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalparameter_set_member(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_new_partstat) {

    icalparameter * _result;
    icalparameter_partstat  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_new_partstat(v);");
    _arg0 = (icalparameter_partstat )SvIV(ST(0));
    _result = (icalparameter *)icalparameter_new_partstat(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalparameterPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_get_partstat) {

    icalparameter_partstat  _result;
    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_get_partstat(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_get_partstat. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _result = (icalparameter_partstat )icalparameter_get_partstat(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_set_partstat) {

    icalparameter * _arg0;
    icalparameter_partstat  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalparameter_set_partstat(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_set_partstat. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _arg1 = (icalparameter_partstat )SvIV(ST(1));
    icalparameter_set_partstat(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_new_range) {

    icalparameter * _result;
    icalparameter_range  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_new_range(v);");
    _arg0 = (icalparameter_range )SvIV(ST(0));
    _result = (icalparameter *)icalparameter_new_range(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalparameterPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_get_range) {

    icalparameter_range  _result;
    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_get_range(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_get_range. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _result = (icalparameter_range )icalparameter_get_range(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_set_range) {

    icalparameter * _arg0;
    icalparameter_range  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalparameter_set_range(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_set_range. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _arg1 = (icalparameter_range )SvIV(ST(1));
    icalparameter_set_range(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_new_related) {

    icalparameter * _result;
    icalparameter_related  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_new_related(v);");
    _arg0 = (icalparameter_related )SvIV(ST(0));
    _result = (icalparameter *)icalparameter_new_related(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalparameterPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_get_related) {

    icalparameter_related  _result;
    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_get_related(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_get_related. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _result = (icalparameter_related )icalparameter_get_related(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_set_related) {

    icalparameter * _arg0;
    icalparameter_related  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalparameter_set_related(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_set_related. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _arg1 = (icalparameter_related )SvIV(ST(1));
    icalparameter_set_related(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_new_reltype) {

    icalparameter * _result;
    icalparameter_reltype  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_new_reltype(v);");
    _arg0 = (icalparameter_reltype )SvIV(ST(0));
    _result = (icalparameter *)icalparameter_new_reltype(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalparameterPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_get_reltype) {

    icalparameter_reltype  _result;
    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_get_reltype(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_get_reltype. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _result = (icalparameter_reltype )icalparameter_get_reltype(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_set_reltype) {

    icalparameter * _arg0;
    icalparameter_reltype  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalparameter_set_reltype(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_set_reltype. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _arg1 = (icalparameter_reltype )SvIV(ST(1));
    icalparameter_set_reltype(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_new_role) {

    icalparameter * _result;
    icalparameter_role  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_new_role(v);");
    _arg0 = (icalparameter_role )SvIV(ST(0));
    _result = (icalparameter *)icalparameter_new_role(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalparameterPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_get_role) {

    icalparameter_role  _result;
    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_get_role(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_get_role. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _result = (icalparameter_role )icalparameter_get_role(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_set_role) {

    icalparameter * _arg0;
    icalparameter_role  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalparameter_set_role(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_set_role. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _arg1 = (icalparameter_role )SvIV(ST(1));
    icalparameter_set_role(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_new_rsvp) {

    icalparameter * _result;
    int  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_new_rsvp(v);");
    _arg0 = (int )SvIV(ST(0));
    _result = (icalparameter *)icalparameter_new_rsvp(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalparameterPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_get_rsvp) {

    int  _result;
    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_get_rsvp(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_get_rsvp. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _result = (int )icalparameter_get_rsvp(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_set_rsvp) {

    icalparameter * _arg0;
    int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalparameter_set_rsvp(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_set_rsvp. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _arg1 = (int )SvIV(ST(1));
    icalparameter_set_rsvp(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_new_sentby) {

    icalparameter * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_new_sentby(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalparameter *)icalparameter_new_sentby(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalparameterPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_get_sentby) {

    char * _result;
    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_get_sentby(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_get_sentby. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalparameter_get_sentby(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_set_sentby) {

    icalparameter * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalparameter_set_sentby(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_set_sentby. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalparameter_set_sentby(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_new_tzid) {

    icalparameter * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_new_tzid(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalparameter *)icalparameter_new_tzid(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalparameterPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_get_tzid) {

    char * _result;
    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_get_tzid(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_get_tzid. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalparameter_get_tzid(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_set_tzid) {

    icalparameter * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalparameter_set_tzid(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_set_tzid. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalparameter_set_tzid(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_new_value) {

    icalparameter * _result;
    icalparameter_value  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_new_value(v);");
    _arg0 = (icalparameter_value )SvIV(ST(0));
    _result = (icalparameter *)icalparameter_new_value(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalparameterPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_get_value) {

    icalparameter_value  _result;
    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_get_value(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_get_value. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _result = (icalparameter_value )icalparameter_get_value(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_set_value) {

    icalparameter * _arg0;
    icalparameter_value  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalparameter_set_value(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_set_value. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _arg1 = (icalparameter_value )SvIV(ST(1));
    icalparameter_set_value(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_new_x) {

    icalparameter * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_new_x(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalparameter *)icalparameter_new_x(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalparameterPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_get_x) {

    char * _result;
    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_get_x(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_get_x. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalparameter_get_x(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_set_x) {

    icalparameter * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalparameter_set_x(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_set_x. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalparameter_set_x(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_new_xlicerrortype) {

    icalparameter * _result;
    icalparameter_xlicerrortype  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_new_xlicerrortype(v);");
    _arg0 = (icalparameter_xlicerrortype )SvIV(ST(0));
    _result = (icalparameter *)icalparameter_new_xlicerrortype(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalparameterPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_get_xlicerrortype) {

    icalparameter_xlicerrortype  _result;
    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_get_xlicerrortype(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_get_xlicerrortype. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _result = (icalparameter_xlicerrortype )icalparameter_get_xlicerrortype(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_set_xlicerrortype) {

    icalparameter * _arg0;
    icalparameter_xlicerrortype  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalparameter_set_xlicerrortype(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_set_xlicerrortype. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _arg1 = (icalparameter_xlicerrortype )SvIV(ST(1));
    icalparameter_set_xlicerrortype(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_new_xliccomparetype) {

    icalparameter * _result;
    icalparameter_xliccomparetype  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_new_xliccomparetype(v);");
    _arg0 = (icalparameter_xliccomparetype )SvIV(ST(0));
    _result = (icalparameter *)icalparameter_new_xliccomparetype(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalparameterPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_get_xliccomparetype) {

    icalparameter_xliccomparetype  _result;
    icalparameter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparameter_get_xliccomparetype(value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_get_xliccomparetype. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _result = (icalparameter_xliccomparetype )icalparameter_get_xliccomparetype(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparameter_set_xliccomparetype) {

    icalparameter * _arg0;
    icalparameter_xliccomparetype  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalparameter_set_xliccomparetype(value,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparameter_set_xliccomparetype. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    _arg1 = (icalparameter_xliccomparetype )SvIV(ST(1));
    icalparameter_set_xliccomparetype(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new) {

    icalproperty * _result;
    icalproperty_kind  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new(kind);");
    _arg0 = (icalproperty_kind )SvIV(ST(0));
    _result = (icalproperty *)icalproperty_new(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_clone) {

    icalproperty * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_clone(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_new_clone. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (icalproperty *)icalproperty_new_clone(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_from_string) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_from_string(str);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_from_string(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_as_ical_string) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_as_ical_string(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_as_ical_string. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_as_ical_string(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_free) {

    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_free(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_free. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    icalproperty_free(_arg0);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_isa) {

    icalproperty_kind  _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_isa(property);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_isa. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (icalproperty_kind )icalproperty_isa(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_isa_property) {

    int  _result;
    void * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_isa_property(property);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_isa_property. Expected voidPtr.");
        XSRETURN(1);
    }
    _result = (int )icalproperty_isa_property(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_add_parameter) {

    icalproperty * _arg0;
    icalparameter * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_add_parameter(prop,parameter);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_add_parameter. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,(char *) 0 )) {
        croak("Type error in argument 2 of icalproperty_add_parameter. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    icalproperty_add_parameter(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_parameter) {

    icalproperty * _arg0;
    icalparameter * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_parameter(prop,parameter);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_parameter. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,(char *) 0 )) {
        croak("Type error in argument 2 of icalproperty_set_parameter. Expected icalparameterPtr.");
        XSRETURN(1);
    }
    icalproperty_set_parameter(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_remove_parameter) {

    icalproperty * _arg0;
    icalparameter_kind  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_remove_parameter(prop,kind);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_remove_parameter. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (icalparameter_kind )SvIV(ST(1));
    icalproperty_remove_parameter(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_count_parameters) {

    int  _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_count_parameters(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_count_parameters. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (int )icalproperty_count_parameters(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_first_parameter) {

    icalparameter * _result;
    icalproperty * _arg0;
    icalparameter_kind  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_get_first_parameter(prop,kind);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_first_parameter. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (icalparameter_kind )SvIV(ST(1));
    _result = (icalparameter *)icalproperty_get_first_parameter(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalparameterPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_next_parameter) {

    icalparameter * _result;
    icalproperty * _arg0;
    icalparameter_kind  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_get_next_parameter(prop,kind);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_next_parameter. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (icalparameter_kind )SvIV(ST(1));
    _result = (icalparameter *)icalproperty_get_next_parameter(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalparameterPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_value) {

    icalproperty * _arg0;
    icalvalue * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_value(prop,value);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_value. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,(char *) 0 )) {
        croak("Type error in argument 2 of icalproperty_set_value. Expected icalvaluePtr.");
        XSRETURN(1);
    }
    icalproperty_set_value(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_value) {

    icalvalue * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_value(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_value. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (icalvalue *)icalproperty_get_value(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalvaluePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_x_name) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_x_name(prop,name);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_x_name. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_x_name(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_x_name) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_x_name(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_x_name. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_x_name(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_method) {

    icalproperty * _result;
    icalproperty_method  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_method(v);");
    _arg0 = (icalproperty_method )SvIV(ST(0));
    _result = (icalproperty *)icalproperty_new_method(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_method) {

    icalproperty * _arg0;
    icalproperty_method  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_method(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_method. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (icalproperty_method )SvIV(ST(1));
    icalproperty_set_method(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_method) {

    icalproperty_method  _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_method(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_method. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (icalproperty_method )icalproperty_get_method(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_xlicmimecid) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_xlicmimecid(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_xlicmimecid(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_xlicmimecid) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_xlicmimecid(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_xlicmimecid. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_xlicmimecid(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_xlicmimecid) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_xlicmimecid(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_xlicmimecid. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_xlicmimecid(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_lastmodified) {

    icalproperty * _result;
    struct icaltimetype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_lastmodified(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icalproperty_new_lastmodified. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (icalproperty *)icalproperty_new_lastmodified(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_lastmodified) {

    icalproperty * _arg0;
    struct icaltimetype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_lastmodified(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_lastmodified. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icaltimetypePtr")) {
        croak("Type error in argument 2 of icalproperty_set_lastmodified. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    icalproperty_set_lastmodified(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_lastmodified) {

    struct icaltimetype * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_lastmodified(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_lastmodified. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (struct icaltimetype *) malloc(sizeof(struct icaltimetype ));
    *(_result) = icalproperty_get_lastmodified(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_uid) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_uid(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_uid(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_uid) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_uid(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_uid. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_uid(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_uid) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_uid(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_uid. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_uid(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_prodid) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_prodid(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_prodid(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_prodid) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_prodid(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_prodid. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_prodid(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_prodid) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_prodid(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_prodid. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_prodid(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_status) {

    icalproperty * _result;
    icalproperty_status  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_status(v);");
    _arg0 = (icalproperty_status )SvIV(ST(0));
    _result = (icalproperty *)icalproperty_new_status(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_status) {

    icalproperty * _arg0;
    icalproperty_status  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_status(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_status. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (icalproperty_status )SvIV(ST(1));
    icalproperty_set_status(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_status) {

    icalproperty_status  _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_status(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_status. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (icalproperty_status )icalproperty_get_status(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_description) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_description(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_description(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_description) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_description(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_description. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_description(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_description) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_description(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_description. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_description(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_duration) {

    icalproperty * _result;
    struct icaldurationtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_duration(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaldurationtypePtr")) {
        croak("Type error in argument 1 of icalproperty_new_duration. Expected struct icaldurationtypePtr.");
        XSRETURN(1);
    }
    _result = (icalproperty *)icalproperty_new_duration(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_duration) {

    icalproperty * _arg0;
    struct icaldurationtype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_duration(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_duration. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icaldurationtypePtr")) {
        croak("Type error in argument 2 of icalproperty_set_duration. Expected struct icaldurationtypePtr.");
        XSRETURN(1);
    }
    icalproperty_set_duration(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_duration) {

    struct icaldurationtype * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_duration(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_duration. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (struct icaldurationtype *) malloc(sizeof(struct icaldurationtype ));
    *(_result) = icalproperty_get_duration(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaldurationtypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_categories) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_categories(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_categories(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_categories) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_categories(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_categories. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_categories(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_categories) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_categories(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_categories. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_categories(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_version) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_version(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_version(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_version) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_version(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_version. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_version(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_version) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_version(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_version. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_version(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_tzoffsetfrom) {

    icalproperty * _result;
    int  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_tzoffsetfrom(v);");
    _arg0 = (int )SvIV(ST(0));
    _result = (icalproperty *)icalproperty_new_tzoffsetfrom(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_tzoffsetfrom) {

    icalproperty * _arg0;
    int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_tzoffsetfrom(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_tzoffsetfrom. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (int )SvIV(ST(1));
    icalproperty_set_tzoffsetfrom(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_tzoffsetfrom) {

    int  _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_tzoffsetfrom(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_tzoffsetfrom. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (int )icalproperty_get_tzoffsetfrom(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_attendee) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_attendee(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_attendee(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_attendee) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_attendee(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_attendee. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_attendee(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_attendee) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_attendee(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_attendee. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_attendee(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_contact) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_contact(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_contact(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_contact) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_contact(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_contact. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_contact(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_contact) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_contact(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_contact. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_contact(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_xlicmimecontenttype) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_xlicmimecontenttype(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_xlicmimecontenttype(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_xlicmimecontenttype) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_xlicmimecontenttype(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_xlicmimecontenttype. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_xlicmimecontenttype(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_xlicmimecontenttype) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_xlicmimecontenttype(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_xlicmimecontenttype. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_xlicmimecontenttype(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_xlicmimeoptinfo) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_xlicmimeoptinfo(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_xlicmimeoptinfo(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_xlicmimeoptinfo) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_xlicmimeoptinfo(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_xlicmimeoptinfo. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_xlicmimeoptinfo(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_xlicmimeoptinfo) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_xlicmimeoptinfo(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_xlicmimeoptinfo. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_xlicmimeoptinfo(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_relatedto) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_relatedto(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_relatedto(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_relatedto) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_relatedto(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_relatedto. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_relatedto(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_relatedto) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_relatedto(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_relatedto. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_relatedto(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_organizer) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_organizer(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_organizer(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_organizer) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_organizer(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_organizer. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_organizer(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_organizer) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_organizer(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_organizer. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_organizer(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_comment) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_comment(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_comment(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_comment) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_comment(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_comment. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_comment(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_comment) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_comment(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_comment. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_comment(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_xlicerror) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_xlicerror(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_xlicerror(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_xlicerror) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_xlicerror(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_xlicerror. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_xlicerror(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_xlicerror) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_xlicerror(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_xlicerror. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_xlicerror(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_trigger) {

    icalproperty * _result;
    union icaltriggertype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_trigger(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"union icaltriggertypePtr")) {
        croak("Type error in argument 1 of icalproperty_new_trigger. Expected union icaltriggertypePtr.");
        XSRETURN(1);
    }
    _result = (icalproperty *)icalproperty_new_trigger(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_trigger) {

    icalproperty * _arg0;
    union icaltriggertype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_trigger(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_trigger. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"union icaltriggertypePtr")) {
        croak("Type error in argument 2 of icalproperty_set_trigger. Expected union icaltriggertypePtr.");
        XSRETURN(1);
    }
    icalproperty_set_trigger(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_trigger) {

    union icaltriggertype * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_trigger(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_trigger. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (union icaltriggertype *) malloc(sizeof(union icaltriggertype ));
    *(_result) = icalproperty_get_trigger(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"union icaltriggertypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_class) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_class(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_class(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_class) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_class(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_class. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_class(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_class) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_class(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_class. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_class(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_x) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_x(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_x(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_x) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_x(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_x. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_x(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_x) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_x(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_x. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_x(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_tzoffsetto) {

    icalproperty * _result;
    int  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_tzoffsetto(v);");
    _arg0 = (int )SvIV(ST(0));
    _result = (icalproperty *)icalproperty_new_tzoffsetto(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_tzoffsetto) {

    icalproperty * _arg0;
    int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_tzoffsetto(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_tzoffsetto. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (int )SvIV(ST(1));
    icalproperty_set_tzoffsetto(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_tzoffsetto) {

    int  _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_tzoffsetto(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_tzoffsetto. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (int )icalproperty_get_tzoffsetto(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_transp) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_transp(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_transp(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_transp) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_transp(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_transp. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_transp(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_transp) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_transp(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_transp. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_transp(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_xlicmimeencoding) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_xlicmimeencoding(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_xlicmimeencoding(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_xlicmimeencoding) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_xlicmimeencoding(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_xlicmimeencoding. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_xlicmimeencoding(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_xlicmimeencoding) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_xlicmimeencoding(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_xlicmimeencoding. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_xlicmimeencoding(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_sequence) {

    icalproperty * _result;
    int  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_sequence(v);");
    _arg0 = (int )SvIV(ST(0));
    _result = (icalproperty *)icalproperty_new_sequence(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_sequence) {

    icalproperty * _arg0;
    int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_sequence(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_sequence. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (int )SvIV(ST(1));
    icalproperty_set_sequence(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_sequence) {

    int  _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_sequence(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_sequence. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (int )icalproperty_get_sequence(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_location) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_location(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_location(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_location) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_location(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_location. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_location(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_location) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_location(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_location. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_location(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_requeststatus) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_requeststatus(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_requeststatus(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_requeststatus) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_requeststatus(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_requeststatus. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_requeststatus(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_requeststatus) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_requeststatus(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_requeststatus. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_requeststatus(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_exdate) {

    icalproperty * _result;
    struct icaltimetype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_exdate(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icalproperty_new_exdate. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (icalproperty *)icalproperty_new_exdate(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_exdate) {

    icalproperty * _arg0;
    struct icaltimetype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_exdate(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_exdate. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icaltimetypePtr")) {
        croak("Type error in argument 2 of icalproperty_set_exdate. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    icalproperty_set_exdate(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_exdate) {

    struct icaltimetype * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_exdate(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_exdate. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (struct icaltimetype *) malloc(sizeof(struct icaltimetype ));
    *(_result) = icalproperty_get_exdate(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_tzid) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_tzid(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_tzid(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_tzid) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_tzid(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_tzid. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_tzid(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_tzid) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_tzid(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_tzid. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_tzid(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_resources) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_resources(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_resources(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_resources) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_resources(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_resources. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_resources(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_resources) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_resources(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_resources. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_resources(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_tzurl) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_tzurl(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_tzurl(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_tzurl) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_tzurl(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_tzurl. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_tzurl(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_tzurl) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_tzurl(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_tzurl. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_tzurl(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_repeat) {

    icalproperty * _result;
    int  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_repeat(v);");
    _arg0 = (int )SvIV(ST(0));
    _result = (icalproperty *)icalproperty_new_repeat(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_repeat) {

    icalproperty * _arg0;
    int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_repeat(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_repeat. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (int )SvIV(ST(1));
    icalproperty_set_repeat(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_repeat) {

    int  _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_repeat(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_repeat. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (int )icalproperty_get_repeat(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_priority) {

    icalproperty * _result;
    int  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_priority(v);");
    _arg0 = (int )SvIV(ST(0));
    _result = (icalproperty *)icalproperty_new_priority(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_priority) {

    icalproperty * _arg0;
    int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_priority(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_priority. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (int )SvIV(ST(1));
    icalproperty_set_priority(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_priority) {

    int  _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_priority(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_priority. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (int )icalproperty_get_priority(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_freebusy) {

    icalproperty * _result;
    struct icalperiodtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_freebusy(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalperiodtypePtr")) {
        croak("Type error in argument 1 of icalproperty_new_freebusy. Expected struct icalperiodtypePtr.");
        XSRETURN(1);
    }
    _result = (icalproperty *)icalproperty_new_freebusy(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_freebusy) {

    icalproperty * _arg0;
    struct icalperiodtype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_freebusy(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_freebusy. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icalperiodtypePtr")) {
        croak("Type error in argument 2 of icalproperty_set_freebusy. Expected struct icalperiodtypePtr.");
        XSRETURN(1);
    }
    icalproperty_set_freebusy(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_freebusy) {

    struct icalperiodtype * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_freebusy(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_freebusy. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (struct icalperiodtype *) malloc(sizeof(struct icalperiodtype ));
    *(_result) = icalproperty_get_freebusy(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icalperiodtypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_dtstart) {

    icalproperty * _result;
    struct icaltimetype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_dtstart(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icalproperty_new_dtstart. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (icalproperty *)icalproperty_new_dtstart(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_dtstart) {

    icalproperty * _arg0;
    struct icaltimetype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_dtstart(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_dtstart. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icaltimetypePtr")) {
        croak("Type error in argument 2 of icalproperty_set_dtstart. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    icalproperty_set_dtstart(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_dtstart) {

    struct icaltimetype * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_dtstart(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_dtstart. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (struct icaltimetype *) malloc(sizeof(struct icaltimetype ));
    *(_result) = icalproperty_get_dtstart(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_recurrenceid) {

    icalproperty * _result;
    struct icaltimetype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_recurrenceid(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icalproperty_new_recurrenceid. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (icalproperty *)icalproperty_new_recurrenceid(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_recurrenceid) {

    icalproperty * _arg0;
    struct icaltimetype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_recurrenceid(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_recurrenceid. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icaltimetypePtr")) {
        croak("Type error in argument 2 of icalproperty_set_recurrenceid. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    icalproperty_set_recurrenceid(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_recurrenceid) {

    struct icaltimetype * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_recurrenceid(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_recurrenceid. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (struct icaltimetype *) malloc(sizeof(struct icaltimetype ));
    *(_result) = icalproperty_get_recurrenceid(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_summary) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_summary(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_summary(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_summary) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_summary(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_summary. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_summary(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_summary) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_summary(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_summary. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_summary(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_dtend) {

    icalproperty * _result;
    struct icaltimetype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_dtend(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icalproperty_new_dtend. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (icalproperty *)icalproperty_new_dtend(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_dtend) {

    icalproperty * _arg0;
    struct icaltimetype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_dtend(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_dtend. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icaltimetypePtr")) {
        croak("Type error in argument 2 of icalproperty_set_dtend. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    icalproperty_set_dtend(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_dtend) {

    struct icaltimetype * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_dtend(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_dtend. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (struct icaltimetype *) malloc(sizeof(struct icaltimetype ));
    *(_result) = icalproperty_get_dtend(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_tzname) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_tzname(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_tzname(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_tzname) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_tzname(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_tzname. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_tzname(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_tzname) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_tzname(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_tzname. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_tzname(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_rdate) {

    icalproperty * _result;
    struct icalperiodtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_rdate(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalperiodtypePtr")) {
        croak("Type error in argument 1 of icalproperty_new_rdate. Expected struct icalperiodtypePtr.");
        XSRETURN(1);
    }
    _result = (icalproperty *)icalproperty_new_rdate(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_rdate) {

    icalproperty * _arg0;
    struct icalperiodtype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_rdate(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_rdate. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icalperiodtypePtr")) {
        croak("Type error in argument 2 of icalproperty_set_rdate. Expected struct icalperiodtypePtr.");
        XSRETURN(1);
    }
    icalproperty_set_rdate(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_rdate) {

    struct icalperiodtype * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_rdate(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_rdate. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (struct icalperiodtype *) malloc(sizeof(struct icalperiodtype ));
    *(_result) = icalproperty_get_rdate(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icalperiodtypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_xlicmimefilename) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_xlicmimefilename(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_xlicmimefilename(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_xlicmimefilename) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_xlicmimefilename(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_xlicmimefilename. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_xlicmimefilename(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_xlicmimefilename) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_xlicmimefilename(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_xlicmimefilename. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_xlicmimefilename(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_url) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_url(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_url(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_url) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_url(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_url. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_url(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_url) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_url(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_url. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_url(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_xlicclustercount) {

    icalproperty * _result;
    int  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_xlicclustercount(v);");
    _arg0 = (int )SvIV(ST(0));
    _result = (icalproperty *)icalproperty_new_xlicclustercount(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_xlicclustercount) {

    icalproperty * _arg0;
    int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_xlicclustercount(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_xlicclustercount. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (int )SvIV(ST(1));
    icalproperty_set_xlicclustercount(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_xlicclustercount) {

    int  _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_xlicclustercount(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_xlicclustercount. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (int )icalproperty_get_xlicclustercount(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_attach) {

    icalproperty * _result;
    struct icalattachtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_attach(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalattachtypePtr")) {
        croak("Type error in argument 1 of icalproperty_new_attach. Expected struct icalattachtypePtr.");
        XSRETURN(1);
    }
    _result = (icalproperty *)icalproperty_new_attach(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_attach) {

    icalproperty * _arg0;
    struct icalattachtype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_attach(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_attach. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icalattachtypePtr")) {
        croak("Type error in argument 2 of icalproperty_set_attach. Expected struct icalattachtypePtr.");
        XSRETURN(1);
    }
    icalproperty_set_attach(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_attach) {

    struct icalattachtype * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_attach(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_attach. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (struct icalattachtype *) malloc(sizeof(struct icalattachtype ));
    *(_result) = icalproperty_get_attach(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icalattachtypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_query) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_query(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_query(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_query) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_query(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_query. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_query(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_query) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_query(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_query. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_query(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_percentcomplete) {

    icalproperty * _result;
    int  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_percentcomplete(v);");
    _arg0 = (int )SvIV(ST(0));
    _result = (icalproperty *)icalproperty_new_percentcomplete(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_percentcomplete) {

    icalproperty * _arg0;
    int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_percentcomplete(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_percentcomplete. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (int )SvIV(ST(1));
    icalproperty_set_percentcomplete(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_percentcomplete) {

    int  _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_percentcomplete(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_percentcomplete. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (int )icalproperty_get_percentcomplete(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_calscale) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_calscale(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_calscale(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_calscale) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_calscale(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_calscale. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_calscale(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_calscale) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_calscale(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_calscale. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_calscale(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_created) {

    icalproperty * _result;
    struct icaltimetype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_created(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icalproperty_new_created. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (icalproperty *)icalproperty_new_created(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_created) {

    icalproperty * _arg0;
    struct icaltimetype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_created(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_created. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icaltimetypePtr")) {
        croak("Type error in argument 2 of icalproperty_set_created. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    icalproperty_set_created(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_created) {

    struct icaltimetype * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_created(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_created. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (struct icaltimetype *) malloc(sizeof(struct icaltimetype ));
    *(_result) = icalproperty_get_created(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_geo) {

    icalproperty * _result;
    struct icalgeotype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_geo(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalgeotypePtr")) {
        croak("Type error in argument 1 of icalproperty_new_geo. Expected struct icalgeotypePtr.");
        XSRETURN(1);
    }
    _result = (icalproperty *)icalproperty_new_geo(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_geo) {

    icalproperty * _arg0;
    struct icalgeotype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_geo(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_geo. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icalgeotypePtr")) {
        croak("Type error in argument 2 of icalproperty_set_geo. Expected struct icalgeotypePtr.");
        XSRETURN(1);
    }
    icalproperty_set_geo(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_geo) {

    struct icalgeotype * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_geo(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_geo. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (struct icalgeotype *) malloc(sizeof(struct icalgeotype ));
    *(_result) = icalproperty_get_geo(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icalgeotypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_xlicmimecharset) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_xlicmimecharset(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_xlicmimecharset(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_xlicmimecharset) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_xlicmimecharset(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_xlicmimecharset. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_xlicmimecharset(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_xlicmimecharset) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_xlicmimecharset(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_xlicmimecharset. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_xlicmimecharset(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_completed) {

    icalproperty * _result;
    struct icaltimetype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_completed(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icalproperty_new_completed. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (icalproperty *)icalproperty_new_completed(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_completed) {

    icalproperty * _arg0;
    struct icaltimetype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_completed(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_completed. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icaltimetypePtr")) {
        croak("Type error in argument 2 of icalproperty_set_completed. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    icalproperty_set_completed(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_completed) {

    struct icaltimetype * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_completed(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_completed. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (struct icaltimetype *) malloc(sizeof(struct icaltimetype ));
    *(_result) = icalproperty_get_completed(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_dtstamp) {

    icalproperty * _result;
    struct icaltimetype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_dtstamp(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icalproperty_new_dtstamp. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (icalproperty *)icalproperty_new_dtstamp(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_dtstamp) {

    icalproperty * _arg0;
    struct icaltimetype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_dtstamp(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_dtstamp. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icaltimetypePtr")) {
        croak("Type error in argument 2 of icalproperty_set_dtstamp. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    icalproperty_set_dtstamp(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_dtstamp) {

    struct icaltimetype * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_dtstamp(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_dtstamp. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (struct icaltimetype *) malloc(sizeof(struct icaltimetype ));
    *(_result) = icalproperty_get_dtstamp(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_due) {

    icalproperty * _result;
    struct icaltimetype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_due(v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icalproperty_new_due. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (icalproperty *)icalproperty_new_due(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_due) {

    icalproperty * _arg0;
    struct icaltimetype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_due(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_due. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icaltimetypePtr")) {
        croak("Type error in argument 2 of icalproperty_set_due. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    icalproperty_set_due(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_due) {

    struct icaltimetype * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_due(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_due. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (struct icaltimetype *) malloc(sizeof(struct icaltimetype ));
    *(_result) = icalproperty_get_due(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_new_action) {

    icalproperty * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_new_action(v);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalproperty *)icalproperty_new_action(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_set_action) {

    icalproperty * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalproperty_set_action(prop,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_set_action. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalproperty_set_action(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalproperty_get_action) {

    char * _result;
    icalproperty * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalproperty_get_action(prop);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalproperty_get_action. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalproperty_get_action(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_pvl_new_element) {

    pvl_elem  _result;
    void * _arg0;
    pvl_elem  _arg1;
    pvl_elem  _arg2;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 3) || (items > 3)) 
        croak("Usage: pvl_new_element(d,next,prior);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of pvl_new_element. Expected voidPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,(char *) 0 )) {
        croak("Type error in argument 2 of pvl_new_element. Expected pvl_elem.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(2),(void **) &_arg2,(char *) 0 )) {
        croak("Type error in argument 3 of pvl_new_element. Expected pvl_elem.");
        XSRETURN(1);
    }
    _result = (pvl_elem )pvl_new_element(_arg0,_arg1,_arg2);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"pvl_elem", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_pvl_newlist) {

    pvl_list  _result;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 0) || (items > 0)) 
        croak("Usage: pvl_newlist();");
    _result = (pvl_list )pvl_newlist();
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"pvl_list", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_pvl_free) {

    pvl_list  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: pvl_free(pvl_list );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of pvl_free. Expected pvl_list.");
        XSRETURN(1);
    }
    pvl_free(_arg0);
    XSRETURN(argvi);
}

XS(_wrap_pvl_unshift) {

    pvl_list  _arg0;
    void * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: pvl_unshift(l,d);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of pvl_unshift. Expected pvl_list.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,(char *) 0 )) {
        croak("Type error in argument 2 of pvl_unshift. Expected voidPtr.");
        XSRETURN(1);
    }
    pvl_unshift(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_pvl_shift) {

    void * _result;
    pvl_list  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: pvl_shift(l);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of pvl_shift. Expected pvl_list.");
        XSRETURN(1);
    }
    _result = (void *)pvl_shift(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"voidPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_pvl_head) {

    pvl_elem  _result;
    pvl_list  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: pvl_head(pvl_list );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of pvl_head. Expected pvl_list.");
        XSRETURN(1);
    }
    _result = (pvl_elem )pvl_head(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"pvl_elem", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_pvl_push) {

    pvl_list  _arg0;
    void * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: pvl_push(l,d);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of pvl_push. Expected pvl_list.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,(char *) 0 )) {
        croak("Type error in argument 2 of pvl_push. Expected voidPtr.");
        XSRETURN(1);
    }
    pvl_push(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_pvl_pop) {

    void * _result;
    pvl_list  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: pvl_pop(l);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of pvl_pop. Expected pvl_list.");
        XSRETURN(1);
    }
    _result = (void *)pvl_pop(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"voidPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_pvl_tail) {

    pvl_elem  _result;
    pvl_list  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: pvl_tail(pvl_list );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of pvl_tail. Expected pvl_list.");
        XSRETURN(1);
    }
    _result = (pvl_elem )pvl_tail(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"pvl_elem", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_pvl_insert_ordered) {

    pvl_list  _arg0;
    pvl_comparef  _arg1;
    void * _arg2;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 3) || (items > 3)) 
        croak("Usage: pvl_insert_ordered(l,f,d);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of pvl_insert_ordered. Expected pvl_list.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"pvl_comparef")) {
        croak("Type error in argument 2 of pvl_insert_ordered. Expected pvl_comparef.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(2),(void **) &_arg2,(char *) 0 )) {
        croak("Type error in argument 3 of pvl_insert_ordered. Expected voidPtr.");
        XSRETURN(1);
    }
    pvl_insert_ordered(_arg0,_arg1,_arg2);
    XSRETURN(argvi);
}

XS(_wrap_pvl_insert_after) {

    pvl_list  _arg0;
    pvl_elem  _arg1;
    void * _arg2;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 3) || (items > 3)) 
        croak("Usage: pvl_insert_after(l,e,d);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of pvl_insert_after. Expected pvl_list.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,(char *) 0 )) {
        croak("Type error in argument 2 of pvl_insert_after. Expected pvl_elem.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(2),(void **) &_arg2,(char *) 0 )) {
        croak("Type error in argument 3 of pvl_insert_after. Expected voidPtr.");
        XSRETURN(1);
    }
    pvl_insert_after(_arg0,_arg1,_arg2);
    XSRETURN(argvi);
}

XS(_wrap_pvl_insert_before) {

    pvl_list  _arg0;
    pvl_elem  _arg1;
    void * _arg2;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 3) || (items > 3)) 
        croak("Usage: pvl_insert_before(l,e,d);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of pvl_insert_before. Expected pvl_list.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,(char *) 0 )) {
        croak("Type error in argument 2 of pvl_insert_before. Expected pvl_elem.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(2),(void **) &_arg2,(char *) 0 )) {
        croak("Type error in argument 3 of pvl_insert_before. Expected voidPtr.");
        XSRETURN(1);
    }
    pvl_insert_before(_arg0,_arg1,_arg2);
    XSRETURN(argvi);
}

XS(_wrap_pvl_remove) {

    void * _result;
    pvl_list  _arg0;
    pvl_elem  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: pvl_remove(pvl_list ,pvl_elem );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of pvl_remove. Expected pvl_list.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,(char *) 0 )) {
        croak("Type error in argument 2 of pvl_remove. Expected pvl_elem.");
        XSRETURN(1);
    }
    _result = (void *)pvl_remove(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"voidPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_pvl_clear) {

    pvl_list  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: pvl_clear(pvl_list );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of pvl_clear. Expected pvl_list.");
        XSRETURN(1);
    }
    pvl_clear(_arg0);
    XSRETURN(argvi);
}

XS(_wrap_pvl_count) {

    int  _result;
    pvl_list  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: pvl_count(pvl_list );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of pvl_count. Expected pvl_list.");
        XSRETURN(1);
    }
    _result = (int )pvl_count(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_pvl_next) {

    pvl_elem  _result;
    pvl_elem  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: pvl_next(e);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of pvl_next. Expected pvl_elem.");
        XSRETURN(1);
    }
    _result = (pvl_elem )pvl_next(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"pvl_elem", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_pvl_prior) {

    pvl_elem  _result;
    pvl_elem  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: pvl_prior(e);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of pvl_prior. Expected pvl_elem.");
        XSRETURN(1);
    }
    _result = (pvl_elem )pvl_prior(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"pvl_elem", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_pvl_data) {

    void * _result;
    pvl_elem  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: pvl_data(pvl_elem );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of pvl_data. Expected pvl_elem.");
        XSRETURN(1);
    }
    _result = (void *)pvl_data(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"voidPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_pvl_find) {

    pvl_elem  _result;
    pvl_list  _arg0;
    pvl_findf  _arg1;
    void * _arg2;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 3) || (items > 3)) 
        croak("Usage: pvl_find(l,f,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of pvl_find. Expected pvl_list.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"pvl_findf")) {
        croak("Type error in argument 2 of pvl_find. Expected pvl_findf.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(2),(void **) &_arg2,(char *) 0 )) {
        croak("Type error in argument 3 of pvl_find. Expected voidPtr.");
        XSRETURN(1);
    }
    _result = (pvl_elem )pvl_find(_arg0,_arg1,_arg2);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"pvl_elem", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_pvl_find_next) {

    pvl_elem  _result;
    pvl_list  _arg0;
    pvl_findf  _arg1;
    void * _arg2;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 3) || (items > 3)) 
        croak("Usage: pvl_find_next(l,f,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of pvl_find_next. Expected pvl_list.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"pvl_findf")) {
        croak("Type error in argument 2 of pvl_find_next. Expected pvl_findf.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(2),(void **) &_arg2,(char *) 0 )) {
        croak("Type error in argument 3 of pvl_find_next. Expected voidPtr.");
        XSRETURN(1);
    }
    _result = (pvl_elem )pvl_find_next(_arg0,_arg1,_arg2);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"pvl_elem", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_pvl_apply) {

    pvl_list  _arg0;
    pvl_applyf  _arg1;
    void * _arg2;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 3) || (items > 3)) 
        croak("Usage: pvl_apply(l,f,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of pvl_apply. Expected pvl_list.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"pvl_applyf")) {
        croak("Type error in argument 2 of pvl_apply. Expected pvl_applyf.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(2),(void **) &_arg2,(char *) 0 )) {
        croak("Type error in argument 3 of pvl_apply. Expected voidPtr.");
        XSRETURN(1);
    }
    pvl_apply(_arg0,_arg1,_arg2);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_new) {

    icalcomponent * _result;
    icalcomponent_kind  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcomponent_new(kind);");
    _arg0 = (icalcomponent_kind )SvIV(ST(0));
    _result = (icalcomponent *)icalcomponent_new(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalcomponentPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_new_clone) {

    icalcomponent * _result;
    icalcomponent * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcomponent_new_clone(component);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_new_clone. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _result = (icalcomponent *)icalcomponent_new_clone(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalcomponentPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_new_from_string) {

    icalcomponent * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcomponent_new_from_string(str);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalcomponent *)icalcomponent_new_from_string(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalcomponentPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_free) {

    icalcomponent * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcomponent_free(component);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_free. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    icalcomponent_free(_arg0);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_as_ical_string) {

    char * _result;
    icalcomponent * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcomponent_as_ical_string(component);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_as_ical_string. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalcomponent_as_ical_string(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_is_valid) {

    int  _result;
    icalcomponent * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcomponent_is_valid(component);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_is_valid. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _result = (int )icalcomponent_is_valid(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_isa) {

    icalcomponent_kind  _result;
    icalcomponent * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcomponent_isa(component);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_isa. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _result = (icalcomponent_kind )icalcomponent_isa(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_isa_component) {

    int  _result;
    void * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcomponent_isa_component(component);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_isa_component. Expected voidPtr.");
        XSRETURN(1);
    }
    _result = (int )icalcomponent_isa_component(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_add_property) {

    icalcomponent * _arg0;
    icalproperty * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalcomponent_add_property(component,property);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_add_property. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,(char *) 0 )) {
        croak("Type error in argument 2 of icalcomponent_add_property. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    icalcomponent_add_property(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_remove_property) {

    icalcomponent * _arg0;
    icalproperty * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalcomponent_remove_property(component,property);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_remove_property. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,(char *) 0 )) {
        croak("Type error in argument 2 of icalcomponent_remove_property. Expected icalpropertyPtr.");
        XSRETURN(1);
    }
    icalcomponent_remove_property(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_count_properties) {

    int  _result;
    icalcomponent * _arg0;
    icalproperty_kind  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalcomponent_count_properties(component,kind);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_count_properties. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _arg1 = (icalproperty_kind )SvIV(ST(1));
    _result = (int )icalcomponent_count_properties(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_get_current_property) {

    icalproperty * _result;
    icalcomponent * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcomponent_get_current_property(component);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_get_current_property. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _result = (icalproperty *)icalcomponent_get_current_property(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_get_first_property) {

    icalproperty * _result;
    icalcomponent * _arg0;
    icalproperty_kind  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalcomponent_get_first_property(component,kind);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_get_first_property. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _arg1 = (icalproperty_kind )SvIV(ST(1));
    _result = (icalproperty *)icalcomponent_get_first_property(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_get_next_property) {

    icalproperty * _result;
    icalcomponent * _arg0;
    icalproperty_kind  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalcomponent_get_next_property(component,kind);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_get_next_property. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _arg1 = (icalproperty_kind )SvIV(ST(1));
    _result = (icalproperty *)icalcomponent_get_next_property(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_get_properties) {

    icalproperty ** _result;
    icalcomponent * _arg0;
    icalproperty_kind  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalcomponent_get_properties(component,kind);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_get_properties. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _arg1 = (icalproperty_kind )SvIV(ST(1));
    _result = (icalproperty **)icalcomponent_get_properties(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalpropertyPtrPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_add_component) {

    icalcomponent * _arg0;
    icalcomponent * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalcomponent_add_component(parent,child);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_add_component. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,(char *) 0 )) {
        croak("Type error in argument 2 of icalcomponent_add_component. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    icalcomponent_add_component(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_remove_component) {

    icalcomponent * _arg0;
    icalcomponent * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalcomponent_remove_component(parent,child);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_remove_component. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,(char *) 0 )) {
        croak("Type error in argument 2 of icalcomponent_remove_component. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    icalcomponent_remove_component(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_count_components) {

    int  _result;
    icalcomponent * _arg0;
    icalcomponent_kind  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalcomponent_count_components(component,kind);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_count_components. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _arg1 = (icalcomponent_kind )SvIV(ST(1));
    _result = (int )icalcomponent_count_components(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_end_component) {

    icalcompiter * _result;
    icalcomponent * _arg0;
    icalcomponent_kind  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalcomponent_end_component(component,kind);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_end_component. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _arg1 = (icalcomponent_kind )SvIV(ST(1));
    _result = (icalcompiter *) malloc(sizeof(icalcompiter ));
    *(_result) = icalcomponent_end_component(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalcompiterPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_get_current_component) {

    icalcomponent * _result;
    icalcomponent * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcomponent_get_current_component(component);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_get_current_component. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _result = (icalcomponent *)icalcomponent_get_current_component(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalcomponentPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_get_first_component) {

    icalcomponent * _result;
    icalcomponent * _arg0;
    icalcomponent_kind  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalcomponent_get_first_component(component,kind);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_get_first_component. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _arg1 = (icalcomponent_kind )SvIV(ST(1));
    _result = (icalcomponent *)icalcomponent_get_first_component(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalcomponentPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_get_next_component) {

    icalcomponent * _result;
    icalcomponent * _arg0;
    icalcomponent_kind  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalcomponent_get_next_component(component,kind);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_get_next_component. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _arg1 = (icalcomponent_kind )SvIV(ST(1));
    _result = (icalcomponent *)icalcomponent_get_next_component(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalcomponentPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_count_errors) {

    int  _result;
    icalcomponent * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcomponent_count_errors(component);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_count_errors. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _result = (int )icalcomponent_count_errors(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_strip_errors) {

    icalcomponent * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcomponent_strip_errors(component);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_strip_errors. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    icalcomponent_strip_errors(_arg0);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_convert_errors) {

    icalcomponent * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcomponent_convert_errors(component);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_convert_errors. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    icalcomponent_convert_errors(_arg0);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_get_parent) {

    icalcomponent * _result;
    icalcomponent * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcomponent_get_parent(component);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_get_parent. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _result = (icalcomponent *)icalcomponent_get_parent(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalcomponentPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_set_parent) {

    icalcomponent * _arg0;
    icalcomponent * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalcomponent_set_parent(component,parent);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_set_parent. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,(char *) 0 )) {
        croak("Type error in argument 2 of icalcomponent_set_parent. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    icalcomponent_set_parent(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalcompiter_next) {

    icalcomponent * _result;
    icalcompiter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcompiter_next(i);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"icalcompiterPtr")) {
        croak("Type error in argument 1 of icalcompiter_next. Expected icalcompiterPtr.");
        XSRETURN(1);
    }
    _result = (icalcomponent *)icalcompiter_next(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalcomponentPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcompiter_prior) {

    icalcomponent * _result;
    icalcompiter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcompiter_prior(i);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"icalcompiterPtr")) {
        croak("Type error in argument 1 of icalcompiter_prior. Expected icalcompiterPtr.");
        XSRETURN(1);
    }
    _result = (icalcomponent *)icalcompiter_prior(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalcomponentPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcompiter_deref) {

    icalcomponent * _result;
    icalcompiter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcompiter_deref(i);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"icalcompiterPtr")) {
        croak("Type error in argument 1 of icalcompiter_deref. Expected icalcompiterPtr.");
        XSRETURN(1);
    }
    _result = (icalcomponent *)icalcompiter_deref(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalcomponentPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_get_first_real_component) {

    icalcomponent * _result;
    icalcomponent * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcomponent_get_first_real_component(c);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_get_first_real_component. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _result = (icalcomponent *)icalcomponent_get_first_real_component(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalcomponentPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_get_span) {

    struct icaltime_span * _result;
    icalcomponent * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcomponent_get_span(comp);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_get_span. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _result = (struct icaltime_span *) malloc(sizeof(struct icaltime_span ));
    *(_result) = icalcomponent_get_span(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltime_spanPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_set_dtstart) {

    icalcomponent * _arg0;
    struct icaltimetype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalcomponent_set_dtstart(comp,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_set_dtstart. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icaltimetypePtr")) {
        croak("Type error in argument 2 of icalcomponent_set_dtstart. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    icalcomponent_set_dtstart(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_get_dtstart) {

    struct icaltimetype * _result;
    icalcomponent * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcomponent_get_dtstart(comp);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_get_dtstart. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _result = (struct icaltimetype *) malloc(sizeof(struct icaltimetype ));
    *(_result) = icalcomponent_get_dtstart(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_get_dtend) {

    struct icaltimetype * _result;
    icalcomponent * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcomponent_get_dtend(comp);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_get_dtend. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _result = (struct icaltimetype *) malloc(sizeof(struct icaltimetype ));
    *(_result) = icalcomponent_get_dtend(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_set_dtend) {

    icalcomponent * _arg0;
    struct icaltimetype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalcomponent_set_dtend(comp,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_set_dtend. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icaltimetypePtr")) {
        croak("Type error in argument 2 of icalcomponent_set_dtend. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    icalcomponent_set_dtend(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_set_duration) {

    icalcomponent * _arg0;
    struct icaldurationtype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalcomponent_set_duration(comp,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_set_duration. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icaldurationtypePtr")) {
        croak("Type error in argument 2 of icalcomponent_set_duration. Expected struct icaldurationtypePtr.");
        XSRETURN(1);
    }
    icalcomponent_set_duration(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_get_duration) {

    struct icaldurationtype * _result;
    icalcomponent * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcomponent_get_duration(comp);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_get_duration. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _result = (struct icaldurationtype *) malloc(sizeof(struct icaldurationtype ));
    *(_result) = icalcomponent_get_duration(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaldurationtypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_set_method) {

    icalcomponent * _arg0;
    icalproperty_method  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalcomponent_set_method(comp,method);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_set_method. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _arg1 = (icalproperty_method )SvIV(ST(1));
    icalcomponent_set_method(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_get_method) {

    icalproperty_method  _result;
    icalcomponent * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcomponent_get_method(comp);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_get_method. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _result = (icalproperty_method )icalcomponent_get_method(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_get_dtstamp) {

    struct icaltimetype * _result;
    icalcomponent * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcomponent_get_dtstamp(comp);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_get_dtstamp. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _result = (struct icaltimetype *) malloc(sizeof(struct icaltimetype ));
    *(_result) = icalcomponent_get_dtstamp(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_set_dtstamp) {

    icalcomponent * _arg0;
    struct icaltimetype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalcomponent_set_dtstamp(comp,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_set_dtstamp. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icaltimetypePtr")) {
        croak("Type error in argument 2 of icalcomponent_set_dtstamp. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    icalcomponent_set_dtstamp(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_set_summary) {

    icalcomponent * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalcomponent_set_summary(comp,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_set_summary. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalcomponent_set_summary(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_get_summary) {

    char * _result;
    icalcomponent * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcomponent_get_summary(comp);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_get_summary. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalcomponent_get_summary(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_set_comment) {

    icalcomponent * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalcomponent_set_comment(comp,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_set_comment. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalcomponent_set_comment(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_get_comment) {

    char * _result;
    icalcomponent * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcomponent_get_comment(comp);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_get_comment. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalcomponent_get_comment(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_set_organizer) {

    icalcomponent * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalcomponent_set_organizer(comp,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_set_organizer. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalcomponent_set_organizer(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_get_organizer) {

    char * _result;
    icalcomponent * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcomponent_get_organizer(comp);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_get_organizer. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalcomponent_get_organizer(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_set_uid) {

    icalcomponent * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalcomponent_set_uid(comp,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_set_uid. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    icalcomponent_set_uid(_arg0,_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_get_uid) {

    char * _result;
    icalcomponent * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcomponent_get_uid(comp);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_get_uid. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _result = (char *)icalcomponent_get_uid(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_set_recurrenceid) {

    icalcomponent * _arg0;
    struct icaltimetype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalcomponent_set_recurrenceid(comp,v);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_set_recurrenceid. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icaltimetypePtr")) {
        croak("Type error in argument 2 of icalcomponent_set_recurrenceid. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    icalcomponent_set_recurrenceid(_arg0,*_arg1);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_get_recurrenceid) {

    struct icaltimetype * _result;
    icalcomponent * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcomponent_get_recurrenceid(comp);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalcomponent_get_recurrenceid. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _result = (struct icaltimetype *) malloc(sizeof(struct icaltimetype ));
    *(_result) = icalcomponent_get_recurrenceid(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_new_vcalendar) {

    icalcomponent * _result;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 0) || (items > 0)) 
        croak("Usage: icalcomponent_new_vcalendar();");
    _result = (icalcomponent *)icalcomponent_new_vcalendar();
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalcomponentPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_new_vevent) {

    icalcomponent * _result;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 0) || (items > 0)) 
        croak("Usage: icalcomponent_new_vevent();");
    _result = (icalcomponent *)icalcomponent_new_vevent();
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalcomponentPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_new_vtodo) {

    icalcomponent * _result;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 0) || (items > 0)) 
        croak("Usage: icalcomponent_new_vtodo();");
    _result = (icalcomponent *)icalcomponent_new_vtodo();
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalcomponentPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_new_vjournal) {

    icalcomponent * _result;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 0) || (items > 0)) 
        croak("Usage: icalcomponent_new_vjournal();");
    _result = (icalcomponent *)icalcomponent_new_vjournal();
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalcomponentPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_new_vfreebusy) {

    icalcomponent * _result;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 0) || (items > 0)) 
        croak("Usage: icalcomponent_new_vfreebusy();");
    _result = (icalcomponent *)icalcomponent_new_vfreebusy();
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalcomponentPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_new_vtimezone) {

    icalcomponent * _result;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 0) || (items > 0)) 
        croak("Usage: icalcomponent_new_vtimezone();");
    _result = (icalcomponent *)icalcomponent_new_vtimezone();
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalcomponentPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_new_xstandard) {

    icalcomponent * _result;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 0) || (items > 0)) 
        croak("Usage: icalcomponent_new_xstandard();");
    _result = (icalcomponent *)icalcomponent_new_xstandard();
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalcomponentPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalcomponent_new_xdaylight) {

    icalcomponent * _result;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 0) || (items > 0)) 
        croak("Usage: icalcomponent_new_xdaylight();");
    _result = (icalcomponent *)icalcomponent_new_xdaylight();
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalcomponentPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparser_new) {

    icalparser * _result;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 0) || (items > 0)) 
        croak("Usage: icalparser_new();");
    _result = (icalparser *)icalparser_new();
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalparserPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparser_add_line) {

    icalcomponent * _result;
    icalparser * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalparser_add_line(parser,str);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparser_add_line. Expected icalparserPtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    _result = (icalcomponent *)icalparser_add_line(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalcomponentPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparser_claim) {

    icalcomponent * _result;
    icalparser * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparser_claim(parser);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparser_claim. Expected icalparserPtr.");
        XSRETURN(1);
    }
    _result = (icalcomponent *)icalparser_claim(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalcomponentPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparser_clean) {

    icalcomponent * _result;
    icalparser * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparser_clean(parser);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparser_clean. Expected icalparserPtr.");
        XSRETURN(1);
    }
    _result = (icalcomponent *)icalparser_clean(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalcomponentPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparser_get_state) {

    icalparser_state  _result;
    icalparser * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparser_get_state(parser);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparser_get_state. Expected icalparserPtr.");
        XSRETURN(1);
    }
    _result = (icalparser_state )icalparser_get_state(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparser_free) {

    icalparser * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparser_free(parser);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalparser_free. Expected icalparserPtr.");
        XSRETURN(1);
    }
    icalparser_free(_arg0);
    XSRETURN(argvi);
}

XS(_wrap_icalparser_parse_string) {

    icalcomponent * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalparser_parse_string(str);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (icalcomponent *)icalparser_parse_string(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalcomponentPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalparser_parse_value) {

    icalvalue * _result;
    icalvalue_kind  _arg0;
    char * _arg1;
    icalcomponent ** _arg2;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 3) || (items > 3)) 
        croak("Usage: icalparser_parse_value(kind,str,errors);");
    _arg0 = (icalvalue_kind )SvIV(ST(0));
    _arg1 = (char *) SvPV(ST(1),na);
    if (SWIG_GetPtr(ST(2),(void **) &_arg2,(char *) 0 )) {
        croak("Type error in argument 3 of icalparser_parse_value. Expected icalcomponentPtrPtr.");
        XSRETURN(1);
    }
    _result = (icalvalue *)icalparser_parse_value(_arg0,_arg1,_arg2);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"icalvaluePtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_string_line_generator) {

    char * _result;
    char * _arg0;
    size_t * _arg1;
    void * _arg2;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 3) || (items > 3)) 
        croak("Usage: string_line_generator(out,buf_size,d);");
    _arg0 = (char *) SvPV(ST(0),na);
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"size_tPtr")) {
        croak("Type error in argument 2 of string_line_generator. Expected size_tPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(2),(void **) &_arg2,(char *) 0 )) {
        croak("Type error in argument 3 of string_line_generator. Expected voidPtr.");
        XSRETURN(1);
    }
    _result = (char *)string_line_generator(_arg0,*_arg1,_arg2);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalmemory_tmp_buffer) {

    void * _result;
    size_t * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalmemory_tmp_buffer(size);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"size_tPtr")) {
        croak("Type error in argument 1 of icalmemory_tmp_buffer. Expected size_tPtr.");
        XSRETURN(1);
    }
    _result = (void *)icalmemory_tmp_buffer(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"voidPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalmemory_tmp_copy) {

    char * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalmemory_tmp_copy(str);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (char *)icalmemory_tmp_copy(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalmemory_add_tmp_buffer) {

    void * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalmemory_add_tmp_buffer(void *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalmemory_add_tmp_buffer. Expected voidPtr.");
        XSRETURN(1);
    }
    icalmemory_add_tmp_buffer(_arg0);
    XSRETURN(argvi);
}

XS(_wrap_icalmemory_free_ring) {

    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 0) || (items > 0)) 
        croak("Usage: icalmemory_free_ring();");
    icalmemory_free_ring();
    XSRETURN(argvi);
}

XS(_wrap_icalmemory_new_buffer) {

    void * _result;
    size_t * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalmemory_new_buffer(size);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"size_tPtr")) {
        croak("Type error in argument 1 of icalmemory_new_buffer. Expected size_tPtr.");
        XSRETURN(1);
    }
    _result = (void *)icalmemory_new_buffer(*_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"voidPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalmemory_resize_buffer) {

    void * _result;
    void * _arg0;
    size_t * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalmemory_resize_buffer(buf,size);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalmemory_resize_buffer. Expected voidPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"size_tPtr")) {
        croak("Type error in argument 2 of icalmemory_resize_buffer. Expected size_tPtr.");
        XSRETURN(1);
    }
    _result = (void *)icalmemory_resize_buffer(_arg0,*_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"voidPtr", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalmemory_free_buffer) {

    void * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalmemory_free_buffer(buf);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalmemory_free_buffer. Expected voidPtr.");
        XSRETURN(1);
    }
    icalmemory_free_buffer(_arg0);
    XSRETURN(argvi);
}

XS(_wrap_icalmemory_append_string) {

    char ** _arg0;
    char ** _arg1;
    size_t * _arg2;
    char * _arg3;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 4) || (items > 4)) 
        croak("Usage: icalmemory_append_string(buf,pos,buf_size,string);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"charPtrPtr")) {
        croak("Type error in argument 1 of icalmemory_append_string. Expected charPtrPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"charPtrPtr")) {
        croak("Type error in argument 2 of icalmemory_append_string. Expected charPtrPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(2),(void **) &_arg2,"size_tPtr")) {
        croak("Type error in argument 3 of icalmemory_append_string. Expected size_tPtr.");
        XSRETURN(1);
    }
    _arg3 = (char *) SvPV(ST(3),na);
    icalmemory_append_string(_arg0,_arg1,_arg2,_arg3);
    XSRETURN(argvi);
}

XS(_wrap_icalmemory_append_char) {

    char ** _arg0;
    char ** _arg1;
    size_t * _arg2;
    char  _arg3;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 4) || (items > 4)) 
        croak("Usage: icalmemory_append_char(buf,pos,buf_size,ch);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"charPtrPtr")) {
        croak("Type error in argument 1 of icalmemory_append_char. Expected charPtrPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"charPtrPtr")) {
        croak("Type error in argument 2 of icalmemory_append_char. Expected charPtrPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(2),(void **) &_arg2,"size_tPtr")) {
        croak("Type error in argument 3 of icalmemory_append_char. Expected size_tPtr.");
        XSRETURN(1);
    }
    _arg3 = (char) *SvPV(ST(3),na);
    icalmemory_append_char(_arg0,_arg1,_arg2,_arg3);
    XSRETURN(argvi);
}

XS(_wrap_icalmemory_strdup) {

    char * _result;
    char * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalmemory_strdup(s);");
    _arg0 = (char *) SvPV(ST(0),na);
    _result = (char *)icalmemory_strdup(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalerror_stop_here) {

    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 0) || (items > 0)) 
        croak("Usage: icalerror_stop_here();");
    icalerror_stop_here();
    XSRETURN(argvi);
}

XS(_wrap_icalerror_crash_here) {

    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 0) || (items > 0)) 
        croak("Usage: icalerror_crash_here();");
    icalerror_crash_here();
    XSRETURN(argvi);
}

XS(_wrap_icalerror_clear_errno) {

    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 0) || (items > 0)) 
        croak("Usage: icalerror_clear_errno();");
    icalerror_clear_errno();
    XSRETURN(argvi);
}

XS(_wrap_icalerror_set_errno) {

    icalerrorenum  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalerror_set_errno(icalerrorenum );");
    _arg0 = (icalerrorenum )SvIV(ST(0));
    icalerror_set_errno(_arg0);
    XSRETURN(argvi);
}

XS(_wrap_icalerror_strerror) {

    char * _result;
    icalerrorenum  _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalerror_strerror(e);");
    _arg0 = (icalerrorenum )SvIV(ST(0));
    _result = (char *)icalerror_strerror(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalrestriction_compare) {

    int  _result;
    icalrestriction_kind  _arg0;
    int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalrestriction_compare(restr,count);");
    _arg0 = (icalrestriction_kind )SvIV(ST(0));
    _arg1 = (int )SvIV(ST(1));
    _result = (int )icalrestriction_compare(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalrestriction_is_parameter_allowed) {

    int  _result;
    icalproperty_kind  _arg0;
    icalparameter_kind  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalrestriction_is_parameter_allowed(property,parameter);");
    _arg0 = (icalproperty_kind )SvIV(ST(0));
    _arg1 = (icalparameter_kind )SvIV(ST(1));
    _result = (int )icalrestriction_is_parameter_allowed(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

XS(_wrap_icalrestriction_check) {

    int  _result;
    icalcomponent * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalrestriction_check(comp);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,(char *) 0 )) {
        croak("Type error in argument 1 of icalrestriction_check. Expected icalcomponentPtr.");
        XSRETURN(1);
    }
    _result = (int )icalrestriction_check(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaltime_span_start_set(_swigobj,_swigval) (_swigobj->start = *(_swigval),_swigval)
XS(_wrap_icaltime_span_start_set) {

    time_t * _result;
    struct icaltime_span * _arg0;
    time_t * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icaltime_span_start_set(struct icaltime_span *,time_t *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltime_spanPtr")) {
        croak("Type error in argument 1 of icaltime_span_start_set. Expected struct icaltime_spanPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"time_tPtr")) {
        croak("Type error in argument 2 of icaltime_span_start_set. Expected time_tPtr.");
        XSRETURN(1);
    }
    _result = (time_t *)icaltime_span_start_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"time_tPtr", (void *) _result);
    XSRETURN(argvi);
}

#define icaltime_span_start_get(_swigobj) (&_swigobj->start)
XS(_wrap_icaltime_span_start_get) {

    time_t * _result;
    struct icaltime_span * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaltime_span_start_get(struct icaltime_span *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltime_spanPtr")) {
        croak("Type error in argument 1 of icaltime_span_start_get. Expected struct icaltime_spanPtr.");
        XSRETURN(1);
    }
    _result = (time_t *)icaltime_span_start_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"time_tPtr", (void *) _result);
    XSRETURN(argvi);
}

#define icaltime_span_end_set(_swigobj,_swigval) (_swigobj->end = *(_swigval),_swigval)
XS(_wrap_icaltime_span_end_set) {

    time_t * _result;
    struct icaltime_span * _arg0;
    time_t * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icaltime_span_end_set(struct icaltime_span *,time_t *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltime_spanPtr")) {
        croak("Type error in argument 1 of icaltime_span_end_set. Expected struct icaltime_spanPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"time_tPtr")) {
        croak("Type error in argument 2 of icaltime_span_end_set. Expected time_tPtr.");
        XSRETURN(1);
    }
    _result = (time_t *)icaltime_span_end_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"time_tPtr", (void *) _result);
    XSRETURN(argvi);
}

#define icaltime_span_end_get(_swigobj) (&_swigobj->end)
XS(_wrap_icaltime_span_end_get) {

    time_t * _result;
    struct icaltime_span * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaltime_span_end_get(struct icaltime_span *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltime_spanPtr")) {
        croak("Type error in argument 1 of icaltime_span_end_get. Expected struct icaltime_spanPtr.");
        XSRETURN(1);
    }
    _result = (time_t *)icaltime_span_end_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"time_tPtr", (void *) _result);
    XSRETURN(argvi);
}

#define icaltime_span_is_busy_set(_swigobj,_swigval) (_swigobj->is_busy = _swigval,_swigval)
XS(_wrap_icaltime_span_is_busy_set) {

    int  _result;
    struct icaltime_span * _arg0;
    int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icaltime_span_is_busy_set(struct icaltime_span *,int );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltime_spanPtr")) {
        croak("Type error in argument 1 of icaltime_span_is_busy_set. Expected struct icaltime_spanPtr.");
        XSRETURN(1);
    }
    _arg1 = (int )SvIV(ST(1));
    _result = (int )icaltime_span_is_busy_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaltime_span_is_busy_get(_swigobj) ((int ) _swigobj->is_busy)
XS(_wrap_icaltime_span_is_busy_get) {

    int  _result;
    struct icaltime_span * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaltime_span_is_busy_get(struct icaltime_span *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltime_spanPtr")) {
        croak("Type error in argument 1 of icaltime_span_is_busy_get. Expected struct icaltime_spanPtr.");
        XSRETURN(1);
    }
    _result = (int )icaltime_span_is_busy_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaltimetype_year_set(_swigobj,_swigval) (_swigobj->year = _swigval,_swigval)
XS(_wrap_icaltimetype_year_set) {

    int  _result;
    struct icaltimetype * _arg0;
    int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icaltimetype_year_set(struct icaltimetype *,int );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icaltimetype_year_set. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _arg1 = (int )SvIV(ST(1));
    _result = (int )icaltimetype_year_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaltimetype_year_get(_swigobj) ((int ) _swigobj->year)
XS(_wrap_icaltimetype_year_get) {

    int  _result;
    struct icaltimetype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaltimetype_year_get(struct icaltimetype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icaltimetype_year_get. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (int )icaltimetype_year_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaltimetype_month_set(_swigobj,_swigval) (_swigobj->month = _swigval,_swigval)
XS(_wrap_icaltimetype_month_set) {

    int  _result;
    struct icaltimetype * _arg0;
    int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icaltimetype_month_set(struct icaltimetype *,int );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icaltimetype_month_set. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _arg1 = (int )SvIV(ST(1));
    _result = (int )icaltimetype_month_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaltimetype_month_get(_swigobj) ((int ) _swigobj->month)
XS(_wrap_icaltimetype_month_get) {

    int  _result;
    struct icaltimetype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaltimetype_month_get(struct icaltimetype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icaltimetype_month_get. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (int )icaltimetype_month_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaltimetype_day_set(_swigobj,_swigval) (_swigobj->day = _swigval,_swigval)
XS(_wrap_icaltimetype_day_set) {

    int  _result;
    struct icaltimetype * _arg0;
    int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icaltimetype_day_set(struct icaltimetype *,int );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icaltimetype_day_set. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _arg1 = (int )SvIV(ST(1));
    _result = (int )icaltimetype_day_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaltimetype_day_get(_swigobj) ((int ) _swigobj->day)
XS(_wrap_icaltimetype_day_get) {

    int  _result;
    struct icaltimetype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaltimetype_day_get(struct icaltimetype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icaltimetype_day_get. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (int )icaltimetype_day_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaltimetype_hour_set(_swigobj,_swigval) (_swigobj->hour = _swigval,_swigval)
XS(_wrap_icaltimetype_hour_set) {

    int  _result;
    struct icaltimetype * _arg0;
    int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icaltimetype_hour_set(struct icaltimetype *,int );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icaltimetype_hour_set. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _arg1 = (int )SvIV(ST(1));
    _result = (int )icaltimetype_hour_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaltimetype_hour_get(_swigobj) ((int ) _swigobj->hour)
XS(_wrap_icaltimetype_hour_get) {

    int  _result;
    struct icaltimetype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaltimetype_hour_get(struct icaltimetype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icaltimetype_hour_get. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (int )icaltimetype_hour_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaltimetype_minute_set(_swigobj,_swigval) (_swigobj->minute = _swigval,_swigval)
XS(_wrap_icaltimetype_minute_set) {

    int  _result;
    struct icaltimetype * _arg0;
    int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icaltimetype_minute_set(struct icaltimetype *,int );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icaltimetype_minute_set. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _arg1 = (int )SvIV(ST(1));
    _result = (int )icaltimetype_minute_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaltimetype_minute_get(_swigobj) ((int ) _swigobj->minute)
XS(_wrap_icaltimetype_minute_get) {

    int  _result;
    struct icaltimetype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaltimetype_minute_get(struct icaltimetype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icaltimetype_minute_get. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (int )icaltimetype_minute_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaltimetype_second_set(_swigobj,_swigval) (_swigobj->second = _swigval,_swigval)
XS(_wrap_icaltimetype_second_set) {

    int  _result;
    struct icaltimetype * _arg0;
    int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icaltimetype_second_set(struct icaltimetype *,int );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icaltimetype_second_set. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _arg1 = (int )SvIV(ST(1));
    _result = (int )icaltimetype_second_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaltimetype_second_get(_swigobj) ((int ) _swigobj->second)
XS(_wrap_icaltimetype_second_get) {

    int  _result;
    struct icaltimetype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaltimetype_second_get(struct icaltimetype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icaltimetype_second_get. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (int )icaltimetype_second_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaltimetype_is_utc_set(_swigobj,_swigval) (_swigobj->is_utc = _swigval,_swigval)
XS(_wrap_icaltimetype_is_utc_set) {

    int  _result;
    struct icaltimetype * _arg0;
    int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icaltimetype_is_utc_set(struct icaltimetype *,int );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icaltimetype_is_utc_set. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _arg1 = (int )SvIV(ST(1));
    _result = (int )icaltimetype_is_utc_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaltimetype_is_utc_get(_swigobj) ((int ) _swigobj->is_utc)
XS(_wrap_icaltimetype_is_utc_get) {

    int  _result;
    struct icaltimetype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaltimetype_is_utc_get(struct icaltimetype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icaltimetype_is_utc_get. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (int )icaltimetype_is_utc_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaltimetype_is_date_set(_swigobj,_swigval) (_swigobj->is_date = _swigval,_swigval)
XS(_wrap_icaltimetype_is_date_set) {

    int  _result;
    struct icaltimetype * _arg0;
    int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icaltimetype_is_date_set(struct icaltimetype *,int );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icaltimetype_is_date_set. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _arg1 = (int )SvIV(ST(1));
    _result = (int )icaltimetype_is_date_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaltimetype_is_date_get(_swigobj) ((int ) _swigobj->is_date)
XS(_wrap_icaltimetype_is_date_get) {

    int  _result;
    struct icaltimetype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaltimetype_is_date_get(struct icaltimetype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaltimetypePtr")) {
        croak("Type error in argument 1 of icaltimetype_is_date_get. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (int )icaltimetype_is_date_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaldurationtype_is_neg_set(_swigobj,_swigval) (_swigobj->is_neg = _swigval,_swigval)
XS(_wrap_icaldurationtype_is_neg_set) {

    int  _result;
    struct icaldurationtype * _arg0;
    int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icaldurationtype_is_neg_set(struct icaldurationtype *,int );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaldurationtypePtr")) {
        croak("Type error in argument 1 of icaldurationtype_is_neg_set. Expected struct icaldurationtypePtr.");
        XSRETURN(1);
    }
    _arg1 = (int )SvIV(ST(1));
    _result = (int )icaldurationtype_is_neg_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaldurationtype_is_neg_get(_swigobj) ((int ) _swigobj->is_neg)
XS(_wrap_icaldurationtype_is_neg_get) {

    int  _result;
    struct icaldurationtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaldurationtype_is_neg_get(struct icaldurationtype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaldurationtypePtr")) {
        croak("Type error in argument 1 of icaldurationtype_is_neg_get. Expected struct icaldurationtypePtr.");
        XSRETURN(1);
    }
    _result = (int )icaldurationtype_is_neg_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaldurationtype_days_set(_swigobj,_swigval) (_swigobj->days = _swigval,_swigval)
XS(_wrap_icaldurationtype_days_set) {

    unsigned int  _result;
    struct icaldurationtype * _arg0;
    unsigned int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icaldurationtype_days_set(struct icaldurationtype *,unsigned int );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaldurationtypePtr")) {
        croak("Type error in argument 1 of icaldurationtype_days_set. Expected struct icaldurationtypePtr.");
        XSRETURN(1);
    }
    _arg1 = (unsigned int )SvIV(ST(1));
    _result = (unsigned int )icaldurationtype_days_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaldurationtype_days_get(_swigobj) ((unsigned int ) _swigobj->days)
XS(_wrap_icaldurationtype_days_get) {

    unsigned int  _result;
    struct icaldurationtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaldurationtype_days_get(struct icaldurationtype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaldurationtypePtr")) {
        croak("Type error in argument 1 of icaldurationtype_days_get. Expected struct icaldurationtypePtr.");
        XSRETURN(1);
    }
    _result = (unsigned int )icaldurationtype_days_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaldurationtype_weeks_set(_swigobj,_swigval) (_swigobj->weeks = _swigval,_swigval)
XS(_wrap_icaldurationtype_weeks_set) {

    unsigned int  _result;
    struct icaldurationtype * _arg0;
    unsigned int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icaldurationtype_weeks_set(struct icaldurationtype *,unsigned int );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaldurationtypePtr")) {
        croak("Type error in argument 1 of icaldurationtype_weeks_set. Expected struct icaldurationtypePtr.");
        XSRETURN(1);
    }
    _arg1 = (unsigned int )SvIV(ST(1));
    _result = (unsigned int )icaldurationtype_weeks_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaldurationtype_weeks_get(_swigobj) ((unsigned int ) _swigobj->weeks)
XS(_wrap_icaldurationtype_weeks_get) {

    unsigned int  _result;
    struct icaldurationtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaldurationtype_weeks_get(struct icaldurationtype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaldurationtypePtr")) {
        croak("Type error in argument 1 of icaldurationtype_weeks_get. Expected struct icaldurationtypePtr.");
        XSRETURN(1);
    }
    _result = (unsigned int )icaldurationtype_weeks_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaldurationtype_hours_set(_swigobj,_swigval) (_swigobj->hours = _swigval,_swigval)
XS(_wrap_icaldurationtype_hours_set) {

    unsigned int  _result;
    struct icaldurationtype * _arg0;
    unsigned int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icaldurationtype_hours_set(struct icaldurationtype *,unsigned int );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaldurationtypePtr")) {
        croak("Type error in argument 1 of icaldurationtype_hours_set. Expected struct icaldurationtypePtr.");
        XSRETURN(1);
    }
    _arg1 = (unsigned int )SvIV(ST(1));
    _result = (unsigned int )icaldurationtype_hours_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaldurationtype_hours_get(_swigobj) ((unsigned int ) _swigobj->hours)
XS(_wrap_icaldurationtype_hours_get) {

    unsigned int  _result;
    struct icaldurationtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaldurationtype_hours_get(struct icaldurationtype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaldurationtypePtr")) {
        croak("Type error in argument 1 of icaldurationtype_hours_get. Expected struct icaldurationtypePtr.");
        XSRETURN(1);
    }
    _result = (unsigned int )icaldurationtype_hours_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaldurationtype_minutes_set(_swigobj,_swigval) (_swigobj->minutes = _swigval,_swigval)
XS(_wrap_icaldurationtype_minutes_set) {

    unsigned int  _result;
    struct icaldurationtype * _arg0;
    unsigned int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icaldurationtype_minutes_set(struct icaldurationtype *,unsigned int );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaldurationtypePtr")) {
        croak("Type error in argument 1 of icaldurationtype_minutes_set. Expected struct icaldurationtypePtr.");
        XSRETURN(1);
    }
    _arg1 = (unsigned int )SvIV(ST(1));
    _result = (unsigned int )icaldurationtype_minutes_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaldurationtype_minutes_get(_swigobj) ((unsigned int ) _swigobj->minutes)
XS(_wrap_icaldurationtype_minutes_get) {

    unsigned int  _result;
    struct icaldurationtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaldurationtype_minutes_get(struct icaldurationtype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaldurationtypePtr")) {
        croak("Type error in argument 1 of icaldurationtype_minutes_get. Expected struct icaldurationtypePtr.");
        XSRETURN(1);
    }
    _result = (unsigned int )icaldurationtype_minutes_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaldurationtype_seconds_set(_swigobj,_swigval) (_swigobj->seconds = _swigval,_swigval)
XS(_wrap_icaldurationtype_seconds_set) {

    unsigned int  _result;
    struct icaldurationtype * _arg0;
    unsigned int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icaldurationtype_seconds_set(struct icaldurationtype *,unsigned int );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaldurationtypePtr")) {
        croak("Type error in argument 1 of icaldurationtype_seconds_set. Expected struct icaldurationtypePtr.");
        XSRETURN(1);
    }
    _arg1 = (unsigned int )SvIV(ST(1));
    _result = (unsigned int )icaldurationtype_seconds_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icaldurationtype_seconds_get(_swigobj) ((unsigned int ) _swigobj->seconds)
XS(_wrap_icaldurationtype_seconds_get) {

    unsigned int  _result;
    struct icaldurationtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaldurationtype_seconds_get(struct icaldurationtype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icaldurationtypePtr")) {
        croak("Type error in argument 1 of icaldurationtype_seconds_get. Expected struct icaldurationtypePtr.");
        XSRETURN(1);
    }
    _result = (unsigned int )icaldurationtype_seconds_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icalperiodtype_start_set(_swigobj,_swigval) (_swigobj->start = *(_swigval),_swigval)
XS(_wrap_icalperiodtype_start_set) {

    struct icaltimetype * _result;
    struct icalperiodtype * _arg0;
    struct icaltimetype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalperiodtype_start_set(struct icalperiodtype *,struct icaltimetype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalperiodtypePtr")) {
        croak("Type error in argument 1 of icalperiodtype_start_set. Expected struct icalperiodtypePtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icaltimetypePtr")) {
        croak("Type error in argument 2 of icalperiodtype_start_set. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (struct icaltimetype *)icalperiodtype_start_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

#define icalperiodtype_start_get(_swigobj) (&_swigobj->start)
XS(_wrap_icalperiodtype_start_get) {

    struct icaltimetype * _result;
    struct icalperiodtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalperiodtype_start_get(struct icalperiodtype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalperiodtypePtr")) {
        croak("Type error in argument 1 of icalperiodtype_start_get. Expected struct icalperiodtypePtr.");
        XSRETURN(1);
    }
    _result = (struct icaltimetype *)icalperiodtype_start_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

#define icalperiodtype_end_set(_swigobj,_swigval) (_swigobj->end = *(_swigval),_swigval)
XS(_wrap_icalperiodtype_end_set) {

    struct icaltimetype * _result;
    struct icalperiodtype * _arg0;
    struct icaltimetype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalperiodtype_end_set(struct icalperiodtype *,struct icaltimetype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalperiodtypePtr")) {
        croak("Type error in argument 1 of icalperiodtype_end_set. Expected struct icalperiodtypePtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icaltimetypePtr")) {
        croak("Type error in argument 2 of icalperiodtype_end_set. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (struct icaltimetype *)icalperiodtype_end_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

#define icalperiodtype_end_get(_swigobj) (&_swigobj->end)
XS(_wrap_icalperiodtype_end_get) {

    struct icaltimetype * _result;
    struct icalperiodtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalperiodtype_end_get(struct icalperiodtype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalperiodtypePtr")) {
        croak("Type error in argument 1 of icalperiodtype_end_get. Expected struct icalperiodtypePtr.");
        XSRETURN(1);
    }
    _result = (struct icaltimetype *)icalperiodtype_end_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

#define icalperiodtype_duration_set(_swigobj,_swigval) (_swigobj->duration = *(_swigval),_swigval)
XS(_wrap_icalperiodtype_duration_set) {

    struct icaldurationtype * _result;
    struct icalperiodtype * _arg0;
    struct icaldurationtype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalperiodtype_duration_set(struct icalperiodtype *,struct icaldurationtype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalperiodtypePtr")) {
        croak("Type error in argument 1 of icalperiodtype_duration_set. Expected struct icalperiodtypePtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icaldurationtypePtr")) {
        croak("Type error in argument 2 of icalperiodtype_duration_set. Expected struct icaldurationtypePtr.");
        XSRETURN(1);
    }
    _result = (struct icaldurationtype *)icalperiodtype_duration_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaldurationtypePtr", (void *) _result);
    XSRETURN(argvi);
}

#define icalperiodtype_duration_get(_swigobj) (&_swigobj->duration)
XS(_wrap_icalperiodtype_duration_get) {

    struct icaldurationtype * _result;
    struct icalperiodtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalperiodtype_duration_get(struct icalperiodtype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalperiodtypePtr")) {
        croak("Type error in argument 1 of icalperiodtype_duration_get. Expected struct icalperiodtypePtr.");
        XSRETURN(1);
    }
    _result = (struct icaldurationtype *)icalperiodtype_duration_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaldurationtypePtr", (void *) _result);
    XSRETURN(argvi);
}

#define icalattachtype_binary_set(_swigobj,_swigval) (_swigobj->binary = _swigval,_swigval)
XS(_wrap_icalattachtype_binary_set) {

    void * _result;
    struct icalattachtype * _arg0;
    void * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalattachtype_binary_set(struct icalattachtype *,void *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalattachtypePtr")) {
        croak("Type error in argument 1 of icalattachtype_binary_set. Expected struct icalattachtypePtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,(char *) 0 )) {
        croak("Type error in argument 2 of icalattachtype_binary_set. Expected voidPtr.");
        XSRETURN(1);
    }
    _result = (void *)icalattachtype_binary_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"voidPtr", (void *) _result);
    XSRETURN(argvi);
}

#define icalattachtype_binary_get(_swigobj) ((void *) _swigobj->binary)
XS(_wrap_icalattachtype_binary_get) {

    void * _result;
    struct icalattachtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalattachtype_binary_get(struct icalattachtype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalattachtypePtr")) {
        croak("Type error in argument 1 of icalattachtype_binary_get. Expected struct icalattachtypePtr.");
        XSRETURN(1);
    }
    _result = (void *)icalattachtype_binary_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"voidPtr", (void *) _result);
    XSRETURN(argvi);
}

#define icalattachtype_owns_binary_set(_swigobj,_swigval) (_swigobj->owns_binary = _swigval,_swigval)
XS(_wrap_icalattachtype_owns_binary_set) {

    int  _result;
    struct icalattachtype * _arg0;
    int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalattachtype_owns_binary_set(struct icalattachtype *,int );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalattachtypePtr")) {
        croak("Type error in argument 1 of icalattachtype_owns_binary_set. Expected struct icalattachtypePtr.");
        XSRETURN(1);
    }
    _arg1 = (int )SvIV(ST(1));
    _result = (int )icalattachtype_owns_binary_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icalattachtype_owns_binary_get(_swigobj) ((int ) _swigobj->owns_binary)
XS(_wrap_icalattachtype_owns_binary_get) {

    int  _result;
    struct icalattachtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalattachtype_owns_binary_get(struct icalattachtype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalattachtypePtr")) {
        croak("Type error in argument 1 of icalattachtype_owns_binary_get. Expected struct icalattachtypePtr.");
        XSRETURN(1);
    }
    _result = (int )icalattachtype_owns_binary_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

static char * icalattachtype_base64_set(struct icalattachtype *obj, char *val) {
    if (obj->base64) free(obj->base64);
    obj->base64 = (char *) malloc(strlen(val)+1);
    strcpy(obj->base64,val);
    return val;
}
XS(_wrap_icalattachtype_base64_set) {

    char * _result;
    struct icalattachtype * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalattachtype_base64_set(struct icalattachtype *,char *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalattachtypePtr")) {
        croak("Type error in argument 1 of icalattachtype_base64_set. Expected struct icalattachtypePtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    _result = (char *)icalattachtype_base64_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

#define icalattachtype_base64_get(_swigobj) ((char *) _swigobj->base64)
XS(_wrap_icalattachtype_base64_get) {

    char * _result;
    struct icalattachtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalattachtype_base64_get(struct icalattachtype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalattachtypePtr")) {
        croak("Type error in argument 1 of icalattachtype_base64_get. Expected struct icalattachtypePtr.");
        XSRETURN(1);
    }
    _result = (char *)icalattachtype_base64_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

#define icalattachtype_owns_base64_set(_swigobj,_swigval) (_swigobj->owns_base64 = _swigval,_swigval)
XS(_wrap_icalattachtype_owns_base64_set) {

    int  _result;
    struct icalattachtype * _arg0;
    int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalattachtype_owns_base64_set(struct icalattachtype *,int );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalattachtypePtr")) {
        croak("Type error in argument 1 of icalattachtype_owns_base64_set. Expected struct icalattachtypePtr.");
        XSRETURN(1);
    }
    _arg1 = (int )SvIV(ST(1));
    _result = (int )icalattachtype_owns_base64_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icalattachtype_owns_base64_get(_swigobj) ((int ) _swigobj->owns_base64)
XS(_wrap_icalattachtype_owns_base64_get) {

    int  _result;
    struct icalattachtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalattachtype_owns_base64_get(struct icalattachtype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalattachtypePtr")) {
        croak("Type error in argument 1 of icalattachtype_owns_base64_get. Expected struct icalattachtypePtr.");
        XSRETURN(1);
    }
    _result = (int )icalattachtype_owns_base64_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

static char * icalattachtype_url_set(struct icalattachtype *obj, char *val) {
    if (obj->url) free(obj->url);
    obj->url = (char *) malloc(strlen(val)+1);
    strcpy(obj->url,val);
    return val;
}
XS(_wrap_icalattachtype_url_set) {

    char * _result;
    struct icalattachtype * _arg0;
    char * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalattachtype_url_set(struct icalattachtype *,char *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalattachtypePtr")) {
        croak("Type error in argument 1 of icalattachtype_url_set. Expected struct icalattachtypePtr.");
        XSRETURN(1);
    }
    _arg1 = (char *) SvPV(ST(1),na);
    _result = (char *)icalattachtype_url_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

#define icalattachtype_url_get(_swigobj) ((char *) _swigobj->url)
XS(_wrap_icalattachtype_url_get) {

    char * _result;
    struct icalattachtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalattachtype_url_get(struct icalattachtype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalattachtypePtr")) {
        croak("Type error in argument 1 of icalattachtype_url_get. Expected struct icalattachtypePtr.");
        XSRETURN(1);
    }
    _result = (char *)icalattachtype_url_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setpv((SV*)ST(argvi++),(char *) _result);
    XSRETURN(argvi);
}

#define icalattachtype_refcount_set(_swigobj,_swigval) (_swigobj->refcount = _swigval,_swigval)
XS(_wrap_icalattachtype_refcount_set) {

    int  _result;
    struct icalattachtype * _arg0;
    int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalattachtype_refcount_set(struct icalattachtype *,int );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalattachtypePtr")) {
        croak("Type error in argument 1 of icalattachtype_refcount_set. Expected struct icalattachtypePtr.");
        XSRETURN(1);
    }
    _arg1 = (int )SvIV(ST(1));
    _result = (int )icalattachtype_refcount_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icalattachtype_refcount_get(_swigobj) ((int ) _swigobj->refcount)
XS(_wrap_icalattachtype_refcount_get) {

    int  _result;
    struct icalattachtype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalattachtype_refcount_get(struct icalattachtype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalattachtypePtr")) {
        croak("Type error in argument 1 of icalattachtype_refcount_get. Expected struct icalattachtypePtr.");
        XSRETURN(1);
    }
    _result = (int )icalattachtype_refcount_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icalgeotype_lat_set(_swigobj,_swigval) (_swigobj->lat = _swigval,_swigval)
XS(_wrap_icalgeotype_lat_set) {

    float  _result;
    struct icalgeotype * _arg0;
    float  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalgeotype_lat_set(struct icalgeotype *,float );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalgeotypePtr")) {
        croak("Type error in argument 1 of icalgeotype_lat_set. Expected struct icalgeotypePtr.");
        XSRETURN(1);
    }
    _arg1 = (float ) SvNV(ST(1));
    _result = (float )icalgeotype_lat_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setnv(ST(argvi++), (double) _result);
    XSRETURN(argvi);
}

#define icalgeotype_lat_get(_swigobj) ((float ) _swigobj->lat)
XS(_wrap_icalgeotype_lat_get) {

    float  _result;
    struct icalgeotype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalgeotype_lat_get(struct icalgeotype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalgeotypePtr")) {
        croak("Type error in argument 1 of icalgeotype_lat_get. Expected struct icalgeotypePtr.");
        XSRETURN(1);
    }
    _result = (float )icalgeotype_lat_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setnv(ST(argvi++), (double) _result);
    XSRETURN(argvi);
}

#define icalgeotype_lon_set(_swigobj,_swigval) (_swigobj->lon = _swigval,_swigval)
XS(_wrap_icalgeotype_lon_set) {

    float  _result;
    struct icalgeotype * _arg0;
    float  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalgeotype_lon_set(struct icalgeotype *,float );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalgeotypePtr")) {
        croak("Type error in argument 1 of icalgeotype_lon_set. Expected struct icalgeotypePtr.");
        XSRETURN(1);
    }
    _arg1 = (float ) SvNV(ST(1));
    _result = (float )icalgeotype_lon_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setnv(ST(argvi++), (double) _result);
    XSRETURN(argvi);
}

#define icalgeotype_lon_get(_swigobj) ((float ) _swigobj->lon)
XS(_wrap_icalgeotype_lon_get) {

    float  _result;
    struct icalgeotype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalgeotype_lon_get(struct icalgeotype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalgeotypePtr")) {
        croak("Type error in argument 1 of icalgeotype_lon_get. Expected struct icalgeotypePtr.");
        XSRETURN(1);
    }
    _result = (float )icalgeotype_lon_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setnv(ST(argvi++), (double) _result);
    XSRETURN(argvi);
}

#define icaltriggertype_time_set(_swigobj,_swigval) (_swigobj->time = *(_swigval),_swigval)
XS(_wrap_icaltriggertype_time_set) {

    struct icaltimetype * _result;
    union icaltriggertype * _arg0;
    struct icaltimetype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icaltriggertype_time_set(union icaltriggertype *,struct icaltimetype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"union icaltriggertypePtr")) {
        croak("Type error in argument 1 of icaltriggertype_time_set. Expected union icaltriggertypePtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icaltimetypePtr")) {
        croak("Type error in argument 2 of icaltriggertype_time_set. Expected struct icaltimetypePtr.");
        XSRETURN(1);
    }
    _result = (struct icaltimetype *)icaltriggertype_time_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

#define icaltriggertype_time_get(_swigobj) (&_swigobj->time)
XS(_wrap_icaltriggertype_time_get) {

    struct icaltimetype * _result;
    union icaltriggertype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaltriggertype_time_get(union icaltriggertype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"union icaltriggertypePtr")) {
        croak("Type error in argument 1 of icaltriggertype_time_get. Expected union icaltriggertypePtr.");
        XSRETURN(1);
    }
    _result = (struct icaltimetype *)icaltriggertype_time_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaltimetypePtr", (void *) _result);
    XSRETURN(argvi);
}

#define icaltriggertype_duration_set(_swigobj,_swigval) (_swigobj->duration = *(_swigval),_swigval)
XS(_wrap_icaltriggertype_duration_set) {

    struct icaldurationtype * _result;
    union icaltriggertype * _arg0;
    struct icaldurationtype * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icaltriggertype_duration_set(union icaltriggertype *,struct icaldurationtype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"union icaltriggertypePtr")) {
        croak("Type error in argument 1 of icaltriggertype_duration_set. Expected union icaltriggertypePtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct icaldurationtypePtr")) {
        croak("Type error in argument 2 of icaltriggertype_duration_set. Expected struct icaldurationtypePtr.");
        XSRETURN(1);
    }
    _result = (struct icaldurationtype *)icaltriggertype_duration_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaldurationtypePtr", (void *) _result);
    XSRETURN(argvi);
}

#define icaltriggertype_duration_get(_swigobj) (&_swigobj->duration)
XS(_wrap_icaltriggertype_duration_get) {

    struct icaldurationtype * _result;
    union icaltriggertype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icaltriggertype_duration_get(union icaltriggertype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"union icaltriggertypePtr")) {
        croak("Type error in argument 1 of icaltriggertype_duration_get. Expected union icaltriggertypePtr.");
        XSRETURN(1);
    }
    _result = (struct icaldurationtype *)icaltriggertype_duration_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct icaldurationtypePtr", (void *) _result);
    XSRETURN(argvi);
}

#define icalreqstattype_code_set(_swigobj,_swigval) (_swigobj->code = _swigval,_swigval)
XS(_wrap_icalreqstattype_code_set) {

    icalrequeststatus  _result;
    struct icalreqstattype * _arg0;
    icalrequeststatus  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalreqstattype_code_set(struct icalreqstattype *,icalrequeststatus );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalreqstattypePtr")) {
        croak("Type error in argument 1 of icalreqstattype_code_set. Expected struct icalreqstattypePtr.");
        XSRETURN(1);
    }
    _arg1 = (icalrequeststatus )SvIV(ST(1));
    _result = (icalrequeststatus )icalreqstattype_code_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icalreqstattype_code_get(_swigobj) ((icalrequeststatus ) _swigobj->code)
XS(_wrap_icalreqstattype_code_get) {

    icalrequeststatus  _result;
    struct icalreqstattype * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalreqstattype_code_get(struct icalreqstattype *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"struct icalreqstattypePtr")) {
        croak("Type error in argument 1 of icalreqstattype_code_get. Expected struct icalreqstattypePtr.");
        XSRETURN(1);
    }
    _result = (icalrequeststatus )icalreqstattype_code_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define pvl_elem_t_MAGIC_set(_swigobj,_swigval) (_swigobj->MAGIC = _swigval,_swigval)
XS(_wrap_pvl_elem_t_MAGIC_set) {

    int  _result;
    pvl_elem_t * _arg0;
    int  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: pvl_elem_t_MAGIC_set(pvl_elem_t *,int );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"pvl_elem_tPtr")) {
        croak("Type error in argument 1 of pvl_elem_t_MAGIC_set. Expected pvl_elem_tPtr.");
        XSRETURN(1);
    }
    _arg1 = (int )SvIV(ST(1));
    _result = (int )pvl_elem_t_MAGIC_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define pvl_elem_t_MAGIC_get(_swigobj) ((int ) _swigobj->MAGIC)
XS(_wrap_pvl_elem_t_MAGIC_get) {

    int  _result;
    pvl_elem_t * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: pvl_elem_t_MAGIC_get(pvl_elem_t *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"pvl_elem_tPtr")) {
        croak("Type error in argument 1 of pvl_elem_t_MAGIC_get. Expected pvl_elem_tPtr.");
        XSRETURN(1);
    }
    _result = (int )pvl_elem_t_MAGIC_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define pvl_elem_t_d_set(_swigobj,_swigval) (_swigobj->d = _swigval,_swigval)
XS(_wrap_pvl_elem_t_d_set) {

    void * _result;
    pvl_elem_t * _arg0;
    void * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: pvl_elem_t_d_set(pvl_elem_t *,void *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"pvl_elem_tPtr")) {
        croak("Type error in argument 1 of pvl_elem_t_d_set. Expected pvl_elem_tPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,(char *) 0 )) {
        croak("Type error in argument 2 of pvl_elem_t_d_set. Expected voidPtr.");
        XSRETURN(1);
    }
    _result = (void *)pvl_elem_t_d_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"voidPtr", (void *) _result);
    XSRETURN(argvi);
}

#define pvl_elem_t_d_get(_swigobj) ((void *) _swigobj->d)
XS(_wrap_pvl_elem_t_d_get) {

    void * _result;
    pvl_elem_t * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: pvl_elem_t_d_get(pvl_elem_t *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"pvl_elem_tPtr")) {
        croak("Type error in argument 1 of pvl_elem_t_d_get. Expected pvl_elem_tPtr.");
        XSRETURN(1);
    }
    _result = (void *)pvl_elem_t_d_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"voidPtr", (void *) _result);
    XSRETURN(argvi);
}

#define pvl_elem_t_next_set(_swigobj,_swigval) (_swigobj->next = _swigval,_swigval)
XS(_wrap_pvl_elem_t_next_set) {

    struct pvl_elem_t * _result;
    pvl_elem_t * _arg0;
    struct pvl_elem_t * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: pvl_elem_t_next_set(pvl_elem_t *,struct pvl_elem_t *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"pvl_elem_tPtr")) {
        croak("Type error in argument 1 of pvl_elem_t_next_set. Expected pvl_elem_tPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct pvl_elem_tPtr")) {
        croak("Type error in argument 2 of pvl_elem_t_next_set. Expected struct pvl_elem_tPtr.");
        XSRETURN(1);
    }
    _result = (struct pvl_elem_t *)pvl_elem_t_next_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct pvl_elem_tPtr", (void *) _result);
    XSRETURN(argvi);
}

#define pvl_elem_t_next_get(_swigobj) ((struct pvl_elem_t *) _swigobj->next)
XS(_wrap_pvl_elem_t_next_get) {

    struct pvl_elem_t * _result;
    pvl_elem_t * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: pvl_elem_t_next_get(pvl_elem_t *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"pvl_elem_tPtr")) {
        croak("Type error in argument 1 of pvl_elem_t_next_get. Expected pvl_elem_tPtr.");
        XSRETURN(1);
    }
    _result = (struct pvl_elem_t *)pvl_elem_t_next_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct pvl_elem_tPtr", (void *) _result);
    XSRETURN(argvi);
}

#define pvl_elem_t_prior_set(_swigobj,_swigval) (_swigobj->prior = _swigval,_swigval)
XS(_wrap_pvl_elem_t_prior_set) {

    struct pvl_elem_t * _result;
    pvl_elem_t * _arg0;
    struct pvl_elem_t * _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: pvl_elem_t_prior_set(pvl_elem_t *,struct pvl_elem_t *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"pvl_elem_tPtr")) {
        croak("Type error in argument 1 of pvl_elem_t_prior_set. Expected pvl_elem_tPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,"struct pvl_elem_tPtr")) {
        croak("Type error in argument 2 of pvl_elem_t_prior_set. Expected struct pvl_elem_tPtr.");
        XSRETURN(1);
    }
    _result = (struct pvl_elem_t *)pvl_elem_t_prior_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct pvl_elem_tPtr", (void *) _result);
    XSRETURN(argvi);
}

#define pvl_elem_t_prior_get(_swigobj) ((struct pvl_elem_t *) _swigobj->prior)
XS(_wrap_pvl_elem_t_prior_get) {

    struct pvl_elem_t * _result;
    pvl_elem_t * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: pvl_elem_t_prior_get(pvl_elem_t *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"pvl_elem_tPtr")) {
        croak("Type error in argument 1 of pvl_elem_t_prior_get. Expected pvl_elem_tPtr.");
        XSRETURN(1);
    }
    _result = (struct pvl_elem_t *)pvl_elem_t_prior_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"struct pvl_elem_tPtr", (void *) _result);
    XSRETURN(argvi);
}

#define icalcompiter_kind_set(_swigobj,_swigval) (_swigobj->kind = _swigval,_swigval)
XS(_wrap_icalcompiter_kind_set) {

    icalcomponent_kind  _result;
    icalcompiter * _arg0;
    icalcomponent_kind  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalcompiter_kind_set(icalcompiter *,icalcomponent_kind );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"icalcompiterPtr")) {
        croak("Type error in argument 1 of icalcompiter_kind_set. Expected icalcompiterPtr.");
        XSRETURN(1);
    }
    _arg1 = (icalcomponent_kind )SvIV(ST(1));
    _result = (icalcomponent_kind )icalcompiter_kind_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icalcompiter_kind_get(_swigobj) ((icalcomponent_kind ) _swigobj->kind)
XS(_wrap_icalcompiter_kind_get) {

    icalcomponent_kind  _result;
    icalcompiter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcompiter_kind_get(icalcompiter *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"icalcompiterPtr")) {
        croak("Type error in argument 1 of icalcompiter_kind_get. Expected icalcompiterPtr.");
        XSRETURN(1);
    }
    _result = (icalcomponent_kind )icalcompiter_kind_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setiv(ST(argvi++),(IV) _result);
    XSRETURN(argvi);
}

#define icalcompiter_iter_set(_swigobj,_swigval) (_swigobj->iter = _swigval,_swigval)
XS(_wrap_icalcompiter_iter_set) {

    pvl_elem  _result;
    icalcompiter * _arg0;
    pvl_elem  _arg1;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 2) || (items > 2)) 
        croak("Usage: icalcompiter_iter_set(icalcompiter *,pvl_elem );");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"icalcompiterPtr")) {
        croak("Type error in argument 1 of icalcompiter_iter_set. Expected icalcompiterPtr.");
        XSRETURN(1);
    }
    if (SWIG_GetPtr(ST(1),(void **) &_arg1,(char *) 0 )) {
        croak("Type error in argument 2 of icalcompiter_iter_set. Expected pvl_elem.");
        XSRETURN(1);
    }
    _result = (pvl_elem )icalcompiter_iter_set(_arg0,_arg1);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"pvl_elem", (void *) _result);
    XSRETURN(argvi);
}

#define icalcompiter_iter_get(_swigobj) ((pvl_elem ) _swigobj->iter)
XS(_wrap_icalcompiter_iter_get) {

    pvl_elem  _result;
    icalcompiter * _arg0;
    int argvi = 0;
    dXSARGS ;

    cv = cv;
    if ((items < 1) || (items > 1)) 
        croak("Usage: icalcompiter_iter_get(icalcompiter *);");
    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"icalcompiterPtr")) {
        croak("Type error in argument 1 of icalcompiter_iter_get. Expected icalcompiterPtr.");
        XSRETURN(1);
    }
    _result = (pvl_elem )icalcompiter_iter_get(_arg0);
    ST(argvi) = sv_newmortal();
    sv_setref_pv(ST(argvi++),"pvl_elem", (void *) _result);
    XSRETURN(argvi);
}

XS(_wrap_perl5_Net__ICal_var_init) {
    dXSARGS;
    SV *sv;
    cv = cv; items = items;
    swig_setiv("ICAL_NO_COMPONENT", (long) ICAL_NO_COMPONENT);
    swig_setiv("ICAL_ANY_COMPONENT", (long) ICAL_ANY_COMPONENT);
    swig_setiv("ICAL_XROOT_COMPONENT", (long) ICAL_XROOT_COMPONENT);
    swig_setiv("ICAL_XATTACH_COMPONENT", (long) ICAL_XATTACH_COMPONENT);
    swig_setiv("ICAL_VEVENT_COMPONENT", (long) ICAL_VEVENT_COMPONENT);
    swig_setiv("ICAL_VTODO_COMPONENT", (long) ICAL_VTODO_COMPONENT);
    swig_setiv("ICAL_VJOURNAL_COMPONENT", (long) ICAL_VJOURNAL_COMPONENT);
    swig_setiv("ICAL_VCALENDAR_COMPONENT", (long) ICAL_VCALENDAR_COMPONENT);
    swig_setiv("ICAL_VFREEBUSY_COMPONENT", (long) ICAL_VFREEBUSY_COMPONENT);
    swig_setiv("ICAL_VALARM_COMPONENT", (long) ICAL_VALARM_COMPONENT);
    swig_setiv("ICAL_XAUDIOALARM_COMPONENT", (long) ICAL_XAUDIOALARM_COMPONENT);
    swig_setiv("ICAL_XDISPLAYALARM_COMPONENT", (long) ICAL_XDISPLAYALARM_COMPONENT);
    swig_setiv("ICAL_XEMAILALARM_COMPONENT", (long) ICAL_XEMAILALARM_COMPONENT);
    swig_setiv("ICAL_XPROCEDUREALARM_COMPONENT", (long) ICAL_XPROCEDUREALARM_COMPONENT);
    swig_setiv("ICAL_VTIMEZONE_COMPONENT", (long) ICAL_VTIMEZONE_COMPONENT);
    swig_setiv("ICAL_XSTANDARD_COMPONENT", (long) ICAL_XSTANDARD_COMPONENT);
    swig_setiv("ICAL_XDAYLIGHT_COMPONENT", (long) ICAL_XDAYLIGHT_COMPONENT);
    swig_setiv("ICAL_X_COMPONENT", (long) ICAL_X_COMPONENT);
    swig_setiv("ICAL_VSCHEDULE_COMPONENT", (long) ICAL_VSCHEDULE_COMPONENT);
    swig_setiv("ICAL_VQUERY_COMPONENT", (long) ICAL_VQUERY_COMPONENT);
    swig_setiv("ICAL_VCAR_COMPONENT", (long) ICAL_VCAR_COMPONENT);
    swig_setiv("ICAL_VCOMMAND_COMPONENT", (long) ICAL_VCOMMAND_COMPONENT);
    swig_setiv("ICAL_XLICINVALID_COMPONENT", (long) ICAL_XLICINVALID_COMPONENT);
    swig_setiv("ICAL_ANY_PROPERTY", (long) ICAL_ANY_PROPERTY);
    swig_setiv("ICAL_CALSCALE_PROPERTY", (long) ICAL_CALSCALE_PROPERTY);
    swig_setiv("ICAL_METHOD_PROPERTY", (long) ICAL_METHOD_PROPERTY);
    swig_setiv("ICAL_PRODID_PROPERTY", (long) ICAL_PRODID_PROPERTY);
    swig_setiv("ICAL_VERSION_PROPERTY", (long) ICAL_VERSION_PROPERTY);
    swig_setiv("ICAL_ATTACH_PROPERTY", (long) ICAL_ATTACH_PROPERTY);
    swig_setiv("ICAL_CATEGORIES_PROPERTY", (long) ICAL_CATEGORIES_PROPERTY);
    swig_setiv("ICAL_CLASS_PROPERTY", (long) ICAL_CLASS_PROPERTY);
    swig_setiv("ICAL_COMMENT_PROPERTY", (long) ICAL_COMMENT_PROPERTY);
    swig_setiv("ICAL_DESCRIPTION_PROPERTY", (long) ICAL_DESCRIPTION_PROPERTY);
    swig_setiv("ICAL_GEO_PROPERTY", (long) ICAL_GEO_PROPERTY);
    swig_setiv("ICAL_LOCATION_PROPERTY", (long) ICAL_LOCATION_PROPERTY);
    swig_setiv("ICAL_PERCENTCOMPLETE_PROPERTY", (long) ICAL_PERCENTCOMPLETE_PROPERTY);
    swig_setiv("ICAL_PRIORITY_PROPERTY", (long) ICAL_PRIORITY_PROPERTY);
    swig_setiv("ICAL_RESOURCES_PROPERTY", (long) ICAL_RESOURCES_PROPERTY);
    swig_setiv("ICAL_STATUS_PROPERTY", (long) ICAL_STATUS_PROPERTY);
    swig_setiv("ICAL_SUMMARY_PROPERTY", (long) ICAL_SUMMARY_PROPERTY);
    swig_setiv("ICAL_COMPLETED_PROPERTY", (long) ICAL_COMPLETED_PROPERTY);
    swig_setiv("ICAL_DTEND_PROPERTY", (long) ICAL_DTEND_PROPERTY);
    swig_setiv("ICAL_DUE_PROPERTY", (long) ICAL_DUE_PROPERTY);
    swig_setiv("ICAL_DTSTART_PROPERTY", (long) ICAL_DTSTART_PROPERTY);
    swig_setiv("ICAL_DURATION_PROPERTY", (long) ICAL_DURATION_PROPERTY);
    swig_setiv("ICAL_FREEBUSY_PROPERTY", (long) ICAL_FREEBUSY_PROPERTY);
    swig_setiv("ICAL_TRANSP_PROPERTY", (long) ICAL_TRANSP_PROPERTY);
    swig_setiv("ICAL_TZID_PROPERTY", (long) ICAL_TZID_PROPERTY);
    swig_setiv("ICAL_TZNAME_PROPERTY", (long) ICAL_TZNAME_PROPERTY);
    swig_setiv("ICAL_TZOFFSETFROM_PROPERTY", (long) ICAL_TZOFFSETFROM_PROPERTY);
    swig_setiv("ICAL_TZOFFSETTO_PROPERTY", (long) ICAL_TZOFFSETTO_PROPERTY);
    swig_setiv("ICAL_TZURL_PROPERTY", (long) ICAL_TZURL_PROPERTY);
    swig_setiv("ICAL_ATTENDEE_PROPERTY", (long) ICAL_ATTENDEE_PROPERTY);
    swig_setiv("ICAL_CONTACT_PROPERTY", (long) ICAL_CONTACT_PROPERTY);
    swig_setiv("ICAL_ORGANIZER_PROPERTY", (long) ICAL_ORGANIZER_PROPERTY);
    swig_setiv("ICAL_RECURRENCEID_PROPERTY", (long) ICAL_RECURRENCEID_PROPERTY);
    swig_setiv("ICAL_RELATEDTO_PROPERTY", (long) ICAL_RELATEDTO_PROPERTY);
    swig_setiv("ICAL_URL_PROPERTY", (long) ICAL_URL_PROPERTY);
    swig_setiv("ICAL_UID_PROPERTY", (long) ICAL_UID_PROPERTY);
    swig_setiv("ICAL_EXDATE_PROPERTY", (long) ICAL_EXDATE_PROPERTY);
    swig_setiv("ICAL_EXRULE_PROPERTY", (long) ICAL_EXRULE_PROPERTY);
    swig_setiv("ICAL_RDATE_PROPERTY", (long) ICAL_RDATE_PROPERTY);
    swig_setiv("ICAL_RRULE_PROPERTY", (long) ICAL_RRULE_PROPERTY);
    swig_setiv("ICAL_ACTION_PROPERTY", (long) ICAL_ACTION_PROPERTY);
    swig_setiv("ICAL_REPEAT_PROPERTY", (long) ICAL_REPEAT_PROPERTY);
    swig_setiv("ICAL_TRIGGER_PROPERTY", (long) ICAL_TRIGGER_PROPERTY);
    swig_setiv("ICAL_CREATED_PROPERTY", (long) ICAL_CREATED_PROPERTY);
    swig_setiv("ICAL_DTSTAMP_PROPERTY", (long) ICAL_DTSTAMP_PROPERTY);
    swig_setiv("ICAL_LASTMODIFIED_PROPERTY", (long) ICAL_LASTMODIFIED_PROPERTY);
    swig_setiv("ICAL_SEQUENCE_PROPERTY", (long) ICAL_SEQUENCE_PROPERTY);
    swig_setiv("ICAL_REQUESTSTATUS_PROPERTY", (long) ICAL_REQUESTSTATUS_PROPERTY);
    swig_setiv("ICAL_X_PROPERTY", (long) ICAL_X_PROPERTY);
    swig_setiv("ICAL_SCOPE_PROPERTY", (long) ICAL_SCOPE_PROPERTY);
    swig_setiv("ICAL_MAXRESULTS_PROPERTY", (long) ICAL_MAXRESULTS_PROPERTY);
    swig_setiv("ICAL_MAXRESULTSSIZE_PROPERTY", (long) ICAL_MAXRESULTSSIZE_PROPERTY);
    swig_setiv("ICAL_QUERY_PROPERTY", (long) ICAL_QUERY_PROPERTY);
    swig_setiv("ICAL_QUERYNAME_PROPERTY", (long) ICAL_QUERYNAME_PROPERTY);
    swig_setiv("ICAL_TARGET_PROPERTY", (long) ICAL_TARGET_PROPERTY);
    swig_setiv("ICAL_XLICERROR_PROPERTY", (long) ICAL_XLICERROR_PROPERTY);
    swig_setiv("ICAL_XLICCLUSTERCOUNT_PROPERTY", (long) ICAL_XLICCLUSTERCOUNT_PROPERTY);
    swig_setiv("ICAL_XLICMIMECONTENTTYPE_PROPERTY", (long) ICAL_XLICMIMECONTENTTYPE_PROPERTY);
    swig_setiv("ICAL_XLICMIMEENCODING_PROPERTY", (long) ICAL_XLICMIMEENCODING_PROPERTY);
    swig_setiv("ICAL_XLICMIMECID_PROPERTY", (long) ICAL_XLICMIMECID_PROPERTY);
    swig_setiv("ICAL_XLICMIMEFILENAME_PROPERTY", (long) ICAL_XLICMIMEFILENAME_PROPERTY);
    swig_setiv("ICAL_XLICMIMECHARSET_PROPERTY", (long) ICAL_XLICMIMECHARSET_PROPERTY);
    swig_setiv("ICAL_XLICMIMEOPTINFO_PROPERTY", (long) ICAL_XLICMIMEOPTINFO_PROPERTY);
    swig_setiv("ICAL_NO_PROPERTY", (long) ICAL_NO_PROPERTY);
    swig_setiv("ICAL_METHOD_PUBLISH", (long) ICAL_METHOD_PUBLISH);
    swig_setiv("ICAL_METHOD_REQUEST", (long) ICAL_METHOD_REQUEST);
    swig_setiv("ICAL_METHOD_REPLY", (long) ICAL_METHOD_REPLY);
    swig_setiv("ICAL_METHOD_ADD", (long) ICAL_METHOD_ADD);
    swig_setiv("ICAL_METHOD_CANCEL", (long) ICAL_METHOD_CANCEL);
    swig_setiv("ICAL_METHOD_REFRESH", (long) ICAL_METHOD_REFRESH);
    swig_setiv("ICAL_METHOD_COUNTER", (long) ICAL_METHOD_COUNTER);
    swig_setiv("ICAL_METHOD_DECLINECOUNTER", (long) ICAL_METHOD_DECLINECOUNTER);
    swig_setiv("ICAL_METHOD_CREATE", (long) ICAL_METHOD_CREATE);
    swig_setiv("ICAL_METHOD_READ", (long) ICAL_METHOD_READ);
    swig_setiv("ICAL_METHOD_RESPONSE", (long) ICAL_METHOD_RESPONSE);
    swig_setiv("ICAL_METHOD_MOVE", (long) ICAL_METHOD_MOVE);
    swig_setiv("ICAL_METHOD_MODIFY", (long) ICAL_METHOD_MODIFY);
    swig_setiv("ICAL_METHOD_GENERATEUID", (long) ICAL_METHOD_GENERATEUID);
    swig_setiv("ICAL_METHOD_DELETE", (long) ICAL_METHOD_DELETE);
    swig_setiv("ICAL_METHOD_NONE", (long) ICAL_METHOD_NONE);
    swig_setiv("ICAL_TRANSP_OPAQUE", (long) ICAL_TRANSP_OPAQUE);
    swig_setiv("ICAL_TRANS_TRANSPARENT", (long) ICAL_TRANS_TRANSPARENT);
    swig_setiv("ICAL_CALSCALE_GREGORIAN", (long) ICAL_CALSCALE_GREGORIAN);
    swig_setiv("ICAL_CLASS_PUBLIC", (long) ICAL_CLASS_PUBLIC);
    swig_setiv("ICAL_CLASS_PRIVATE", (long) ICAL_CLASS_PRIVATE);
    swig_setiv("ICAL_CLASS_CONFIDENTIAL", (long) ICAL_CLASS_CONFIDENTIAL);
    swig_setiv("ICAL_CLASS_XNAME", (long) ICAL_CLASS_XNAME);
    swig_setiv("ICAL_STATUS_NONE", (long) ICAL_STATUS_NONE);
    swig_setiv("ICAL_STATUS_TENTATIVE", (long) ICAL_STATUS_TENTATIVE);
    swig_setiv("ICAL_STATUS_CONFIRMED", (long) ICAL_STATUS_CONFIRMED);
    swig_setiv("ICAL_STATUS_CANCELLED", (long) ICAL_STATUS_CANCELLED);
    swig_setiv("ICAL_STATUS_NEEDSACTION", (long) ICAL_STATUS_NEEDSACTION);
    swig_setiv("ICAL_STATUS_COMPLETED", (long) ICAL_STATUS_COMPLETED);
    swig_setiv("ICAL_STATUS_INPROCESS", (long) ICAL_STATUS_INPROCESS);
    swig_setiv("ICAL_STATUS_DRAFT", (long) ICAL_STATUS_DRAFT);
    swig_setiv("ICAL_STATUS_FINAL", (long) ICAL_STATUS_FINAL);
    swig_setiv("ICAL_ACTION_AUDIO", (long) ICAL_ACTION_AUDIO);
    swig_setiv("ICAL_ACTION_DISPLAY", (long) ICAL_ACTION_DISPLAY);
    swig_setiv("ICAL_ACTION_EMAIL", (long) ICAL_ACTION_EMAIL);
    swig_setiv("ICAL_ACTION_PROCEDURE", (long) ICAL_ACTION_PROCEDURE);
    swig_setiv("ICAL_ACTION_XNAME", (long) ICAL_ACTION_XNAME);
    swig_setiv("ICAL_NO_VALUE", (long) ICAL_NO_VALUE);
    swig_setiv("ICAL_ATTACH_VALUE", (long) ICAL_ATTACH_VALUE);
    swig_setiv("ICAL_BINARY_VALUE", (long) ICAL_BINARY_VALUE);
    swig_setiv("ICAL_BOOLEAN_VALUE", (long) ICAL_BOOLEAN_VALUE);
    swig_setiv("ICAL_CALADDRESS_VALUE", (long) ICAL_CALADDRESS_VALUE);
    swig_setiv("ICAL_DATE_VALUE", (long) ICAL_DATE_VALUE);
    swig_setiv("ICAL_DATETIME_VALUE", (long) ICAL_DATETIME_VALUE);
    swig_setiv("ICAL_DATETIMEDATE_VALUE", (long) ICAL_DATETIMEDATE_VALUE);
    swig_setiv("ICAL_DATETIMEPERIOD_VALUE", (long) ICAL_DATETIMEPERIOD_VALUE);
    swig_setiv("ICAL_DURATION_VALUE", (long) ICAL_DURATION_VALUE);
    swig_setiv("ICAL_FLOAT_VALUE", (long) ICAL_FLOAT_VALUE);
    swig_setiv("ICAL_GEO_VALUE", (long) ICAL_GEO_VALUE);
    swig_setiv("ICAL_INTEGER_VALUE", (long) ICAL_INTEGER_VALUE);
    swig_setiv("ICAL_METHOD_VALUE", (long) ICAL_METHOD_VALUE);
    swig_setiv("ICAL_STATUS_VALUE", (long) ICAL_STATUS_VALUE);
    swig_setiv("ICAL_PERIOD_VALUE", (long) ICAL_PERIOD_VALUE);
    swig_setiv("ICAL_RECUR_VALUE", (long) ICAL_RECUR_VALUE);
    swig_setiv("ICAL_STRING_VALUE", (long) ICAL_STRING_VALUE);
    swig_setiv("ICAL_TEXT_VALUE", (long) ICAL_TEXT_VALUE);
    swig_setiv("ICAL_TIME_VALUE", (long) ICAL_TIME_VALUE);
    swig_setiv("ICAL_TRIGGER_VALUE", (long) ICAL_TRIGGER_VALUE);
    swig_setiv("ICAL_URI_VALUE", (long) ICAL_URI_VALUE);
    swig_setiv("ICAL_UTCOFFSET_VALUE", (long) ICAL_UTCOFFSET_VALUE);
    swig_setiv("ICAL_QUERY_VALUE", (long) ICAL_QUERY_VALUE);
    swig_setiv("ICAL_XNAME_VALUE", (long) ICAL_XNAME_VALUE);
    swig_setiv("ICAL_NO_PARAMETER", (long) ICAL_NO_PARAMETER);
    swig_setiv("ICAL_ANY_PARAMETER", (long) ICAL_ANY_PARAMETER);
    swig_setiv("ICAL_ALTREP_PARAMETER", (long) ICAL_ALTREP_PARAMETER);
    swig_setiv("ICAL_CN_PARAMETER", (long) ICAL_CN_PARAMETER);
    swig_setiv("ICAL_CUTYPE_PARAMETER", (long) ICAL_CUTYPE_PARAMETER);
    swig_setiv("ICAL_DELEGATEDFROM_PARAMETER", (long) ICAL_DELEGATEDFROM_PARAMETER);
    swig_setiv("ICAL_DELEGATEDTO_PARAMETER", (long) ICAL_DELEGATEDTO_PARAMETER);
    swig_setiv("ICAL_DIR_PARAMETER", (long) ICAL_DIR_PARAMETER);
    swig_setiv("ICAL_ENCODING_PARAMETER", (long) ICAL_ENCODING_PARAMETER);
    swig_setiv("ICAL_FMTTYPE_PARAMETER", (long) ICAL_FMTTYPE_PARAMETER);
    swig_setiv("ICAL_FBTYPE_PARAMETER", (long) ICAL_FBTYPE_PARAMETER);
    swig_setiv("ICAL_LANGUAGE_PARAMETER", (long) ICAL_LANGUAGE_PARAMETER);
    swig_setiv("ICAL_MEMBER_PARAMETER", (long) ICAL_MEMBER_PARAMETER);
    swig_setiv("ICAL_PARTSTAT_PARAMETER", (long) ICAL_PARTSTAT_PARAMETER);
    swig_setiv("ICAL_RANGE_PARAMETER", (long) ICAL_RANGE_PARAMETER);
    swig_setiv("ICAL_RELATED_PARAMETER", (long) ICAL_RELATED_PARAMETER);
    swig_setiv("ICAL_RELTYPE_PARAMETER", (long) ICAL_RELTYPE_PARAMETER);
    swig_setiv("ICAL_ROLE_PARAMETER", (long) ICAL_ROLE_PARAMETER);
    swig_setiv("ICAL_RSVP_PARAMETER", (long) ICAL_RSVP_PARAMETER);
    swig_setiv("ICAL_SENTBY_PARAMETER", (long) ICAL_SENTBY_PARAMETER);
    swig_setiv("ICAL_TZID_PARAMETER", (long) ICAL_TZID_PARAMETER);
    swig_setiv("ICAL_VALUE_PARAMETER", (long) ICAL_VALUE_PARAMETER);
    swig_setiv("ICAL_XLICERRORTYPE_PARAMETER", (long) ICAL_XLICERRORTYPE_PARAMETER);
    swig_setiv("ICAL_XLICCOMPARETYPE_PARAMETER", (long) ICAL_XLICCOMPARETYPE_PARAMETER);
    swig_setiv("ICAL_X_PARAMETER", (long) ICAL_X_PARAMETER);
    swig_setiv("ICAL_CUTYPE_INDIVIDUAL", (long) ICAL_CUTYPE_INDIVIDUAL);
    swig_setiv("ICAL_CUTYPE_GROUP", (long) ICAL_CUTYPE_GROUP);
    swig_setiv("ICAL_CUTYPE_RESOURCE", (long) ICAL_CUTYPE_RESOURCE);
    swig_setiv("ICAL_CUTYPE_ROOM", (long) ICAL_CUTYPE_ROOM);
    swig_setiv("ICAL_CUTYPE_UNKNOWN", (long) ICAL_CUTYPE_UNKNOWN);
    swig_setiv("ICAL_CUTYPE_XNAME", (long) ICAL_CUTYPE_XNAME);
    swig_setiv("ICAL_ENCODING_8BIT", (long) ICAL_ENCODING_8BIT);
    swig_setiv("ICAL_ENCODING_BASE64", (long) ICAL_ENCODING_BASE64);
    swig_setiv("ICAL_ENCODING_XNAME", (long) ICAL_ENCODING_XNAME);
    swig_setiv("ICAL_FBTYPE_FREE", (long) ICAL_FBTYPE_FREE);
    swig_setiv("ICAL_FBTYPE_BUSY", (long) ICAL_FBTYPE_BUSY);
    swig_setiv("ICAL_FBTYPE_BUSYUNAVAILABLE", (long) ICAL_FBTYPE_BUSYUNAVAILABLE);
    swig_setiv("ICAL_FBTYPE_BUSYTENTATIVE", (long) ICAL_FBTYPE_BUSYTENTATIVE);
    swig_setiv("ICAL_FBTYPE_XNAME", (long) ICAL_FBTYPE_XNAME);
    swig_setiv("ICAL_PARTSTAT_NEEDSACTION", (long) ICAL_PARTSTAT_NEEDSACTION);
    swig_setiv("ICAL_PARTSTAT_ACCEPTED", (long) ICAL_PARTSTAT_ACCEPTED);
    swig_setiv("ICAL_PARTSTAT_DECLINED", (long) ICAL_PARTSTAT_DECLINED);
    swig_setiv("ICAL_PARTSTAT_TENTATIVE", (long) ICAL_PARTSTAT_TENTATIVE);
    swig_setiv("ICAL_PARTSTAT_DELEGATED", (long) ICAL_PARTSTAT_DELEGATED);
    swig_setiv("ICAL_PARTSTAT_COMPLETED", (long) ICAL_PARTSTAT_COMPLETED);
    swig_setiv("ICAL_PARTSTAT_INPROCESS", (long) ICAL_PARTSTAT_INPROCESS);
    swig_setiv("ICAL_PARTSTAT_XNAME", (long) ICAL_PARTSTAT_XNAME);
    swig_setiv("ICAL_PARTSTAT_NONE", (long) ICAL_PARTSTAT_NONE);
    swig_setiv("ICAL_RANGE_THISANDPRIOR", (long) ICAL_RANGE_THISANDPRIOR);
    swig_setiv("ICAL_RANGE_THISANDFUTURE", (long) ICAL_RANGE_THISANDFUTURE);
    swig_setiv("ICAL_RELATED_START", (long) ICAL_RELATED_START);
    swig_setiv("ICAL_RELATED_END", (long) ICAL_RELATED_END);
    swig_setiv("ICAL_RELTYPE_PARENT", (long) ICAL_RELTYPE_PARENT);
    swig_setiv("ICAL_RELTYPE_CHILD", (long) ICAL_RELTYPE_CHILD);
    swig_setiv("ICAL_RELTYPE_SIBLING", (long) ICAL_RELTYPE_SIBLING);
    swig_setiv("ICAL_RELTYPE_XNAME", (long) ICAL_RELTYPE_XNAME);
    swig_setiv("ICAL_ROLE_CHAIR", (long) ICAL_ROLE_CHAIR);
    swig_setiv("ICAL_ROLE_REQPARTICIPANT", (long) ICAL_ROLE_REQPARTICIPANT);
    swig_setiv("ICAL_ROLE_OPTPARTICIPANT", (long) ICAL_ROLE_OPTPARTICIPANT);
    swig_setiv("ICAL_ROLE_NONPARTICIPANT", (long) ICAL_ROLE_NONPARTICIPANT);
    swig_setiv("ICAL_ROLE_XNAME", (long) ICAL_ROLE_XNAME);
    swig_setiv("ICAL_XLICERRORTYPE_COMPONENTPARSEERROR", (long) ICAL_XLICERRORTYPE_COMPONENTPARSEERROR);
    swig_setiv("ICAL_XLICERRORTYPE_PARAMETERVALUEPARSEERROR", (long) ICAL_XLICERRORTYPE_PARAMETERVALUEPARSEERROR);
    swig_setiv("ICAL_XLICERRORTYPE_PARAMETERNAMEPARSEERROR", (long) ICAL_XLICERRORTYPE_PARAMETERNAMEPARSEERROR);
    swig_setiv("ICAL_XLICERRORTYPE_PROPERTYPARSEERROR", (long) ICAL_XLICERRORTYPE_PROPERTYPARSEERROR);
    swig_setiv("ICAL_XLICERRORTYPE_VALUEPARSEERROR", (long) ICAL_XLICERRORTYPE_VALUEPARSEERROR);
    swig_setiv("ICAL_XLICERRORTYPE_UNKVCALPROP", (long) ICAL_XLICERRORTYPE_UNKVCALPROP);
    swig_setiv("ICAL_XLICERRORTYPE_INVALIDITIP", (long) ICAL_XLICERRORTYPE_INVALIDITIP);
    swig_setiv("ICAL_XLICERRORTYPE_MIMEPARSEERROR", (long) ICAL_XLICERRORTYPE_MIMEPARSEERROR);
    swig_setiv("ICAL_XLICCOMPARETYPE_EQUAL", (long) ICAL_XLICCOMPARETYPE_EQUAL);
    swig_setiv("ICAL_XLICCOMPARETYPE_LESS", (long) ICAL_XLICCOMPARETYPE_LESS);
    swig_setiv("ICAL_XLICCOMPARETYPE_LESSEQUAL", (long) ICAL_XLICCOMPARETYPE_LESSEQUAL);
    swig_setiv("ICAL_XLICCOMPARETYPE_GREATER", (long) ICAL_XLICCOMPARETYPE_GREATER);
    swig_setiv("ICAL_XLICCOMPARETYPE_GREATEREQUAL", (long) ICAL_XLICCOMPARETYPE_GREATEREQUAL);
    swig_setiv("ICAL_XLICCOMPARETYPE_NOTEQUAL", (long) ICAL_XLICCOMPARETYPE_NOTEQUAL);
    swig_setiv("ICAL_XLICCOMPARETYPE_REGEX", (long) ICAL_XLICCOMPARETYPE_REGEX);
    swig_setiv("ICAL_VALUE_XNAME", (long) ICAL_VALUE_XNAME);
    swig_setiv("ICAL_VALUE_BINARY", (long) ICAL_VALUE_BINARY);
    swig_setiv("ICAL_VALUE_BOOLEAN", (long) ICAL_VALUE_BOOLEAN);
    swig_setiv("ICAL_VALUE_CALADDRESS", (long) ICAL_VALUE_CALADDRESS);
    swig_setiv("ICAL_VALUE_DATE", (long) ICAL_VALUE_DATE);
    swig_setiv("ICAL_VALUE_DATETIME", (long) ICAL_VALUE_DATETIME);
    swig_setiv("ICAL_VALUE_DURATION", (long) ICAL_VALUE_DURATION);
    swig_setiv("ICAL_VALUE_FLOAT", (long) ICAL_VALUE_FLOAT);
    swig_setiv("ICAL_VALUE_INTEGER", (long) ICAL_VALUE_INTEGER);
    swig_setiv("ICAL_VALUE_PERIOD", (long) ICAL_VALUE_PERIOD);
    swig_setiv("ICAL_VALUE_RECUR", (long) ICAL_VALUE_RECUR);
    swig_setiv("ICAL_VALUE_TEXT", (long) ICAL_VALUE_TEXT);
    swig_setiv("ICAL_VALUE_TIME", (long) ICAL_VALUE_TIME);
    swig_setiv("ICAL_VALUE_UTCOFFSET", (long) ICAL_VALUE_UTCOFFSET);
    swig_setiv("ICAL_VALUE_URI", (long) ICAL_VALUE_URI);
    swig_setiv("ICAL_VALUE_ERROR", (long) ICAL_VALUE_ERROR);
    swig_setiv("ICAL_UNKNOWN_STATUS", (long) ICAL_UNKNOWN_STATUS);
    swig_setiv("ICAL_2_0_SUCCESS_STATUS", (long) ICAL_2_0_SUCCESS_STATUS);
    swig_setiv("ICAL_2_1_FALLBACK_STATUS", (long) ICAL_2_1_FALLBACK_STATUS);
    swig_setiv("ICAL_2_2_IGPROP_STATUS", (long) ICAL_2_2_IGPROP_STATUS);
    swig_setiv("ICAL_2_3_IGPARAM_STATUS", (long) ICAL_2_3_IGPARAM_STATUS);
    swig_setiv("ICAL_2_4_IGXPROP_STATUS", (long) ICAL_2_4_IGXPROP_STATUS);
    swig_setiv("ICAL_2_5_IGXPARAM_STATUS", (long) ICAL_2_5_IGXPARAM_STATUS);
    swig_setiv("ICAL_2_6_IGCOMP_STATUS", (long) ICAL_2_6_IGCOMP_STATUS);
    swig_setiv("ICAL_2_7_FORWARD_STATUS", (long) ICAL_2_7_FORWARD_STATUS);
    swig_setiv("ICAL_2_8_ONEEVENT_STATUS", (long) ICAL_2_8_ONEEVENT_STATUS);
    swig_setiv("ICAL_2_9_TRUNC_STATUS", (long) ICAL_2_9_TRUNC_STATUS);
    swig_setiv("ICAL_2_10_ONETODO_STATUS", (long) ICAL_2_10_ONETODO_STATUS);
    swig_setiv("ICAL_2_11_TRUNCRRULE_STATUS", (long) ICAL_2_11_TRUNCRRULE_STATUS);
    swig_setiv("ICAL_3_0_INVPROPNAME_STATUS", (long) ICAL_3_0_INVPROPNAME_STATUS);
    swig_setiv("ICAL_3_1_INVPROPVAL_STATUS", (long) ICAL_3_1_INVPROPVAL_STATUS);
    swig_setiv("ICAL_3_2_INVPARAM_STATUS", (long) ICAL_3_2_INVPARAM_STATUS);
    swig_setiv("ICAL_3_3_INVPARAMVAL_STATUS", (long) ICAL_3_3_INVPARAMVAL_STATUS);
    swig_setiv("ICAL_3_4_INVCOMP_STATUS", (long) ICAL_3_4_INVCOMP_STATUS);
    swig_setiv("ICAL_3_5_INVTIME_STATUS", (long) ICAL_3_5_INVTIME_STATUS);
    swig_setiv("ICAL_3_6_INVRULE_STATUS", (long) ICAL_3_6_INVRULE_STATUS);
    swig_setiv("ICAL_3_7_INVCU_STATUS", (long) ICAL_3_7_INVCU_STATUS);
    swig_setiv("ICAL_3_8_NOAUTH_STATUS", (long) ICAL_3_8_NOAUTH_STATUS);
    swig_setiv("ICAL_3_9_BADVERSION_STATUS", (long) ICAL_3_9_BADVERSION_STATUS);
    swig_setiv("ICAL_3_10_TOOBIG_STATUS", (long) ICAL_3_10_TOOBIG_STATUS);
    swig_setiv("ICAL_3_11_MISSREQCOMP_STATUS", (long) ICAL_3_11_MISSREQCOMP_STATUS);
    swig_setiv("ICAL_3_12_UNKCOMP_STATUS", (long) ICAL_3_12_UNKCOMP_STATUS);
    swig_setiv("ICAL_3_13_BADCOMP_STATUS", (long) ICAL_3_13_BADCOMP_STATUS);
    swig_setiv("ICAL_3_14_NOCAP_STATUS", (long) ICAL_3_14_NOCAP_STATUS);
    swig_setiv("ICAL_4_0_BUSY_STATUS", (long) ICAL_4_0_BUSY_STATUS);
    swig_setiv("ICAL_5_0_MAYBE_STATUS", (long) ICAL_5_0_MAYBE_STATUS);
    swig_setiv("ICAL_5_1_UNAVAIL_STATUS", (long) ICAL_5_1_UNAVAIL_STATUS);
    swig_setiv("ICAL_5_2_NOSERVICE_STATUS", (long) ICAL_5_2_NOSERVICE_STATUS);
    swig_setiv("ICAL_5_3_NOSCHED_STATUS", (long) ICAL_5_3_NOSCHED_STATUS);
    swig_setiv("ICALPARSER_ERROR", (long) ICALPARSER_ERROR);
    swig_setiv("ICALPARSER_SUCCESS", (long) ICALPARSER_SUCCESS);
    swig_setiv("ICALPARSER_BEGIN_COMP", (long) ICALPARSER_BEGIN_COMP);
    swig_setiv("ICALPARSER_END_COMP", (long) ICALPARSER_END_COMP);
    swig_setiv("ICALPARSER_IN_PROGRESS", (long) ICALPARSER_IN_PROGRESS);
    swig_setiv("ICAL_BADARG_ERROR", (long) ICAL_BADARG_ERROR);
    swig_setiv("ICAL_NEWFAILED_ERROR", (long) ICAL_NEWFAILED_ERROR);
    swig_setiv("ICAL_MALFORMEDDATA_ERROR", (long) ICAL_MALFORMEDDATA_ERROR);
    swig_setiv("ICAL_PARSE_ERROR", (long) ICAL_PARSE_ERROR);
    swig_setiv("ICAL_INTERNAL_ERROR", (long) ICAL_INTERNAL_ERROR);
    swig_setiv("ICAL_FILE_ERROR", (long) ICAL_FILE_ERROR);
    swig_setiv("ICAL_ALLOCATION_ERROR", (long) ICAL_ALLOCATION_ERROR);
    swig_setiv("ICAL_USAGE_ERROR", (long) ICAL_USAGE_ERROR);
    swig_setiv("ICAL_NO_ERROR", (long) ICAL_NO_ERROR);
    swig_setiv("ICAL_MULTIPLEINCLUSION_ERROR", (long) ICAL_MULTIPLEINCLUSION_ERROR);
    swig_setiv("ICAL_TIMEDOUT_ERROR", (long) ICAL_TIMEDOUT_ERROR);
    swig_setiv("ICAL_UNKNOWN_ERROR", (long) ICAL_UNKNOWN_ERROR);
    swig_setiv("ICAL_RESTRICTION_NONE", (long) ICAL_RESTRICTION_NONE);
    swig_setiv("ICAL_RESTRICTION_ZERO", (long) ICAL_RESTRICTION_ZERO);
    swig_setiv("ICAL_RESTRICTION_ONE", (long) ICAL_RESTRICTION_ONE);
    swig_setiv("ICAL_RESTRICTION_ZEROPLUS", (long) ICAL_RESTRICTION_ZEROPLUS);
    swig_setiv("ICAL_RESTRICTION_ONEPLUS", (long) ICAL_RESTRICTION_ONEPLUS);
    swig_setiv("ICAL_RESTRICTION_ZEROORONE", (long) ICAL_RESTRICTION_ZEROORONE);
    swig_setiv("ICAL_RESTRICTION_ONEEXCLUSIVE", (long) ICAL_RESTRICTION_ONEEXCLUSIVE);
    swig_setiv("ICAL_RESTRICTION_ONEMUTUAL", (long) ICAL_RESTRICTION_ONEMUTUAL);
    swig_setiv("ICAL_RESTRICTION_UNKNOWN", (long) ICAL_RESTRICTION_UNKNOWN);
    swig_setiv("ICAL_RECURRENCE_ARRAY_MAX", (long) ICAL_RECURRENCE_ARRAY_MAX);
    swig_setiv("ICAL_RECURRENCE_ARRAY_MAX_BYTE", (long) ICAL_RECURRENCE_ARRAY_MAX_BYTE);
    swig_setpv("icalreqstattype_desc", "icalreqstattype::desc");
    swig_setpv("icalreqstattype_debug", "icalreqstattype::debug");
    XSRETURN(1);
}
#ifdef __cplusplus
extern "C"
#endif
XS(boot_Net__ICal) {
	 dXSARGS;
	 char *file = __FILE__;
	 cv = cv; items = items;
	 newXS("Net::ICal::var_Net__ICal_init", _wrap_perl5_Net__ICal_var_init, file);
	 newXS("Net::ICal::icaltime_null_time", _wrap_icaltime_null_time, file);
	 newXS("Net::ICal::icaltime_is_null_time", _wrap_icaltime_is_null_time, file);
	 newXS("Net::ICal::icaltime_normalize", _wrap_icaltime_normalize, file);
	 newXS("Net::ICal::icaltime_day_of_year", _wrap_icaltime_day_of_year, file);
	 newXS("Net::ICal::icaltime_from_day_of_year", _wrap_icaltime_from_day_of_year, file);
	 newXS("Net::ICal::icaltime_day_of_week", _wrap_icaltime_day_of_week, file);
	 newXS("Net::ICal::icaltime_start_doy_of_week", _wrap_icaltime_start_doy_of_week, file);
	 newXS("Net::ICal::icaltime_from_timet", _wrap_icaltime_from_timet, file);
	 newXS("Net::ICal::icaltime_from_string", _wrap_icaltime_from_string, file);
	 newXS("Net::ICal::icaltime_as_timet", _wrap_icaltime_as_timet, file);
	 newXS("Net::ICal::icaltime_as_ctime", _wrap_icaltime_as_ctime, file);
	 newXS("Net::ICal::icaltime_week_number", _wrap_icaltime_week_number, file);
	 newXS("Net::ICal::icaltime_from_week_number", _wrap_icaltime_from_week_number, file);
	 newXS("Net::ICal::icaltime_compare", _wrap_icaltime_compare, file);
	 newXS("Net::ICal::icaltime_days_in_month", _wrap_icaltime_days_in_month, file);
	 newXS("Net::ICal::icaltime_utc_offset", _wrap_icaltime_utc_offset, file);
	 newXS("Net::ICal::icaltime_local_utc_offset", _wrap_icaltime_local_utc_offset, file);
	 newXS("Net::ICal::icaltime_as_utc", _wrap_icaltime_as_utc, file);
	 newXS("Net::ICal::icaltime_as_zone", _wrap_icaltime_as_zone, file);
	 newXS("Net::ICal::icaldurationtype_from_timet", _wrap_icaldurationtype_from_timet, file);
	 newXS("Net::ICal::icaldurationtype_from_string", _wrap_icaldurationtype_from_string, file);
	 newXS("Net::ICal::icaldurationtype_as_timet", _wrap_icaldurationtype_as_timet, file);
	 newXS("Net::ICal::icalperiodtype_duration", _wrap_icalperiodtype_duration, file);
	 newXS("Net::ICal::icalperiodtype_end", _wrap_icalperiodtype_end, file);
	 newXS("Net::ICal::icaltime_add", _wrap_icaltime_add, file);
	 newXS("Net::ICal::icaltime_subtract", _wrap_icaltime_subtract, file);
	 newXS("Net::ICal::icalenum_reqstat_desc", _wrap_icalenum_reqstat_desc, file);
	 newXS("Net::ICal::icalenum_reqstat_major", _wrap_icalenum_reqstat_major, file);
	 newXS("Net::ICal::icalenum_reqstat_minor", _wrap_icalenum_reqstat_minor, file);
	 newXS("Net::ICal::icalenum_num_to_reqstat", _wrap_icalenum_num_to_reqstat, file);
	 newXS("Net::ICal::icalenum_property_kind_to_string", _wrap_icalenum_property_kind_to_string, file);
	 newXS("Net::ICal::icalenum_string_to_property_kind", _wrap_icalenum_string_to_property_kind, file);
	 newXS("Net::ICal::icalenum_value_kind_to_string", _wrap_icalenum_value_kind_to_string, file);
	 newXS("Net::ICal::icalenum_value_kind_by_prop", _wrap_icalenum_value_kind_by_prop, file);
	 newXS("Net::ICal::icalenum_parameter_kind_to_string", _wrap_icalenum_parameter_kind_to_string, file);
	 newXS("Net::ICal::icalenum_string_to_parameter_kind", _wrap_icalenum_string_to_parameter_kind, file);
	 newXS("Net::ICal::icalenum_component_kind_to_string", _wrap_icalenum_component_kind_to_string, file);
	 newXS("Net::ICal::icalenum_string_to_component_kind", _wrap_icalenum_string_to_component_kind, file);
	 newXS("Net::ICal::icalenum_property_kind_to_value_kind", _wrap_icalenum_property_kind_to_value_kind, file);
	 newXS("Net::ICal::icalenum_method_to_string", _wrap_icalenum_method_to_string, file);
	 newXS("Net::ICal::icalenum_string_to_method", _wrap_icalenum_string_to_method, file);
	 newXS("Net::ICal::icalenum_status_to_string", _wrap_icalenum_status_to_string, file);
	 newXS("Net::ICal::icalenum_string_to_status", _wrap_icalenum_string_to_status, file);
	 newXS("Net::ICal::icalattachtype_new", _wrap_icalattachtype_new, file);
	 newXS("Net::ICal::icalattachtype_add_reference", _wrap_icalattachtype_add_reference, file);
	 newXS("Net::ICal::icalattachtype_free", _wrap_icalattachtype_free, file);
	 newXS("Net::ICal::icalattachtype_set_url", _wrap_icalattachtype_set_url, file);
	 newXS("Net::ICal::icalattachtype_get_url", _wrap_icalattachtype_get_url, file);
	 newXS("Net::ICal::icalattachtype_set_base64", _wrap_icalattachtype_set_base64, file);
	 newXS("Net::ICal::icalattachtype_get_base64", _wrap_icalattachtype_get_base64, file);
	 newXS("Net::ICal::icalattachtype_set_binary", _wrap_icalattachtype_set_binary, file);
	 newXS("Net::ICal::icalattachtype_get_binary", _wrap_icalattachtype_get_binary, file);
	 newXS("Net::ICal::icalreqstattype_from_string", _wrap_icalreqstattype_from_string, file);
	 newXS("Net::ICal::icalreqstattype_as_string", _wrap_icalreqstattype_as_string, file);
	 newXS("Net::ICal::icalvalue_new", _wrap_icalvalue_new, file);
	 newXS("Net::ICal::icalvalue_new_clone", _wrap_icalvalue_new_clone, file);
	 newXS("Net::ICal::icalvalue_new_from_string", _wrap_icalvalue_new_from_string, file);
	 newXS("Net::ICal::icalvalue_free", _wrap_icalvalue_free, file);
	 newXS("Net::ICal::icalvalue_is_valid", _wrap_icalvalue_is_valid, file);
	 newXS("Net::ICal::icalvalue_as_ical_string", _wrap_icalvalue_as_ical_string, file);
	 newXS("Net::ICal::icalvalue_isa", _wrap_icalvalue_isa, file);
	 newXS("Net::ICal::icalvalue_isa_value", _wrap_icalvalue_isa_value, file);
	 newXS("Net::ICal::icalvalue_compare", _wrap_icalvalue_compare, file);
	 newXS("Net::ICal::icalvalue_new_attach", _wrap_icalvalue_new_attach, file);
	 newXS("Net::ICal::icalvalue_get_attach", _wrap_icalvalue_get_attach, file);
	 newXS("Net::ICal::icalvalue_set_attach", _wrap_icalvalue_set_attach, file);
	 newXS("Net::ICal::icalvalue_new_binary", _wrap_icalvalue_new_binary, file);
	 newXS("Net::ICal::icalvalue_get_binary", _wrap_icalvalue_get_binary, file);
	 newXS("Net::ICal::icalvalue_set_binary", _wrap_icalvalue_set_binary, file);
	 newXS("Net::ICal::icalvalue_new_boolean", _wrap_icalvalue_new_boolean, file);
	 newXS("Net::ICal::icalvalue_get_boolean", _wrap_icalvalue_get_boolean, file);
	 newXS("Net::ICal::icalvalue_set_boolean", _wrap_icalvalue_set_boolean, file);
	 newXS("Net::ICal::icalvalue_new_caladdress", _wrap_icalvalue_new_caladdress, file);
	 newXS("Net::ICal::icalvalue_get_caladdress", _wrap_icalvalue_get_caladdress, file);
	 newXS("Net::ICal::icalvalue_set_caladdress", _wrap_icalvalue_set_caladdress, file);
	 newXS("Net::ICal::icalvalue_new_date", _wrap_icalvalue_new_date, file);
	 newXS("Net::ICal::icalvalue_get_date", _wrap_icalvalue_get_date, file);
	 newXS("Net::ICal::icalvalue_set_date", _wrap_icalvalue_set_date, file);
	 newXS("Net::ICal::icalvalue_new_datetime", _wrap_icalvalue_new_datetime, file);
	 newXS("Net::ICal::icalvalue_get_datetime", _wrap_icalvalue_get_datetime, file);
	 newXS("Net::ICal::icalvalue_set_datetime", _wrap_icalvalue_set_datetime, file);
	 newXS("Net::ICal::icalvalue_new_datetimedate", _wrap_icalvalue_new_datetimedate, file);
	 newXS("Net::ICal::icalvalue_get_datetimedate", _wrap_icalvalue_get_datetimedate, file);
	 newXS("Net::ICal::icalvalue_set_datetimedate", _wrap_icalvalue_set_datetimedate, file);
	 newXS("Net::ICal::icalvalue_new_datetimeperiod", _wrap_icalvalue_new_datetimeperiod, file);
	 newXS("Net::ICal::icalvalue_get_datetimeperiod", _wrap_icalvalue_get_datetimeperiod, file);
	 newXS("Net::ICal::icalvalue_set_datetimeperiod", _wrap_icalvalue_set_datetimeperiod, file);
	 newXS("Net::ICal::icalvalue_new_duration", _wrap_icalvalue_new_duration, file);
	 newXS("Net::ICal::icalvalue_get_duration", _wrap_icalvalue_get_duration, file);
	 newXS("Net::ICal::icalvalue_set_duration", _wrap_icalvalue_set_duration, file);
	 newXS("Net::ICal::icalvalue_new_float", _wrap_icalvalue_new_float, file);
	 newXS("Net::ICal::icalvalue_get_float", _wrap_icalvalue_get_float, file);
	 newXS("Net::ICal::icalvalue_set_float", _wrap_icalvalue_set_float, file);
	 newXS("Net::ICal::icalvalue_new_geo", _wrap_icalvalue_new_geo, file);
	 newXS("Net::ICal::icalvalue_get_geo", _wrap_icalvalue_get_geo, file);
	 newXS("Net::ICal::icalvalue_set_geo", _wrap_icalvalue_set_geo, file);
	 newXS("Net::ICal::icalvalue_new_integer", _wrap_icalvalue_new_integer, file);
	 newXS("Net::ICal::icalvalue_get_integer", _wrap_icalvalue_get_integer, file);
	 newXS("Net::ICal::icalvalue_set_integer", _wrap_icalvalue_set_integer, file);
	 newXS("Net::ICal::icalvalue_new_method", _wrap_icalvalue_new_method, file);
	 newXS("Net::ICal::icalvalue_get_method", _wrap_icalvalue_get_method, file);
	 newXS("Net::ICal::icalvalue_set_method", _wrap_icalvalue_set_method, file);
	 newXS("Net::ICal::icalvalue_new_period", _wrap_icalvalue_new_period, file);
	 newXS("Net::ICal::icalvalue_get_period", _wrap_icalvalue_get_period, file);
	 newXS("Net::ICal::icalvalue_set_period", _wrap_icalvalue_set_period, file);
	 newXS("Net::ICal::icalvalue_new_string", _wrap_icalvalue_new_string, file);
	 newXS("Net::ICal::icalvalue_get_string", _wrap_icalvalue_get_string, file);
	 newXS("Net::ICal::icalvalue_set_string", _wrap_icalvalue_set_string, file);
	 newXS("Net::ICal::icalvalue_new_text", _wrap_icalvalue_new_text, file);
	 newXS("Net::ICal::icalvalue_get_text", _wrap_icalvalue_get_text, file);
	 newXS("Net::ICal::icalvalue_set_text", _wrap_icalvalue_set_text, file);
	 newXS("Net::ICal::icalvalue_new_time", _wrap_icalvalue_new_time, file);
	 newXS("Net::ICal::icalvalue_get_time", _wrap_icalvalue_get_time, file);
	 newXS("Net::ICal::icalvalue_set_time", _wrap_icalvalue_set_time, file);
	 newXS("Net::ICal::icalvalue_new_trigger", _wrap_icalvalue_new_trigger, file);
	 newXS("Net::ICal::icalvalue_get_trigger", _wrap_icalvalue_get_trigger, file);
	 newXS("Net::ICal::icalvalue_set_trigger", _wrap_icalvalue_set_trigger, file);
	 newXS("Net::ICal::icalvalue_new_uri", _wrap_icalvalue_new_uri, file);
	 newXS("Net::ICal::icalvalue_get_uri", _wrap_icalvalue_get_uri, file);
	 newXS("Net::ICal::icalvalue_set_uri", _wrap_icalvalue_set_uri, file);
	 newXS("Net::ICal::icalvalue_new_utcoffset", _wrap_icalvalue_new_utcoffset, file);
	 newXS("Net::ICal::icalvalue_get_utcoffset", _wrap_icalvalue_get_utcoffset, file);
	 newXS("Net::ICal::icalvalue_set_utcoffset", _wrap_icalvalue_set_utcoffset, file);
	 newXS("Net::ICal::icalvalue_new_query", _wrap_icalvalue_new_query, file);
	 newXS("Net::ICal::icalvalue_get_query", _wrap_icalvalue_get_query, file);
	 newXS("Net::ICal::icalvalue_set_query", _wrap_icalvalue_set_query, file);
	 newXS("Net::ICal::icalvalue_new_status", _wrap_icalvalue_new_status, file);
	 newXS("Net::ICal::icalvalue_get_status", _wrap_icalvalue_get_status, file);
	 newXS("Net::ICal::icalvalue_set_status", _wrap_icalvalue_set_status, file);
	 newXS("Net::ICal::icalparameter_new", _wrap_icalparameter_new, file);
	 newXS("Net::ICal::icalparameter_new_clone", _wrap_icalparameter_new_clone, file);
	 newXS("Net::ICal::icalparameter_new_from_string", _wrap_icalparameter_new_from_string, file);
	 newXS("Net::ICal::icalparameter_free", _wrap_icalparameter_free, file);
	 newXS("Net::ICal::icalparameter_as_ical_string", _wrap_icalparameter_as_ical_string, file);
	 newXS("Net::ICal::icalparameter_is_valid", _wrap_icalparameter_is_valid, file);
	 newXS("Net::ICal::icalparameter_isa", _wrap_icalparameter_isa, file);
	 newXS("Net::ICal::icalparameter_isa_parameter", _wrap_icalparameter_isa_parameter, file);
	 newXS("Net::ICal::icalparameter_set_xname", _wrap_icalparameter_set_xname, file);
	 newXS("Net::ICal::icalparameter_get_xname", _wrap_icalparameter_get_xname, file);
	 newXS("Net::ICal::icalparameter_set_xvalue", _wrap_icalparameter_set_xvalue, file);
	 newXS("Net::ICal::icalparameter_get_xvalue", _wrap_icalparameter_get_xvalue, file);
	 newXS("Net::ICal::icalparameter_new_altrep", _wrap_icalparameter_new_altrep, file);
	 newXS("Net::ICal::icalparameter_get_altrep", _wrap_icalparameter_get_altrep, file);
	 newXS("Net::ICal::icalparameter_set_altrep", _wrap_icalparameter_set_altrep, file);
	 newXS("Net::ICal::icalparameter_new_cn", _wrap_icalparameter_new_cn, file);
	 newXS("Net::ICal::icalparameter_get_cn", _wrap_icalparameter_get_cn, file);
	 newXS("Net::ICal::icalparameter_set_cn", _wrap_icalparameter_set_cn, file);
	 newXS("Net::ICal::icalparameter_new_cutype", _wrap_icalparameter_new_cutype, file);
	 newXS("Net::ICal::icalparameter_get_cutype", _wrap_icalparameter_get_cutype, file);
	 newXS("Net::ICal::icalparameter_set_cutype", _wrap_icalparameter_set_cutype, file);
	 newXS("Net::ICal::icalparameter_new_delegatedfrom", _wrap_icalparameter_new_delegatedfrom, file);
	 newXS("Net::ICal::icalparameter_get_delegatedfrom", _wrap_icalparameter_get_delegatedfrom, file);
	 newXS("Net::ICal::icalparameter_set_delegatedfrom", _wrap_icalparameter_set_delegatedfrom, file);
	 newXS("Net::ICal::icalparameter_new_delegatedto", _wrap_icalparameter_new_delegatedto, file);
	 newXS("Net::ICal::icalparameter_get_delegatedto", _wrap_icalparameter_get_delegatedto, file);
	 newXS("Net::ICal::icalparameter_set_delegatedto", _wrap_icalparameter_set_delegatedto, file);
	 newXS("Net::ICal::icalparameter_new_dir", _wrap_icalparameter_new_dir, file);
	 newXS("Net::ICal::icalparameter_get_dir", _wrap_icalparameter_get_dir, file);
	 newXS("Net::ICal::icalparameter_set_dir", _wrap_icalparameter_set_dir, file);
	 newXS("Net::ICal::icalparameter_new_encoding", _wrap_icalparameter_new_encoding, file);
	 newXS("Net::ICal::icalparameter_get_encoding", _wrap_icalparameter_get_encoding, file);
	 newXS("Net::ICal::icalparameter_set_encoding", _wrap_icalparameter_set_encoding, file);
	 newXS("Net::ICal::icalparameter_new_fbtype", _wrap_icalparameter_new_fbtype, file);
	 newXS("Net::ICal::icalparameter_get_fbtype", _wrap_icalparameter_get_fbtype, file);
	 newXS("Net::ICal::icalparameter_set_fbtype", _wrap_icalparameter_set_fbtype, file);
	 newXS("Net::ICal::icalparameter_new_fmttype", _wrap_icalparameter_new_fmttype, file);
	 newXS("Net::ICal::icalparameter_get_fmttype", _wrap_icalparameter_get_fmttype, file);
	 newXS("Net::ICal::icalparameter_set_fmttype", _wrap_icalparameter_set_fmttype, file);
	 newXS("Net::ICal::icalparameter_new_language", _wrap_icalparameter_new_language, file);
	 newXS("Net::ICal::icalparameter_get_language", _wrap_icalparameter_get_language, file);
	 newXS("Net::ICal::icalparameter_set_language", _wrap_icalparameter_set_language, file);
	 newXS("Net::ICal::icalparameter_new_member", _wrap_icalparameter_new_member, file);
	 newXS("Net::ICal::icalparameter_get_member", _wrap_icalparameter_get_member, file);
	 newXS("Net::ICal::icalparameter_set_member", _wrap_icalparameter_set_member, file);
	 newXS("Net::ICal::icalparameter_new_partstat", _wrap_icalparameter_new_partstat, file);
	 newXS("Net::ICal::icalparameter_get_partstat", _wrap_icalparameter_get_partstat, file);
	 newXS("Net::ICal::icalparameter_set_partstat", _wrap_icalparameter_set_partstat, file);
	 newXS("Net::ICal::icalparameter_new_range", _wrap_icalparameter_new_range, file);
	 newXS("Net::ICal::icalparameter_get_range", _wrap_icalparameter_get_range, file);
	 newXS("Net::ICal::icalparameter_set_range", _wrap_icalparameter_set_range, file);
	 newXS("Net::ICal::icalparameter_new_related", _wrap_icalparameter_new_related, file);
	 newXS("Net::ICal::icalparameter_get_related", _wrap_icalparameter_get_related, file);
	 newXS("Net::ICal::icalparameter_set_related", _wrap_icalparameter_set_related, file);
	 newXS("Net::ICal::icalparameter_new_reltype", _wrap_icalparameter_new_reltype, file);
	 newXS("Net::ICal::icalparameter_get_reltype", _wrap_icalparameter_get_reltype, file);
	 newXS("Net::ICal::icalparameter_set_reltype", _wrap_icalparameter_set_reltype, file);
	 newXS("Net::ICal::icalparameter_new_role", _wrap_icalparameter_new_role, file);
	 newXS("Net::ICal::icalparameter_get_role", _wrap_icalparameter_get_role, file);
	 newXS("Net::ICal::icalparameter_set_role", _wrap_icalparameter_set_role, file);
	 newXS("Net::ICal::icalparameter_new_rsvp", _wrap_icalparameter_new_rsvp, file);
	 newXS("Net::ICal::icalparameter_get_rsvp", _wrap_icalparameter_get_rsvp, file);
	 newXS("Net::ICal::icalparameter_set_rsvp", _wrap_icalparameter_set_rsvp, file);
	 newXS("Net::ICal::icalparameter_new_sentby", _wrap_icalparameter_new_sentby, file);
	 newXS("Net::ICal::icalparameter_get_sentby", _wrap_icalparameter_get_sentby, file);
	 newXS("Net::ICal::icalparameter_set_sentby", _wrap_icalparameter_set_sentby, file);
	 newXS("Net::ICal::icalparameter_new_tzid", _wrap_icalparameter_new_tzid, file);
	 newXS("Net::ICal::icalparameter_get_tzid", _wrap_icalparameter_get_tzid, file);
	 newXS("Net::ICal::icalparameter_set_tzid", _wrap_icalparameter_set_tzid, file);
	 newXS("Net::ICal::icalparameter_new_value", _wrap_icalparameter_new_value, file);
	 newXS("Net::ICal::icalparameter_get_value", _wrap_icalparameter_get_value, file);
	 newXS("Net::ICal::icalparameter_set_value", _wrap_icalparameter_set_value, file);
	 newXS("Net::ICal::icalparameter_new_x", _wrap_icalparameter_new_x, file);
	 newXS("Net::ICal::icalparameter_get_x", _wrap_icalparameter_get_x, file);
	 newXS("Net::ICal::icalparameter_set_x", _wrap_icalparameter_set_x, file);
	 newXS("Net::ICal::icalparameter_new_xlicerrortype", _wrap_icalparameter_new_xlicerrortype, file);
	 newXS("Net::ICal::icalparameter_get_xlicerrortype", _wrap_icalparameter_get_xlicerrortype, file);
	 newXS("Net::ICal::icalparameter_set_xlicerrortype", _wrap_icalparameter_set_xlicerrortype, file);
	 newXS("Net::ICal::icalparameter_new_xliccomparetype", _wrap_icalparameter_new_xliccomparetype, file);
	 newXS("Net::ICal::icalparameter_get_xliccomparetype", _wrap_icalparameter_get_xliccomparetype, file);
	 newXS("Net::ICal::icalparameter_set_xliccomparetype", _wrap_icalparameter_set_xliccomparetype, file);
	 newXS("Net::ICal::icalproperty_new", _wrap_icalproperty_new, file);
	 newXS("Net::ICal::icalproperty_new_clone", _wrap_icalproperty_new_clone, file);
	 newXS("Net::ICal::icalproperty_new_from_string", _wrap_icalproperty_new_from_string, file);
	 newXS("Net::ICal::icalproperty_as_ical_string", _wrap_icalproperty_as_ical_string, file);
	 newXS("Net::ICal::icalproperty_free", _wrap_icalproperty_free, file);
	 newXS("Net::ICal::icalproperty_isa", _wrap_icalproperty_isa, file);
	 newXS("Net::ICal::icalproperty_isa_property", _wrap_icalproperty_isa_property, file);
	 newXS("Net::ICal::icalproperty_add_parameter", _wrap_icalproperty_add_parameter, file);
	 newXS("Net::ICal::icalproperty_set_parameter", _wrap_icalproperty_set_parameter, file);
	 newXS("Net::ICal::icalproperty_remove_parameter", _wrap_icalproperty_remove_parameter, file);
	 newXS("Net::ICal::icalproperty_count_parameters", _wrap_icalproperty_count_parameters, file);
	 newXS("Net::ICal::icalproperty_get_first_parameter", _wrap_icalproperty_get_first_parameter, file);
	 newXS("Net::ICal::icalproperty_get_next_parameter", _wrap_icalproperty_get_next_parameter, file);
	 newXS("Net::ICal::icalproperty_set_value", _wrap_icalproperty_set_value, file);
	 newXS("Net::ICal::icalproperty_get_value", _wrap_icalproperty_get_value, file);
	 newXS("Net::ICal::icalproperty_set_x_name", _wrap_icalproperty_set_x_name, file);
	 newXS("Net::ICal::icalproperty_get_x_name", _wrap_icalproperty_get_x_name, file);
	 newXS("Net::ICal::icalproperty_new_method", _wrap_icalproperty_new_method, file);
	 newXS("Net::ICal::icalproperty_set_method", _wrap_icalproperty_set_method, file);
	 newXS("Net::ICal::icalproperty_get_method", _wrap_icalproperty_get_method, file);
	 newXS("Net::ICal::icalproperty_new_xlicmimecid", _wrap_icalproperty_new_xlicmimecid, file);
	 newXS("Net::ICal::icalproperty_set_xlicmimecid", _wrap_icalproperty_set_xlicmimecid, file);
	 newXS("Net::ICal::icalproperty_get_xlicmimecid", _wrap_icalproperty_get_xlicmimecid, file);
	 newXS("Net::ICal::icalproperty_new_lastmodified", _wrap_icalproperty_new_lastmodified, file);
	 newXS("Net::ICal::icalproperty_set_lastmodified", _wrap_icalproperty_set_lastmodified, file);
	 newXS("Net::ICal::icalproperty_get_lastmodified", _wrap_icalproperty_get_lastmodified, file);
	 newXS("Net::ICal::icalproperty_new_uid", _wrap_icalproperty_new_uid, file);
	 newXS("Net::ICal::icalproperty_set_uid", _wrap_icalproperty_set_uid, file);
	 newXS("Net::ICal::icalproperty_get_uid", _wrap_icalproperty_get_uid, file);
	 newXS("Net::ICal::icalproperty_new_prodid", _wrap_icalproperty_new_prodid, file);
	 newXS("Net::ICal::icalproperty_set_prodid", _wrap_icalproperty_set_prodid, file);
	 newXS("Net::ICal::icalproperty_get_prodid", _wrap_icalproperty_get_prodid, file);
	 newXS("Net::ICal::icalproperty_new_status", _wrap_icalproperty_new_status, file);
	 newXS("Net::ICal::icalproperty_set_status", _wrap_icalproperty_set_status, file);
	 newXS("Net::ICal::icalproperty_get_status", _wrap_icalproperty_get_status, file);
	 newXS("Net::ICal::icalproperty_new_description", _wrap_icalproperty_new_description, file);
	 newXS("Net::ICal::icalproperty_set_description", _wrap_icalproperty_set_description, file);
	 newXS("Net::ICal::icalproperty_get_description", _wrap_icalproperty_get_description, file);
	 newXS("Net::ICal::icalproperty_new_duration", _wrap_icalproperty_new_duration, file);
	 newXS("Net::ICal::icalproperty_set_duration", _wrap_icalproperty_set_duration, file);
	 newXS("Net::ICal::icalproperty_get_duration", _wrap_icalproperty_get_duration, file);
	 newXS("Net::ICal::icalproperty_new_categories", _wrap_icalproperty_new_categories, file);
	 newXS("Net::ICal::icalproperty_set_categories", _wrap_icalproperty_set_categories, file);
	 newXS("Net::ICal::icalproperty_get_categories", _wrap_icalproperty_get_categories, file);
	 newXS("Net::ICal::icalproperty_new_version", _wrap_icalproperty_new_version, file);
	 newXS("Net::ICal::icalproperty_set_version", _wrap_icalproperty_set_version, file);
	 newXS("Net::ICal::icalproperty_get_version", _wrap_icalproperty_get_version, file);
	 newXS("Net::ICal::icalproperty_new_tzoffsetfrom", _wrap_icalproperty_new_tzoffsetfrom, file);
	 newXS("Net::ICal::icalproperty_set_tzoffsetfrom", _wrap_icalproperty_set_tzoffsetfrom, file);
	 newXS("Net::ICal::icalproperty_get_tzoffsetfrom", _wrap_icalproperty_get_tzoffsetfrom, file);
	 newXS("Net::ICal::icalproperty_new_attendee", _wrap_icalproperty_new_attendee, file);
	 newXS("Net::ICal::icalproperty_set_attendee", _wrap_icalproperty_set_attendee, file);
	 newXS("Net::ICal::icalproperty_get_attendee", _wrap_icalproperty_get_attendee, file);
	 newXS("Net::ICal::icalproperty_new_contact", _wrap_icalproperty_new_contact, file);
	 newXS("Net::ICal::icalproperty_set_contact", _wrap_icalproperty_set_contact, file);
	 newXS("Net::ICal::icalproperty_get_contact", _wrap_icalproperty_get_contact, file);
	 newXS("Net::ICal::icalproperty_new_xlicmimecontenttype", _wrap_icalproperty_new_xlicmimecontenttype, file);
	 newXS("Net::ICal::icalproperty_set_xlicmimecontenttype", _wrap_icalproperty_set_xlicmimecontenttype, file);
	 newXS("Net::ICal::icalproperty_get_xlicmimecontenttype", _wrap_icalproperty_get_xlicmimecontenttype, file);
	 newXS("Net::ICal::icalproperty_new_xlicmimeoptinfo", _wrap_icalproperty_new_xlicmimeoptinfo, file);
	 newXS("Net::ICal::icalproperty_set_xlicmimeoptinfo", _wrap_icalproperty_set_xlicmimeoptinfo, file);
	 newXS("Net::ICal::icalproperty_get_xlicmimeoptinfo", _wrap_icalproperty_get_xlicmimeoptinfo, file);
	 newXS("Net::ICal::icalproperty_new_relatedto", _wrap_icalproperty_new_relatedto, file);
	 newXS("Net::ICal::icalproperty_set_relatedto", _wrap_icalproperty_set_relatedto, file);
	 newXS("Net::ICal::icalproperty_get_relatedto", _wrap_icalproperty_get_relatedto, file);
	 newXS("Net::ICal::icalproperty_new_organizer", _wrap_icalproperty_new_organizer, file);
	 newXS("Net::ICal::icalproperty_set_organizer", _wrap_icalproperty_set_organizer, file);
	 newXS("Net::ICal::icalproperty_get_organizer", _wrap_icalproperty_get_organizer, file);
	 newXS("Net::ICal::icalproperty_new_comment", _wrap_icalproperty_new_comment, file);
	 newXS("Net::ICal::icalproperty_set_comment", _wrap_icalproperty_set_comment, file);
	 newXS("Net::ICal::icalproperty_get_comment", _wrap_icalproperty_get_comment, file);
	 newXS("Net::ICal::icalproperty_new_xlicerror", _wrap_icalproperty_new_xlicerror, file);
	 newXS("Net::ICal::icalproperty_set_xlicerror", _wrap_icalproperty_set_xlicerror, file);
	 newXS("Net::ICal::icalproperty_get_xlicerror", _wrap_icalproperty_get_xlicerror, file);
	 newXS("Net::ICal::icalproperty_new_trigger", _wrap_icalproperty_new_trigger, file);
	 newXS("Net::ICal::icalproperty_set_trigger", _wrap_icalproperty_set_trigger, file);
	 newXS("Net::ICal::icalproperty_get_trigger", _wrap_icalproperty_get_trigger, file);
	 newXS("Net::ICal::icalproperty_new_class", _wrap_icalproperty_new_class, file);
	 newXS("Net::ICal::icalproperty_set_class", _wrap_icalproperty_set_class, file);
	 newXS("Net::ICal::icalproperty_get_class", _wrap_icalproperty_get_class, file);
	 newXS("Net::ICal::icalproperty_new_x", _wrap_icalproperty_new_x, file);
	 newXS("Net::ICal::icalproperty_set_x", _wrap_icalproperty_set_x, file);
	 newXS("Net::ICal::icalproperty_get_x", _wrap_icalproperty_get_x, file);
	 newXS("Net::ICal::icalproperty_new_tzoffsetto", _wrap_icalproperty_new_tzoffsetto, file);
	 newXS("Net::ICal::icalproperty_set_tzoffsetto", _wrap_icalproperty_set_tzoffsetto, file);
	 newXS("Net::ICal::icalproperty_get_tzoffsetto", _wrap_icalproperty_get_tzoffsetto, file);
	 newXS("Net::ICal::icalproperty_new_transp", _wrap_icalproperty_new_transp, file);
	 newXS("Net::ICal::icalproperty_set_transp", _wrap_icalproperty_set_transp, file);
	 newXS("Net::ICal::icalproperty_get_transp", _wrap_icalproperty_get_transp, file);
	 newXS("Net::ICal::icalproperty_new_xlicmimeencoding", _wrap_icalproperty_new_xlicmimeencoding, file);
	 newXS("Net::ICal::icalproperty_set_xlicmimeencoding", _wrap_icalproperty_set_xlicmimeencoding, file);
	 newXS("Net::ICal::icalproperty_get_xlicmimeencoding", _wrap_icalproperty_get_xlicmimeencoding, file);
	 newXS("Net::ICal::icalproperty_new_sequence", _wrap_icalproperty_new_sequence, file);
	 newXS("Net::ICal::icalproperty_set_sequence", _wrap_icalproperty_set_sequence, file);
	 newXS("Net::ICal::icalproperty_get_sequence", _wrap_icalproperty_get_sequence, file);
	 newXS("Net::ICal::icalproperty_new_location", _wrap_icalproperty_new_location, file);
	 newXS("Net::ICal::icalproperty_set_location", _wrap_icalproperty_set_location, file);
	 newXS("Net::ICal::icalproperty_get_location", _wrap_icalproperty_get_location, file);
	 newXS("Net::ICal::icalproperty_new_requeststatus", _wrap_icalproperty_new_requeststatus, file);
	 newXS("Net::ICal::icalproperty_set_requeststatus", _wrap_icalproperty_set_requeststatus, file);
	 newXS("Net::ICal::icalproperty_get_requeststatus", _wrap_icalproperty_get_requeststatus, file);
	 newXS("Net::ICal::icalproperty_new_exdate", _wrap_icalproperty_new_exdate, file);
	 newXS("Net::ICal::icalproperty_set_exdate", _wrap_icalproperty_set_exdate, file);
	 newXS("Net::ICal::icalproperty_get_exdate", _wrap_icalproperty_get_exdate, file);
	 newXS("Net::ICal::icalproperty_new_tzid", _wrap_icalproperty_new_tzid, file);
	 newXS("Net::ICal::icalproperty_set_tzid", _wrap_icalproperty_set_tzid, file);
	 newXS("Net::ICal::icalproperty_get_tzid", _wrap_icalproperty_get_tzid, file);
	 newXS("Net::ICal::icalproperty_new_resources", _wrap_icalproperty_new_resources, file);
	 newXS("Net::ICal::icalproperty_set_resources", _wrap_icalproperty_set_resources, file);
	 newXS("Net::ICal::icalproperty_get_resources", _wrap_icalproperty_get_resources, file);
	 newXS("Net::ICal::icalproperty_new_tzurl", _wrap_icalproperty_new_tzurl, file);
	 newXS("Net::ICal::icalproperty_set_tzurl", _wrap_icalproperty_set_tzurl, file);
	 newXS("Net::ICal::icalproperty_get_tzurl", _wrap_icalproperty_get_tzurl, file);
	 newXS("Net::ICal::icalproperty_new_repeat", _wrap_icalproperty_new_repeat, file);
	 newXS("Net::ICal::icalproperty_set_repeat", _wrap_icalproperty_set_repeat, file);
	 newXS("Net::ICal::icalproperty_get_repeat", _wrap_icalproperty_get_repeat, file);
	 newXS("Net::ICal::icalproperty_new_priority", _wrap_icalproperty_new_priority, file);
	 newXS("Net::ICal::icalproperty_set_priority", _wrap_icalproperty_set_priority, file);
	 newXS("Net::ICal::icalproperty_get_priority", _wrap_icalproperty_get_priority, file);
	 newXS("Net::ICal::icalproperty_new_freebusy", _wrap_icalproperty_new_freebusy, file);
	 newXS("Net::ICal::icalproperty_set_freebusy", _wrap_icalproperty_set_freebusy, file);
	 newXS("Net::ICal::icalproperty_get_freebusy", _wrap_icalproperty_get_freebusy, file);
	 newXS("Net::ICal::icalproperty_new_dtstart", _wrap_icalproperty_new_dtstart, file);
	 newXS("Net::ICal::icalproperty_set_dtstart", _wrap_icalproperty_set_dtstart, file);
	 newXS("Net::ICal::icalproperty_get_dtstart", _wrap_icalproperty_get_dtstart, file);
	 newXS("Net::ICal::icalproperty_new_recurrenceid", _wrap_icalproperty_new_recurrenceid, file);
	 newXS("Net::ICal::icalproperty_set_recurrenceid", _wrap_icalproperty_set_recurrenceid, file);
	 newXS("Net::ICal::icalproperty_get_recurrenceid", _wrap_icalproperty_get_recurrenceid, file);
	 newXS("Net::ICal::icalproperty_new_summary", _wrap_icalproperty_new_summary, file);
	 newXS("Net::ICal::icalproperty_set_summary", _wrap_icalproperty_set_summary, file);
	 newXS("Net::ICal::icalproperty_get_summary", _wrap_icalproperty_get_summary, file);
	 newXS("Net::ICal::icalproperty_new_dtend", _wrap_icalproperty_new_dtend, file);
	 newXS("Net::ICal::icalproperty_set_dtend", _wrap_icalproperty_set_dtend, file);
	 newXS("Net::ICal::icalproperty_get_dtend", _wrap_icalproperty_get_dtend, file);
	 newXS("Net::ICal::icalproperty_new_tzname", _wrap_icalproperty_new_tzname, file);
	 newXS("Net::ICal::icalproperty_set_tzname", _wrap_icalproperty_set_tzname, file);
	 newXS("Net::ICal::icalproperty_get_tzname", _wrap_icalproperty_get_tzname, file);
	 newXS("Net::ICal::icalproperty_new_rdate", _wrap_icalproperty_new_rdate, file);
	 newXS("Net::ICal::icalproperty_set_rdate", _wrap_icalproperty_set_rdate, file);
	 newXS("Net::ICal::icalproperty_get_rdate", _wrap_icalproperty_get_rdate, file);
	 newXS("Net::ICal::icalproperty_new_xlicmimefilename", _wrap_icalproperty_new_xlicmimefilename, file);
	 newXS("Net::ICal::icalproperty_set_xlicmimefilename", _wrap_icalproperty_set_xlicmimefilename, file);
	 newXS("Net::ICal::icalproperty_get_xlicmimefilename", _wrap_icalproperty_get_xlicmimefilename, file);
	 newXS("Net::ICal::icalproperty_new_url", _wrap_icalproperty_new_url, file);
	 newXS("Net::ICal::icalproperty_set_url", _wrap_icalproperty_set_url, file);
	 newXS("Net::ICal::icalproperty_get_url", _wrap_icalproperty_get_url, file);
	 newXS("Net::ICal::icalproperty_new_xlicclustercount", _wrap_icalproperty_new_xlicclustercount, file);
	 newXS("Net::ICal::icalproperty_set_xlicclustercount", _wrap_icalproperty_set_xlicclustercount, file);
	 newXS("Net::ICal::icalproperty_get_xlicclustercount", _wrap_icalproperty_get_xlicclustercount, file);
	 newXS("Net::ICal::icalproperty_new_attach", _wrap_icalproperty_new_attach, file);
	 newXS("Net::ICal::icalproperty_set_attach", _wrap_icalproperty_set_attach, file);
	 newXS("Net::ICal::icalproperty_get_attach", _wrap_icalproperty_get_attach, file);
	 newXS("Net::ICal::icalproperty_new_query", _wrap_icalproperty_new_query, file);
	 newXS("Net::ICal::icalproperty_set_query", _wrap_icalproperty_set_query, file);
	 newXS("Net::ICal::icalproperty_get_query", _wrap_icalproperty_get_query, file);
	 newXS("Net::ICal::icalproperty_new_percentcomplete", _wrap_icalproperty_new_percentcomplete, file);
	 newXS("Net::ICal::icalproperty_set_percentcomplete", _wrap_icalproperty_set_percentcomplete, file);
	 newXS("Net::ICal::icalproperty_get_percentcomplete", _wrap_icalproperty_get_percentcomplete, file);
	 newXS("Net::ICal::icalproperty_new_calscale", _wrap_icalproperty_new_calscale, file);
	 newXS("Net::ICal::icalproperty_set_calscale", _wrap_icalproperty_set_calscale, file);
	 newXS("Net::ICal::icalproperty_get_calscale", _wrap_icalproperty_get_calscale, file);
	 newXS("Net::ICal::icalproperty_new_created", _wrap_icalproperty_new_created, file);
	 newXS("Net::ICal::icalproperty_set_created", _wrap_icalproperty_set_created, file);
	 newXS("Net::ICal::icalproperty_get_created", _wrap_icalproperty_get_created, file);
	 newXS("Net::ICal::icalproperty_new_geo", _wrap_icalproperty_new_geo, file);
	 newXS("Net::ICal::icalproperty_set_geo", _wrap_icalproperty_set_geo, file);
	 newXS("Net::ICal::icalproperty_get_geo", _wrap_icalproperty_get_geo, file);
	 newXS("Net::ICal::icalproperty_new_xlicmimecharset", _wrap_icalproperty_new_xlicmimecharset, file);
	 newXS("Net::ICal::icalproperty_set_xlicmimecharset", _wrap_icalproperty_set_xlicmimecharset, file);
	 newXS("Net::ICal::icalproperty_get_xlicmimecharset", _wrap_icalproperty_get_xlicmimecharset, file);
	 newXS("Net::ICal::icalproperty_new_completed", _wrap_icalproperty_new_completed, file);
	 newXS("Net::ICal::icalproperty_set_completed", _wrap_icalproperty_set_completed, file);
	 newXS("Net::ICal::icalproperty_get_completed", _wrap_icalproperty_get_completed, file);
	 newXS("Net::ICal::icalproperty_new_dtstamp", _wrap_icalproperty_new_dtstamp, file);
	 newXS("Net::ICal::icalproperty_set_dtstamp", _wrap_icalproperty_set_dtstamp, file);
	 newXS("Net::ICal::icalproperty_get_dtstamp", _wrap_icalproperty_get_dtstamp, file);
	 newXS("Net::ICal::icalproperty_new_due", _wrap_icalproperty_new_due, file);
	 newXS("Net::ICal::icalproperty_set_due", _wrap_icalproperty_set_due, file);
	 newXS("Net::ICal::icalproperty_get_due", _wrap_icalproperty_get_due, file);
	 newXS("Net::ICal::icalproperty_new_action", _wrap_icalproperty_new_action, file);
	 newXS("Net::ICal::icalproperty_set_action", _wrap_icalproperty_set_action, file);
	 newXS("Net::ICal::icalproperty_get_action", _wrap_icalproperty_get_action, file);
	 newXS("Net::ICal::pvl_new_element", _wrap_pvl_new_element, file);
	 newXS("Net::ICal::pvl_newlist", _wrap_pvl_newlist, file);
	 newXS("Net::ICal::pvl_free", _wrap_pvl_free, file);
	 newXS("Net::ICal::pvl_unshift", _wrap_pvl_unshift, file);
	 newXS("Net::ICal::pvl_shift", _wrap_pvl_shift, file);
	 newXS("Net::ICal::pvl_head", _wrap_pvl_head, file);
	 newXS("Net::ICal::pvl_push", _wrap_pvl_push, file);
	 newXS("Net::ICal::pvl_pop", _wrap_pvl_pop, file);
	 newXS("Net::ICal::pvl_tail", _wrap_pvl_tail, file);
	 newXS("Net::ICal::pvl_insert_ordered", _wrap_pvl_insert_ordered, file);
	 newXS("Net::ICal::pvl_insert_after", _wrap_pvl_insert_after, file);
	 newXS("Net::ICal::pvl_insert_before", _wrap_pvl_insert_before, file);
	 newXS("Net::ICal::pvl_remove", _wrap_pvl_remove, file);
	 newXS("Net::ICal::pvl_clear", _wrap_pvl_clear, file);
	 newXS("Net::ICal::pvl_count", _wrap_pvl_count, file);
	 newXS("Net::ICal::pvl_next", _wrap_pvl_next, file);
	 newXS("Net::ICal::pvl_prior", _wrap_pvl_prior, file);
	 newXS("Net::ICal::pvl_data", _wrap_pvl_data, file);
	 newXS("Net::ICal::pvl_find", _wrap_pvl_find, file);
	 newXS("Net::ICal::pvl_find_next", _wrap_pvl_find_next, file);
	 newXS("Net::ICal::pvl_apply", _wrap_pvl_apply, file);
	 newXS("Net::ICal::icalcomponent_new", _wrap_icalcomponent_new, file);
	 newXS("Net::ICal::icalcomponent_new_clone", _wrap_icalcomponent_new_clone, file);
	 newXS("Net::ICal::icalcomponent_new_from_string", _wrap_icalcomponent_new_from_string, file);
	 newXS("Net::ICal::icalcomponent_free", _wrap_icalcomponent_free, file);
	 newXS("Net::ICal::icalcomponent_as_ical_string", _wrap_icalcomponent_as_ical_string, file);
	 newXS("Net::ICal::icalcomponent_is_valid", _wrap_icalcomponent_is_valid, file);
	 newXS("Net::ICal::icalcomponent_isa", _wrap_icalcomponent_isa, file);
	 newXS("Net::ICal::icalcomponent_isa_component", _wrap_icalcomponent_isa_component, file);
	 newXS("Net::ICal::icalcomponent_add_property", _wrap_icalcomponent_add_property, file);
	 newXS("Net::ICal::icalcomponent_remove_property", _wrap_icalcomponent_remove_property, file);
	 newXS("Net::ICal::icalcomponent_count_properties", _wrap_icalcomponent_count_properties, file);
	 newXS("Net::ICal::icalcomponent_get_current_property", _wrap_icalcomponent_get_current_property, file);
	 newXS("Net::ICal::icalcomponent_get_first_property", _wrap_icalcomponent_get_first_property, file);
	 newXS("Net::ICal::icalcomponent_get_next_property", _wrap_icalcomponent_get_next_property, file);
	 newXS("Net::ICal::icalcomponent_get_properties", _wrap_icalcomponent_get_properties, file);
	 newXS("Net::ICal::icalcomponent_add_component", _wrap_icalcomponent_add_component, file);
	 newXS("Net::ICal::icalcomponent_remove_component", _wrap_icalcomponent_remove_component, file);
	 newXS("Net::ICal::icalcomponent_count_components", _wrap_icalcomponent_count_components, file);
	 newXS("Net::ICal::icalcomponent_end_component", _wrap_icalcomponent_end_component, file);
	 newXS("Net::ICal::icalcomponent_get_current_component", _wrap_icalcomponent_get_current_component, file);
	 newXS("Net::ICal::icalcomponent_get_first_component", _wrap_icalcomponent_get_first_component, file);
	 newXS("Net::ICal::icalcomponent_get_next_component", _wrap_icalcomponent_get_next_component, file);
	 newXS("Net::ICal::icalcomponent_count_errors", _wrap_icalcomponent_count_errors, file);
	 newXS("Net::ICal::icalcomponent_strip_errors", _wrap_icalcomponent_strip_errors, file);
	 newXS("Net::ICal::icalcomponent_convert_errors", _wrap_icalcomponent_convert_errors, file);
	 newXS("Net::ICal::icalcomponent_get_parent", _wrap_icalcomponent_get_parent, file);
	 newXS("Net::ICal::icalcomponent_set_parent", _wrap_icalcomponent_set_parent, file);
	 newXS("Net::ICal::icalcompiter_next", _wrap_icalcompiter_next, file);
	 newXS("Net::ICal::icalcompiter_prior", _wrap_icalcompiter_prior, file);
	 newXS("Net::ICal::icalcompiter_deref", _wrap_icalcompiter_deref, file);
	 newXS("Net::ICal::icalcomponent_get_first_real_component", _wrap_icalcomponent_get_first_real_component, file);
	 newXS("Net::ICal::icalcomponent_get_span", _wrap_icalcomponent_get_span, file);
	 newXS("Net::ICal::icalcomponent_set_dtstart", _wrap_icalcomponent_set_dtstart, file);
	 newXS("Net::ICal::icalcomponent_get_dtstart", _wrap_icalcomponent_get_dtstart, file);
	 newXS("Net::ICal::icalcomponent_get_dtend", _wrap_icalcomponent_get_dtend, file);
	 newXS("Net::ICal::icalcomponent_set_dtend", _wrap_icalcomponent_set_dtend, file);
	 newXS("Net::ICal::icalcomponent_set_duration", _wrap_icalcomponent_set_duration, file);
	 newXS("Net::ICal::icalcomponent_get_duration", _wrap_icalcomponent_get_duration, file);
	 newXS("Net::ICal::icalcomponent_set_method", _wrap_icalcomponent_set_method, file);
	 newXS("Net::ICal::icalcomponent_get_method", _wrap_icalcomponent_get_method, file);
	 newXS("Net::ICal::icalcomponent_get_dtstamp", _wrap_icalcomponent_get_dtstamp, file);
	 newXS("Net::ICal::icalcomponent_set_dtstamp", _wrap_icalcomponent_set_dtstamp, file);
	 newXS("Net::ICal::icalcomponent_set_summary", _wrap_icalcomponent_set_summary, file);
	 newXS("Net::ICal::icalcomponent_get_summary", _wrap_icalcomponent_get_summary, file);
	 newXS("Net::ICal::icalcomponent_set_comment", _wrap_icalcomponent_set_comment, file);
	 newXS("Net::ICal::icalcomponent_get_comment", _wrap_icalcomponent_get_comment, file);
	 newXS("Net::ICal::icalcomponent_set_organizer", _wrap_icalcomponent_set_organizer, file);
	 newXS("Net::ICal::icalcomponent_get_organizer", _wrap_icalcomponent_get_organizer, file);
	 newXS("Net::ICal::icalcomponent_set_uid", _wrap_icalcomponent_set_uid, file);
	 newXS("Net::ICal::icalcomponent_get_uid", _wrap_icalcomponent_get_uid, file);
	 newXS("Net::ICal::icalcomponent_set_recurrenceid", _wrap_icalcomponent_set_recurrenceid, file);
	 newXS("Net::ICal::icalcomponent_get_recurrenceid", _wrap_icalcomponent_get_recurrenceid, file);
	 newXS("Net::ICal::icalcomponent_new_vcalendar", _wrap_icalcomponent_new_vcalendar, file);
	 newXS("Net::ICal::icalcomponent_new_vevent", _wrap_icalcomponent_new_vevent, file);
	 newXS("Net::ICal::icalcomponent_new_vtodo", _wrap_icalcomponent_new_vtodo, file);
	 newXS("Net::ICal::icalcomponent_new_vjournal", _wrap_icalcomponent_new_vjournal, file);
	 newXS("Net::ICal::icalcomponent_new_vfreebusy", _wrap_icalcomponent_new_vfreebusy, file);
	 newXS("Net::ICal::icalcomponent_new_vtimezone", _wrap_icalcomponent_new_vtimezone, file);
	 newXS("Net::ICal::icalcomponent_new_xstandard", _wrap_icalcomponent_new_xstandard, file);
	 newXS("Net::ICal::icalcomponent_new_xdaylight", _wrap_icalcomponent_new_xdaylight, file);
	 newXS("Net::ICal::icalparser_new", _wrap_icalparser_new, file);
	 newXS("Net::ICal::icalparser_add_line", _wrap_icalparser_add_line, file);
	 newXS("Net::ICal::icalparser_claim", _wrap_icalparser_claim, file);
	 newXS("Net::ICal::icalparser_clean", _wrap_icalparser_clean, file);
	 newXS("Net::ICal::icalparser_get_state", _wrap_icalparser_get_state, file);
	 newXS("Net::ICal::icalparser_free", _wrap_icalparser_free, file);
	 newXS("Net::ICal::icalparser_parse_string", _wrap_icalparser_parse_string, file);
	 newXS("Net::ICal::icalparser_parse_value", _wrap_icalparser_parse_value, file);
	 newXS("Net::ICal::string_line_generator", _wrap_string_line_generator, file);
	 newXS("Net::ICal::icalmemory_tmp_buffer", _wrap_icalmemory_tmp_buffer, file);
	 newXS("Net::ICal::icalmemory_tmp_copy", _wrap_icalmemory_tmp_copy, file);
	 newXS("Net::ICal::icalmemory_add_tmp_buffer", _wrap_icalmemory_add_tmp_buffer, file);
	 newXS("Net::ICal::icalmemory_free_ring", _wrap_icalmemory_free_ring, file);
	 newXS("Net::ICal::icalmemory_new_buffer", _wrap_icalmemory_new_buffer, file);
	 newXS("Net::ICal::icalmemory_resize_buffer", _wrap_icalmemory_resize_buffer, file);
	 newXS("Net::ICal::icalmemory_free_buffer", _wrap_icalmemory_free_buffer, file);
	 newXS("Net::ICal::icalmemory_append_string", _wrap_icalmemory_append_string, file);
	 newXS("Net::ICal::icalmemory_append_char", _wrap_icalmemory_append_char, file);
	 newXS("Net::ICal::icalmemory_strdup", _wrap_icalmemory_strdup, file);
	 newXS("Net::ICal::icalerror_stop_here", _wrap_icalerror_stop_here, file);
	 newXS("Net::ICal::icalerror_crash_here", _wrap_icalerror_crash_here, file);
	 newXS("Net::ICal::icalerror_clear_errno", _wrap_icalerror_clear_errno, file);
	 newXS("Net::ICal::icalerror_set_errno", _wrap_icalerror_set_errno, file);
	 newXS("Net::ICal::icalerror_strerror", _wrap_icalerror_strerror, file);
	 newXS("Net::ICal::icalrestriction_compare", _wrap_icalrestriction_compare, file);
	 newXS("Net::ICal::icalrestriction_is_parameter_allowed", _wrap_icalrestriction_is_parameter_allowed, file);
	 newXS("Net::ICal::icalrestriction_check", _wrap_icalrestriction_check, file);
	 newXS("Net::ICal::icaltime_span_start_set", _wrap_icaltime_span_start_set, file);
	 newXS("Net::ICal::icaltime_span_start_get", _wrap_icaltime_span_start_get, file);
	 newXS("Net::ICal::icaltime_span_end_set", _wrap_icaltime_span_end_set, file);
	 newXS("Net::ICal::icaltime_span_end_get", _wrap_icaltime_span_end_get, file);
	 newXS("Net::ICal::icaltime_span_is_busy_set", _wrap_icaltime_span_is_busy_set, file);
	 newXS("Net::ICal::icaltime_span_is_busy_get", _wrap_icaltime_span_is_busy_get, file);
	 newXS("Net::ICal::icaltimetype_year_set", _wrap_icaltimetype_year_set, file);
	 newXS("Net::ICal::icaltimetype_year_get", _wrap_icaltimetype_year_get, file);
	 newXS("Net::ICal::icaltimetype_month_set", _wrap_icaltimetype_month_set, file);
	 newXS("Net::ICal::icaltimetype_month_get", _wrap_icaltimetype_month_get, file);
	 newXS("Net::ICal::icaltimetype_day_set", _wrap_icaltimetype_day_set, file);
	 newXS("Net::ICal::icaltimetype_day_get", _wrap_icaltimetype_day_get, file);
	 newXS("Net::ICal::icaltimetype_hour_set", _wrap_icaltimetype_hour_set, file);
	 newXS("Net::ICal::icaltimetype_hour_get", _wrap_icaltimetype_hour_get, file);
	 newXS("Net::ICal::icaltimetype_minute_set", _wrap_icaltimetype_minute_set, file);
	 newXS("Net::ICal::icaltimetype_minute_get", _wrap_icaltimetype_minute_get, file);
	 newXS("Net::ICal::icaltimetype_second_set", _wrap_icaltimetype_second_set, file);
	 newXS("Net::ICal::icaltimetype_second_get", _wrap_icaltimetype_second_get, file);
	 newXS("Net::ICal::icaltimetype_is_utc_set", _wrap_icaltimetype_is_utc_set, file);
	 newXS("Net::ICal::icaltimetype_is_utc_get", _wrap_icaltimetype_is_utc_get, file);
	 newXS("Net::ICal::icaltimetype_is_date_set", _wrap_icaltimetype_is_date_set, file);
	 newXS("Net::ICal::icaltimetype_is_date_get", _wrap_icaltimetype_is_date_get, file);
	 newXS("Net::ICal::icaldurationtype_is_neg_set", _wrap_icaldurationtype_is_neg_set, file);
	 newXS("Net::ICal::icaldurationtype_is_neg_get", _wrap_icaldurationtype_is_neg_get, file);
	 newXS("Net::ICal::icaldurationtype_days_set", _wrap_icaldurationtype_days_set, file);
	 newXS("Net::ICal::icaldurationtype_days_get", _wrap_icaldurationtype_days_get, file);
	 newXS("Net::ICal::icaldurationtype_weeks_set", _wrap_icaldurationtype_weeks_set, file);
	 newXS("Net::ICal::icaldurationtype_weeks_get", _wrap_icaldurationtype_weeks_get, file);
	 newXS("Net::ICal::icaldurationtype_hours_set", _wrap_icaldurationtype_hours_set, file);
	 newXS("Net::ICal::icaldurationtype_hours_get", _wrap_icaldurationtype_hours_get, file);
	 newXS("Net::ICal::icaldurationtype_minutes_set", _wrap_icaldurationtype_minutes_set, file);
	 newXS("Net::ICal::icaldurationtype_minutes_get", _wrap_icaldurationtype_minutes_get, file);
	 newXS("Net::ICal::icaldurationtype_seconds_set", _wrap_icaldurationtype_seconds_set, file);
	 newXS("Net::ICal::icaldurationtype_seconds_get", _wrap_icaldurationtype_seconds_get, file);
	 newXS("Net::ICal::icalperiodtype_start_set", _wrap_icalperiodtype_start_set, file);
	 newXS("Net::ICal::icalperiodtype_start_get", _wrap_icalperiodtype_start_get, file);
	 newXS("Net::ICal::icalperiodtype_end_set", _wrap_icalperiodtype_end_set, file);
	 newXS("Net::ICal::icalperiodtype_end_get", _wrap_icalperiodtype_end_get, file);
	 newXS("Net::ICal::icalperiodtype_duration_set", _wrap_icalperiodtype_duration_set, file);
	 newXS("Net::ICal::icalperiodtype_duration_get", _wrap_icalperiodtype_duration_get, file);
	 newXS("Net::ICal::icalattachtype_binary_set", _wrap_icalattachtype_binary_set, file);
	 newXS("Net::ICal::icalattachtype_binary_get", _wrap_icalattachtype_binary_get, file);
	 newXS("Net::ICal::icalattachtype_owns_binary_set", _wrap_icalattachtype_owns_binary_set, file);
	 newXS("Net::ICal::icalattachtype_owns_binary_get", _wrap_icalattachtype_owns_binary_get, file);
	 newXS("Net::ICal::icalattachtype_base64_set", _wrap_icalattachtype_base64_set, file);
	 newXS("Net::ICal::icalattachtype_base64_get", _wrap_icalattachtype_base64_get, file);
	 newXS("Net::ICal::icalattachtype_owns_base64_set", _wrap_icalattachtype_owns_base64_set, file);
	 newXS("Net::ICal::icalattachtype_owns_base64_get", _wrap_icalattachtype_owns_base64_get, file);
	 newXS("Net::ICal::icalattachtype_url_set", _wrap_icalattachtype_url_set, file);
	 newXS("Net::ICal::icalattachtype_url_get", _wrap_icalattachtype_url_get, file);
	 newXS("Net::ICal::icalattachtype_refcount_set", _wrap_icalattachtype_refcount_set, file);
	 newXS("Net::ICal::icalattachtype_refcount_get", _wrap_icalattachtype_refcount_get, file);
	 newXS("Net::ICal::icalgeotype_lat_set", _wrap_icalgeotype_lat_set, file);
	 newXS("Net::ICal::icalgeotype_lat_get", _wrap_icalgeotype_lat_get, file);
	 newXS("Net::ICal::icalgeotype_lon_set", _wrap_icalgeotype_lon_set, file);
	 newXS("Net::ICal::icalgeotype_lon_get", _wrap_icalgeotype_lon_get, file);
	 newXS("Net::ICal::icaltriggertype_time_set", _wrap_icaltriggertype_time_set, file);
	 newXS("Net::ICal::icaltriggertype_time_get", _wrap_icaltriggertype_time_get, file);
	 newXS("Net::ICal::icaltriggertype_duration_set", _wrap_icaltriggertype_duration_set, file);
	 newXS("Net::ICal::icaltriggertype_duration_get", _wrap_icaltriggertype_duration_get, file);
	 newXS("Net::ICal::icalreqstattype_code_set", _wrap_icalreqstattype_code_set, file);
	 newXS("Net::ICal::icalreqstattype_code_get", _wrap_icalreqstattype_code_get, file);
	 newXS("Net::ICal::pvl_elem_t_MAGIC_set", _wrap_pvl_elem_t_MAGIC_set, file);
	 newXS("Net::ICal::pvl_elem_t_MAGIC_get", _wrap_pvl_elem_t_MAGIC_get, file);
	 newXS("Net::ICal::pvl_elem_t_d_set", _wrap_pvl_elem_t_d_set, file);
	 newXS("Net::ICal::pvl_elem_t_d_get", _wrap_pvl_elem_t_d_get, file);
	 newXS("Net::ICal::pvl_elem_t_next_set", _wrap_pvl_elem_t_next_set, file);
	 newXS("Net::ICal::pvl_elem_t_next_get", _wrap_pvl_elem_t_next_get, file);
	 newXS("Net::ICal::pvl_elem_t_prior_set", _wrap_pvl_elem_t_prior_set, file);
	 newXS("Net::ICal::pvl_elem_t_prior_get", _wrap_pvl_elem_t_prior_get, file);
	 newXS("Net::ICal::icalcompiter_kind_set", _wrap_icalcompiter_kind_set, file);
	 newXS("Net::ICal::icalcompiter_kind_get", _wrap_icalcompiter_kind_get, file);
	 newXS("Net::ICal::icalcompiter_iter_set", _wrap_icalcompiter_iter_set, file);
	 newXS("Net::ICal::icalcompiter_iter_get", _wrap_icalcompiter_iter_get, file);
/*
 * These are the pointer type-equivalency mappings. 
 * (Used by the SWIG pointer type-checker).
 */
	 SWIG_RegisterMapping("icalattachtype","struct icalattachtype",0);
	 SWIG_RegisterMapping("unsigned short","short",0);
	 SWIG_RegisterMapping("struct icaldurationtype","icaldurationtype",0);
	 SWIG_RegisterMapping("icalperiodtype","struct icalperiodtype",0);
	 SWIG_RegisterMapping("struct icalattachtype","icalattachtype",0);
	 SWIG_RegisterMapping("union icaltriggertype","icaltriggertype",0);
	 SWIG_RegisterMapping("icaltriggertype","union icaltriggertype",0);
	 SWIG_RegisterMapping("long","unsigned long",0);
	 SWIG_RegisterMapping("long","signed long",0);
	 SWIG_RegisterMapping("struct icaltimetype","icaltimetype",0);
	 SWIG_RegisterMapping("struct icalcompiter","icalcompiter",0);
	 SWIG_RegisterMapping("icaldurationtype","struct icaldurationtype",0);
	 SWIG_RegisterMapping("signed short","short",0);
	 SWIG_RegisterMapping("struct icalperiodtype","icalperiodtype",0);
	 SWIG_RegisterMapping("struct icaltime_span","icaltime_span",0);
	 SWIG_RegisterMapping("icalreqstattype","struct icalreqstattype",0);
	 SWIG_RegisterMapping("pvl_elem_t","struct pvl_elem_t",0);
	 SWIG_RegisterMapping("icaltime_span","struct icaltime_span",0);
	 SWIG_RegisterMapping("signed int","int",0);
	 SWIG_RegisterMapping("short","unsigned short",0);
	 SWIG_RegisterMapping("short","signed short",0);
	 SWIG_RegisterMapping("unsigned long","long",0);
	 SWIG_RegisterMapping("struct icalreqstattype","icalreqstattype",0);
	 SWIG_RegisterMapping("int","unsigned int",0);
	 SWIG_RegisterMapping("int","signed int",0);
	 SWIG_RegisterMapping("unsigned int","int",0);
	 SWIG_RegisterMapping("icaltimetype","struct icaltimetype",0);
	 SWIG_RegisterMapping("icalcompiter","struct icalcompiter",0);
	 SWIG_RegisterMapping("struct icalgeotype","icalgeotype",0);
	 SWIG_RegisterMapping("struct pvl_elem_t","pvl_elem_t",0);
	 SWIG_RegisterMapping("icalgeotype","struct icalgeotype",0);
	 SWIG_RegisterMapping("signed long","long",0);
	 ST(0) = &sv_yes;
	 XSRETURN(1);
}
